/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.authentication;

import java.util.Vector;
import sun.server.realm.*;
import webdav.common.SubsystemInterface;

/** 
 *
 * @author		Marc Eaddy
 * @version		1.0
 * @date        2 May 1998
 */

public class AuthenticationManager
    implements AuthenticationInterface, SubsystemInterface
{
	/**
	 * 
	 *
	 * @param strAuthorizationHeader    [IN] The "Authorization" header
     *                                  from the Http request.
     * @param strRealm                  [IN] The realm.
     * @param strMethod                 [IN] The HTTP/WebDAV request
     *                                  method.
     * @param strRequestUri             [IN] The resource URI from the
     *                                  HTTP request.
	 * @param vecWWWAuthenticateHeaders [OUT] On AUTH_FAILURE, will
     *                                  contain 1 or more
     *                                  WWWAuthenticateHeaders which
     *                                  should be sent to the client
     *                                  in the response.
	 *
	 * @return		One of the following status codes:<P>
	 *
     * <UL>
	 *		<LI>AUTH_SUCCESS            The client was successfully
     *                                  authenticated.
	 *		<LI>AUTH_INTERNAL_ERROR     There was an internal error.
	 *		<LI>AUTH_FAILURE            The client either submitted
     *                                  invalid authorization or no
     *                                  authorization at all.  The
     *                                  vecWWWAuthenticateHeaders
     *                                  will be filled with 1 or more
     *                                  WWWAuthenticateHeader objects
     *                                  which should be sent to the
     *                                  client in the response.
     *      <LI>AUTH_BAD_REQUEST        The authorization header was
     *                                  invalid.
	 * </UL>
	 */
	public int authenticate(String strAuthorizationHeader,
                            String strRemoteIPAddress,
                            String strRealm,
                            String strMethod,
                            String strRequestUri,
                            Vector vecWWWAuthenticateHeaders)
    {
        vecWWWAuthenticateHeaders.removeAllElements();

		if (strAuthorizationHeader == null)
		{
            System.out.println("No Authorization header found");

		    WWWAuthenticateHeader hdrWWWAuthenticate =
				new WWWAuthenticateHeader(	strRealm,
											strRemoteIPAddress);

            vecWWWAuthenticateHeaders.addElement(hdrWWWAuthenticate);

            return AUTH_FAILURE;
        }

		// Get the Authorization info from the client
		AuthorizationHeader hdrAuthorization =
				new AuthorizationHeader(strAuthorizationHeader);

        System.out.println("Auth Header = " + hdrAuthorization);

	    // Make sure that the client is using the appropriate
		// Authorization type
		if (!hdrAuthorization.getType().equalsIgnoreCase("Digest") ||
			!hdrAuthorization.getAlgorithm().equalsIgnoreCase("MD5"))
		{
            System.out.println("Authorization header is wrong type");
		    WWWAuthenticateHeader hdrWWWAuthenticate =
		        new WWWAuthenticateHeader(	strRealm,
										    strRemoteIPAddress);

            vecWWWAuthenticateHeaders.addElement(hdrWWWAuthenticate);

            return AUTH_FAILURE;
		}

		// Now verify the authenticaion info to make sure that
        // the client is properly authenticated

        String strUser = hdrAuthorization.getParam("username");
        String strPassword = null;

        Realm realm = null;
        User user = null;

        try
        {
            realm = Realm.get("defaultRealm");
            user = realm.getUser(strUser);

            if (user == null)
            {
                throw new NoSuchUserException(strUser);
            }

		    // Lookup the password for this username
		    strPassword = (String) user.getAuthInfo();

            if (strPassword == null)
            {
                throw new NoSuchUserException(strUser);
            }
        }
        catch (NoSuchRealmException ex)
        {
            ex.printStackTrace();
            return AUTH_INTERNAL_ERROR;
        }
        catch (BadRealmException ex)
        {
            ex.printStackTrace();
            return AUTH_INTERNAL_ERROR;
        }
        catch (NoSuchUserException ex)
        {
            System.out.println("Authorization failed: '" + 
                                strUser + 
                                "' - Unknown user");

		    WWWAuthenticateHeader hdrWWWAuthenticate =
		        new WWWAuthenticateHeader(	strRealm,
										    strRemoteIPAddress);

            vecWWWAuthenticateHeaders.addElement(hdrWWWAuthenticate);
            return AUTH_FAILURE;
        }

		// Get the nonce from the client's Authorization header.
		// This is not safe, we should use the nonce taken from
		// the WWW-Authenticate header instead.
		String strNonce = new String(hdrAuthorization.getParam("nonce"));

		boolean bIsAuthenticated = 
			AuthorizationHeader.verifyPassword(	hdrAuthorization,
												strPassword,
												strRealm,
												strNonce,
												strMethod,
												strRequestUri);
		if (!bIsAuthenticated)
		{
            System.out.println("Failed to authenticate user");

            WWWAuthenticateHeader hdrWWWAuthenticate =
		        new WWWAuthenticateHeader(	strRealm,
										    strRemoteIPAddress);

            vecWWWAuthenticateHeaders.addElement(hdrWWWAuthenticate);

            return AUTH_FAILURE;
		}
        else
        {
            return AUTH_SUCCESS;
        }
    }
}

