/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.authentication;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.Hashtable;
import webdav.common.HeaderWithParameters;
import webdav.common.DigestHelper;

/**
 * Represents the "Authorization" header to support the
 * challenge-response paradigm of Digest Access authentication
 * defined by "RFC 2069 - Digest Access Authentication".  This
 * class can be used by the client to construct the header and
 * by the server to parse the header.
 *
 * <P>The WebDAV specification requires that all WebDAV-compliant
 * servers MUST support Digest authentication (Sec. 16.1,
 * "Authentication of Clients", WebDAV Protocol Specification,
 * Version 8).
 *
 * <P>The Authorization header has the following BNF format:
 *
 * <PRE>
 *
 * Authorization       = "Authorization" ":" "Digest" digest-response
 *
 * digest-response     = 1#( username | realm | nonce | digest-uri |
 *                       response | [ digest ] | [ algorithm ] |
 *                       opaque )
 *
 * username            = "username" "=" username-value
 * username-value      = quoted-string
 * digest-uri          = "uri" "=" digest-uri-value
 * digest-uri-value    = request-uri         ; As specified by HTTP/1.1
 * response            = "response" "=" response-digest
 * digest              = "digest" "=" entity-digest
 *
 * response-digest     = <"> *LHEX <">
 * entity-digest       = <"> *LHEX <">
 * LHEX                = "0" | "1" | "2" | "3" | "4" | "5" | "6" | "7" |
 *                       "8" | "9" | "a" | "b" | "c" | "d" | "e" | "f"
 * </PRE>
 *
 * <P> This class does not make use of the "entity-digest"
 * property.
 *
 * <P> When a client makes a request that requires
 * authorization, the server replies with "401 Forbidden" and
 * includes a WWW-Authenticate header.  (This is the "challenge".)
 * The client must resubmit its request and include an
 * appropriate Authorization header. (This is the "response".)
 * After the server validates the Authorization header, the
 * server satisfies the request.
 *
 * @see         RFC2069_Digest_Access_Authentication
 * @see         RFC1321_The_MD5_Message-Digest_Algorithm
 *
 * @author		Marc Eaddy
 * @version		1.0, 20 Apr 1998
 */

public class AuthorizationHeader
	extends HeaderWithParameters
{
	// DATA

	private static final String s_strDefaultAlgorithm	= "MD5"; // RFC 1321
	private static final String s_strPrivateKey			= "AuthorizationSecret";
	private static final String s_strHeaderName			= "Authorization"; 
	
	private String strAuthorizationType	= null;
	private String strAlgorithm			= null;

	/**
	 * Creates an AuthorizationHeader
	 *
	 * <P> This is the CTOR that the client would use to
	 * create the header.
	 *
	 */
	public AuthorizationHeader(	WWWAuthenticateHeader hdrWWWAuth,
									String strUsername,
									String strPassword,
									String strMethod,
									String strDigestUri)
	{
		super(s_strHeaderName, hdrWWWAuth);

		strAuthorizationType	= new String("Digest");
		strAlgorithm			= s_strDefaultAlgorithm;

		setParam("username", strUsername);
		setParam("uri", strDigestUri);

		String strResponse = makeResponse(	strAlgorithm,
											strUsername, 
											strPassword, 
											getParam("realm"),
											getParam("nonce"), 
											strMethod, 
											strDigestUri);
		
		setParam("response", strResponse);
	}

	/**
	 * Parses the "Authorization" HTTP header.  It is
	 * assumed that the header does not contain the header
	 * name.  This is the normal behaviour when retrieving
	 * a header from HttpRequest or HttpResponse using
	 * getHeader().
	 *
	 * <P> This is the CTOR used by the server to parse a
	 * Authorization header received from a client.
	 *
	 * @see HttpRequest
	 * @see HttpResponse
	 */
	public AuthorizationHeader(String strAuthorizationHeader)
	{
		super(s_strHeaderName, strAuthorizationHeader);

		StringTokenizer strtok = 
			new StringTokenizer(strAuthorizationHeader);

		if (!strtok.hasMoreElements())
		{
			return;
		}

		strAuthorizationType = new String(strtok.nextToken(" \t"));

        String strAlgParam = getParam("algorithm");

        if (strAlgParam != null)
        {
		    strAlgorithm = new String(strAlgParam);
        }
        else
        {
		    strAlgorithm = s_strDefaultAlgorithm;
        }
	}

	/**
	 * @return	The authentication type of this header (e.g.,
	 *			"BASIC" or "DIGEST").
	 */
	public String getType()
	{
		return strAuthorizationType;
	}

	/**
	 * @return	The algorithm name used by this header (e.g.,
	 *			"MD5").
	 */
	public String getAlgorithm()
	{
		return strAlgorithm;
	}

	/**
	 * Converts the AuthorizationHeader into a string suitable
	 * for transmission as an HTTP header.
	 */
	public String toString()
	{
		return new String(strAuthorizationType + " " + paramsToString());
	}

	/**
	 * A helper function to create the "response" parameter as
	 * specified by the algorithm in RFC 2069.  Here is a brief
	 * description of the algorithm:
	 *
	 * <P>First, we obtain the shared secret.  Both the client and
	 * the server must be able to determine the shared secret.
	 * The secret is composed of the username, realm and password.<BR>
	 *
	 * <var>strA1</var>	= <var>strUsername</var> + ":" + 
     * <var>strRealm</var> + ":" + <var>strPassword</var><BR>
	 * <var>strH_A1</var>	= shared secret = H(A1)
	 *
	 * <P>The client will determine the secret by asking the user
	 * for his username and password.  The client will know the
	 * realm because this is what was returned in the original
	 * WWW-Authenticate header sent by the server.  The client
	 * can concatenate these three parameters and create the
	 * shared secret by hashing the result (<var>strH_A1</var>).
	 *
	 * <P>The server will determine the secret in one of two ways.
	 * The username and realm would be obtained from the
	 * Authorization header received from the client.  The
	 * first method comes from the server looking up the
	 * password for that username and realm.  The server can
	 * concatenate these three parameters and create the shared
	 * secret by hashing the result (<var>strH_A1</var>).  This
     * function requires the server to know the password directly.
	 *
	 * <P>The other way is for the server to look up the completed
	 * shared secret for that username and realm.  This method
	 * is useful because the server would not need to know the
	 * password for the user, just the shared secret.  The
	 * other form of this function requires the server to know
	 * the shared secret only.
	 *
	 * <P>Next, we concatentate the method and the uri to create
	 * <var>strH_A2</var>.  Both the client and the server will
     * know these parameters and be able to supply them.<BR>
	 *
	 * <var>strA2</var>	= <var>strMethod</var> + ":" + 
     * <var>strDigestUri</var><BR>
	 * <var>strH_A2</var>	= H(A2)<BR>
	 * 
	 * <P>Finally, we create the response by concatenating the
	 * shared secret (<var>strH_A1</var>) with <var>strH_A2</var>
     * and hasing the the result.<BR>
	 *
	 * <var>strResponse</var> = H( <var>strH_A1</var> + ":" + 
     * <var>strH_A2</var>);<BR>
	 *
	 * @param	strUsername	[IN] Username of the client user.
	 *						The client would obtain this by
	 *						asking the user directly or by 
	 *						obtaining it from the OS.  The
	 *						server would obtain this from the
	 *						Authorization header received
	 *						from the client.
	 *
	 * @param	strPassword [IN] Password for the client user.
	 *                      The client would obtain this by
	 *						asking the user directly or by
	 *						obtaining it from the OS.  The
	 *						server would obtain this via an
	 *						unspecified authentication method
	 *						(e.g., password file, OS access
	 *						control).
	 *
	 * @param	strRealm	[IN] The realm for which the
	 *						username and password are relevant.
	 *						The client gets this from the
	 *						WWW-Authenticate header sent by
	 *						the server.
	 *						See WWWAuthenticateHeader.
	 *
	 * @param	strNonce	[IN] The client gets this from
	 *						the WWW-Authenticate header sent
	 *						by the server.
	 *						See WWWAuthenticateHeader.
	 *
	 * @param	strMethod	[IN] The method that requires the
	 *						authentication.
	 *
	 * @param	strDigestUri [IN] The request uri that requires
	 *						the authentication.
	 *
	 * @return	The digest response header parameter ("response").
	 */
	private static String makeResponse(	String strAlgorithm,
										String strUsername, 
										String strPassword, 
										String strRealm,
										String strNonce, 
										String strMethod, 
										String strDigestUri)
	{
		MessageDigest messageDigest = null;

		try
		{
			messageDigest = MessageDigest.getInstance(strAlgorithm);
		}
		catch (NoSuchAlgorithmException ex)
		{
			return null;
		}

		String strA1 = strUsername + ":" + strRealm + ":" + strPassword;

		String strHash_A1 = DigestHelper.Hash(messageDigest, strA1);

		String strSecret = strHash_A1;

		return makeResponse(strAlgorithm,
							strSecret,
							strNonce,
							strMethod,
							strDigestUri);
	}

	/**
	 * A helper function to create the "response" parameter as
	 * specified by the algorithm in RFC 2069.  This form of the
	 * makeResponse() function allows the server to specify the
	 * shared secret instead of the username, password and realm.
	 * See the alternate form of makeResponse() for details on
	 * the algorithm and the function parameters.
	 *
	 * @see		#makeResponse(	String strAlgorithm,
	 *						    String strUsername, 
	 *							String strPassword, 
	 *							String strRealm,
	 *							String strNonce, 
	 *							String strMethod, 
	 *							String strDigestUri)
	 *
	 * @param	strSecret	[IN] The shared secret as described
	 *						in the alternate form of makeResponse().
	 *
	 * @return	The digest response header parameter ("response").
	 */
	private static String makeResponse(	String strAlgorithm,
										String strSecret,
										String strNonce, 
										String strMethod, 
										String strDigestUri)
	{
		MessageDigest messageDigest = null;

		try
		{
			messageDigest = MessageDigest.getInstance(strAlgorithm);
		}
		catch (NoSuchAlgorithmException ex)
		{
			return null;
		}

		String strA2 = strMethod + ":" + strDigestUri;

		String strHash_A2 = DigestHelper.Hash(messageDigest, strA2);

		String strData   = strNonce + ":" + strHash_A2;

		return DigestHelper.Hash(	messageDigest,
									strData,
									strSecret);
	}

	/**
	 * This helper function allows the server to verify that the
	 * AuthorizationHeader received from the client actually
	 * authenticates the client to allow the client to perform
	 * the requested method on the request uri.  The parameters
	 * are fully described in the makeResponse() function
	 * description.  This form of verify() requires the username,
	 * password and realm.
	 *
	 * @see		#makeResponse(	String strAlgorithm,
	 *						    String strUsername, 
	 *							String strPassword, 
	 *							String strRealm,
	 *							String strNonce, 
	 *							String strMethod, 
	 *							String strDigestUri)
	 *
	 * @param	hdrAuthorization	[IN] The Authorization header
	 *								received by the server from the
	 *								client.
	 * 
	 * @return	TRUE if the client has been authenticated
	 *			successfully, otherwise FALSE
	 *
	 */
	public static boolean verifyPassword(	AuthorizationHeader hdrAuthorization,
											String strPassword,
											String strRealm,
											String strNonce,
											String strMethod,
											String strDigestUri)
	{
		// Check the authentication header for validity.  If the header
		// was altered by a devious third-party, we can at least prevent
		// some obvious changes.
		if (strPassword == null ||
			hdrAuthorization.getParam("realm") == null ||
            !hdrAuthorization.getParam("realm").equalsIgnoreCase(strRealm) ||
            hdrAuthorization.getParam("uri") == null ||
			!hdrAuthorization.getParam("uri").equalsIgnoreCase(strDigestUri) ||
            hdrAuthorization.getParam("nonce") == null ||
			!hdrAuthorization.getParam("nonce").equalsIgnoreCase(strNonce))
		{
            System.out.println("Authorization header was invalid");
			return false;
		}

		String strResponseExpected = makeResponse(	hdrAuthorization.getAlgorithm(),
													hdrAuthorization.getParam("username"),
													strPassword,
													strRealm,
													hdrAuthorization.getParam("nonce"),
													strMethod,
													strDigestUri);

		String strResponseReceived = hdrAuthorization.getParam("response");

		return strResponseExpected.equals(strResponseReceived);
	}

	/**
	 * This helper function allows the server to verify that the
	 * AuthorizationHeader received from the client actually
	 * authenticates the client to allow the client to perform
	 * the requested method on the request uri.  The parameters
	 * are fully described in the makeResponse() function
	 * description.  This form of verify() requires the server to
	 * supply the shared secret.
	 *
	 * @see		#makeResponse(	String strAlgorithm,
	 *						    String strSecret, 
	 *							String strNonce, 
	 *							String strMethod, 
	 *							String strDigestUri)
	 *
	 * @param	hdrAuthorization	[IN] The Authorization header
	 *								received by the server from the
	 *								client.
	 * 
	 * @return	TRUE if the client has been authenticated
	 *			successfully, otherwise FALSE
	 *
	 */
	public static boolean verifySecret(	AuthorizationHeader hdrAuthorization,
										String strSecret,
										String strMethod,
										String strDigestUri)
	{
		String strResponseExpected = makeResponse(	hdrAuthorization.getAlgorithm(),
													strSecret,
													hdrAuthorization.getParam("nonce"),
													strMethod,
													strDigestUri);

		String strResponseReceived = hdrAuthorization.getParam("response");

		return strResponseExpected.equals(strResponseReceived);
	}

	/**
	 * Test driver for this class.
	 */
	public static final void main(String[] args)
	{
		WWWAuthenticateHeader hdrWWWAuthenticate = 
			new WWWAuthenticateHeader("my_realm", "1.2.3.4");

		System.out.println(hdrWWWAuthenticate);

		AuthorizationHeader hdrAuthorization =
			new AuthorizationHeader(	hdrWWWAuthenticate,
										"marc",
										"shhh",
										"GET",
										"/index.htm");

		System.out.println(hdrAuthorization);

		boolean bAuthenticated = 
			verifyPassword(	hdrAuthorization,
					"shhh",
					"my_realm",
					hdrAuthorization.getParam("nonce"),
					"GET",
					"/index.htm");

		System.out.println("Authenticated? " + bAuthenticated);
	}
};