/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.authentication;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Vector;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.Hashtable;
import webdav.common.HeaderWithParameters;
import webdav.common.DigestHelper;

/**
 * Represents the "WWW-Authenticate" header to support the
 * challenge-response paradigm of Digest Access authentication
 * defined by "RFC 2069 - Digest Access Authentication".  This
 * class can be used by the server to construct the header and
 * by the client to parse the header.
 *
 * <P>The WebDAV specification requires that all WebDAV-compliant
 * servers MUST support Digest authentication (Sec. 16.1,
 * "Authentication of Clients", WebDAV Protocol Specification,
 * Version 8).
 *
 * <P>The WWW-Authenticate header has the following BNF format:
 *
 * <PRE>
 *
 *   WWW-Authenticate    = "WWW-Authenticate" ":" "Digest"
 *                            digest-challenge
 *
 *   digest-challenge    = 1#( realm | [ domain ] | nonce |
 *                        [ digest-opaque ] |[ stale ] | [ algorithm ] )
 *
 *   realm               = "realm" "=" realm-value
 *   realm-value         = quoted-string
 *   domain              = "domain" "=" <"> 1#URI <">
 *   nonce               = "nonce" "=" nonce-value
 *   nonce-value         = quoted-string
 *   opaque              = "opaque" "=" quoted-string
 *   stale               = "stale" "=" ( "true" | "false" )
 *   algorithm           = "algorithm" "=" ( "MD5" | token )
 *
 * </PRE>
 *
 * <P> This basic implementation of Digest Authentication
 * will not make use of the "opaque", "domain" or "stale"
 * parameters.  This allows our implementation of Digest
 * Authentication to be open to various security attacks
 * but enforces the minimal requirements of the WebDAV
 * protocol specification (which is our primary intent).
 *
 * <P> When a client makes a request that requires
 * authorization, the server replies with "401 Forbidden" and
 * includes a WWW-Authenticate header.  (This is the "challenge".)
 * The client must resubmit its request and include an
 * appropriate Authentication header. (This is the "response".)
 * After the server validates the Authentication header, the
 * server then can satisfy the request.
 *
 * @see         RFC2069_Digest_Access_Authentication
 * @see         RFC1321_The_MD5_Message-Digest_Algorithm
 *
 * @author		Marc Eaddy
 * @version		1.0, 04 Apr 1998
 */

public class WWWAuthenticateHeader
	extends HeaderWithParameters
{
	// DATA

	private static final String s_strDefaultAlgorithm	= "MD5"; // RFC 1321
	private static final String s_strPrivateKey			= "WWWAuthSecret";
	private static final String s_strHeaderName			= "WWW-Authenticate";

	private String strAuthenticationType	= null;
	private String strAlgorithm				= null;

	/**
	 * Creates an WWWAuthenticateHeader with default
	 * nonce (based on strClientIP).
	 *
	 * @param	strRealm	[IN] Indicates to the client which
	 *                      username and password they will need
	 *                      to supply.  Ex: "registered_users@
	 *                      gotham.news.com".
	 *
	 * @param	strClientIP [IN] The client's IP address as a
	 *                      dotted quad (e.g., "206.15.120.35").
	 */
	public WWWAuthenticateHeader(	String strRealm,
									String strClientIP)
	{
		super(s_strHeaderName);

		strAuthenticationType	= new String("Digest");
		strAlgorithm			= s_strDefaultAlgorithm;

		String strNonce = makeNonce(strAlgorithm,
									strClientIP,
									s_strPrivateKey);

        setParam("realm",           new String(strRealm));
        setParam("domain",          null);
        setParam("nonce",           strNonce);
        setParam("digest-opaque",   null);
        setParam("stale",           null);
        setParam("algorithm",       s_strDefaultAlgorithm);
	}
    
	/**
	 * Parses the "WWW-Authenticate" HTTP header.  It is
	 * assumed that the header does not contain the header
	 * name.  This is the normal behaviour when retrieving
	 * a header from HttpRequest or HttpResponse using
	 * getHeader().  The header parameters will be parsed
	 * from the string and assigned to this object.
	 *
	 * @see HttpRequest
	 * @see HttpResponse
	 */
	public WWWAuthenticateHeader(String strWWWAuthenticateHeader)
	{
		super(s_strHeaderName);

        setParam("realm",           null);
        setParam("domain",          null);
        setParam("nonce",           null);
        setParam("digest-opaque",   null);
        setParam("stale",           null);
        setParam("algorithm",       null);

        parseHeaderProperties(strWWWAuthenticateHeader);

		StringTokenizer strtok = 
			new StringTokenizer(strWWWAuthenticateHeader);

		if (!strtok.hasMoreElements())
		{
			return;
		}

		strAuthenticationType = new String(strtok.nextToken(" \t"));
		strAlgorithm = getParam("algorithm");

		if (strAlgorithm == null)
		{
			strAlgorithm = s_strDefaultAlgorithm;
		}
	}

	/**
	 * @return	The authentication type of this header (e.g.,
	 *			"BASIC" or "DIGEST").
	 */
	public String getType()
	{
		return strAuthenticationType;
	}

	/**
	 * @return	The algorithm name used by this header (e.g.,
	 *			"MD5").
	 */
	public String getAlgorithm()
	{
		return strAlgorithm;
	}

	/**
	 * Converts the WWWAuthenticateHeader into a string suitable
	 * for transmission as an HTTP header.
	 */
	public String toString()
	{
		return new String(strAuthenticationType + " " + paramsToString());
	}

	/**
	 * Initializes the nonce string by performing the
	 * following operation:<P>
	 *
	 * MD5( strClientIP + ":" + timestamp + ":" + strPrivateKey )
	 *
	 * @param	strClientIP	[IN] The client's IP address as a
	 *						dotted quad (e.g., "206.15.120.35").
	 */
	private static String makeNonce(String strAlgorithm,
									String strClientIP, 
									String strPrivateKey)
	{
		MessageDigest messageDigest = null;

		try
		{
			messageDigest = MessageDigest.getInstance(strAlgorithm);
		}
		catch (NoSuchAlgorithmException ex)
		{
			return null;
		}

		String strTimeStamp = 
			Long.toString(System.currentTimeMillis());

		String strData = strClientIP + ":" + strTimeStamp + ":" +
			strPrivateKey;

		return DigestHelper.Hash(messageDigest, strData);
	}

	/**
	 * Test driver for this class.
	 */
	public static final void main(String[] args)
	{
		WWWAuthenticateHeader hdrWWWAuthenticate = 
			new WWWAuthenticateHeader("my_realm", "1.2.3.4");

		String strHdr = hdrWWWAuthenticate.toString();

		System.out.println(strHdr);

		WWWAuthenticateHeader hdrNew =
			new WWWAuthenticateHeader(strHdr);

		String strNewHdr = hdrNew.toString();

		System.out.println(strNewHdr);

		if (!hdrNew.equals(hdrWWWAuthenticate))
		{
			System.out.println("Not equal!");
		}
	}
};