/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.common;

import webdav.lockman.*;
import webdav.nsman.*;
import webdav.propman.*;
import webdav.authentication.*;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.NoSuchElementException;

/**
 * The SubsystemRegistry is a global (static) registry of subsystems
 * where subsystems are registered by their type and Uri scope.
 * Clients of the registry can use the lookup() function to retrieve
 * the interfaces to other subsystems.
 *
 * <P>All functions in this class are static so you can access the
 * registry without creating a registry object.
 *
 * @see			SubsystemScope
 * @see         SubsystemInterface
 * @author		Marc Eaddy
 * @version		2.0, 14 Dec 1997
 */
public class SubsystemRegistry
{
	private static Hashtable tableSubsystems = new Hashtable();
	private static UriScope scopeUri = new UriScope("*.*");

	static
	{
		// Create some subsystems and associated scopes.  These will
		// probably be created by an outside source.

		// Lock Manager
		SubsystemScope scopeLockManager	= 
			new SubsystemScope("Lock", scopeUri );

		SimpleLockManager ifaceLockManager = new SimpleLockManagerImpl();

		// Namespace Manager
		SubsystemScope scopeNamespaceManager = 
			new SubsystemScope("Namespace", scopeUri );

		// We have a particular iface for the namespace manager
		NamespaceManager ifaceNamespaceManager = new NamespaceManager();

		// Property Manager
		SubsystemScope scopePropertyManager = 
			new SubsystemScope("Property", scopeUri );

		PropertySubsystem ifacePropertyManager = new PropertySubsystem();

		// Authentication Manager
		SubsystemScope scopeAuthenticationManager	= 
			new SubsystemScope("Authentication", scopeUri );

		AuthenticationManager ifaceAuthenticationManager = 
            new AuthenticationManager();

		// Register the subsystems along with their associated scopes
		SubsystemRegistry.register(scopeLockManager, ifaceLockManager );
		SubsystemRegistry.register(scopeNamespaceManager, ifaceNamespaceManager );
		SubsystemRegistry.register(scopePropertyManager, ifacePropertyManager );
        SubsystemRegistry.register(scopeAuthenticationManager, ifaceAuthenticationManager);

		// Register live property handlers - must be done AFTER the PropertyManager
		// subsystem has been registered
		ifacePropertyManager.register( new LockPropertyHandler() );
    }


	// Do not allow creation of a SubsystemRegistry because its a static
	// class.
	public SubsystemRegistry()
	{ } // Null CTOR

	/**
	 * Registers a subsystem interface associated with a subsystem scope.
	 */
	public static void register(SubsystemScope scope, SubsystemInterface iface)
	{
		if (!tableSubsystems.containsKey(scope))
		{
			tableSubsystems.put(scope, iface);
		}
	}

	/**
	 * Removes the subsystem associated with the specified scope.
	 */
	public static void unregister(SubsystemScope scope)
	{
		if (tableSubsystems.containsKey(scope))
		{
			tableSubsystems.remove(scope);
		}
	}

	/**
	 * Retrieves the subsystem interface for the specified scope.
	 *
	 * @return		SubsystemInterface which must be cast to the
	 *				appropriate subsystems desired before use.
	 *
	 * @exception	NoSuchElementException if the scope is not
	 *				registered.
	 */
	public static SubsystemInterface lookup(String strSubsystemName, Uri uriRequest)
		throws NoSuchElementException
	{
		SubsystemScope scope = 
			new SubsystemScope(strSubsystemName, scopeUri );

		if (tableSubsystems.containsKey(scope))
		{
			return (SubsystemInterface) tableSubsystems.get(scope);
		}
		else
		{
			throw new NoSuchElementException(
				"Subsystem does not exist - " + scope.toString() );
		}
	}

	/**
	 * Retrieves all the registered subsystem scopes.
	 */
	public static Enumeration subsystems()
	{
		return tableSubsystems.keys();
	}

	// Tests this class
	public static void main(String[] args)
	{
/*		UriScope scopeUri = new UriScope("*.*");

		// Create some subsystems and associated scopes.  These will
		// probably be created by an outside source.

		// Lock Manager
		SubsystemScope scopeLockManager	= 
			new SubsystemScope("Lock", scopeUri );

		LockManager ifaceLockManager = new LockManager();

		// Namespace Manager
		SubsystemScope scopeNamespaceManager = 
			new SubsystemScope("Namespace", scopeUri );

		// We have a particular iface for the namespace manager
		NamespaceManager ifaceNamespaceManager = new NamespaceManager();

		// Property Manager
		SubsystemScope scopePropertyManager = 
			new SubsystemScope("Property", scopeUri );

		PropertyManager ifacePropertyManager = new PropertyManager();

		// Bogus subsystem to test UnRegister()
		SubsystemScope scopeBogus = 
			new SubsystemScope("Bogus", scopeUri );
		
		PropertyManager ifaceBogus = new PropertyManager();

		// Register the subsystems along with their associated scopes
		SubsystemRegistry.register(scopeLockManager, ifaceLockManager );
		SubsystemRegistry.register(scopeLockManager, ifaceLockManager ); // No effect
		SubsystemRegistry.register(scopeLockManager, ifaceLockManager ); // No effect
		SubsystemRegistry.register(scopeNamespaceManager, ifaceNamespaceManager );
		SubsystemRegistry.register(scopePropertyManager, ifacePropertyManager );
		SubsystemRegistry.register(scopeNamespaceManager, ifaceNamespaceManager ); // No effect
		SubsystemRegistry.register(scopeBogus, ifaceBogus ); // No effect

		// List the registered subsystems
		for ( Enumeration e = SubsystemRegistry.subsystems(); e.hasMoreElements(); )
		{
			SubsystemScope scope = (SubsystemScope) e.nextElement();
			System.out.println( "Registered subsystem: " + scope.getType() );
		}

		// Use the registry like the Executive and the other subsystems will
		// use it: retrieve a particular subsystem interface.
		NamespaceInterface ifaceNamespace = 
			(NamespaceInterface) SubsystemRegistry.lookup( scopeNamespaceManager );

		if (null == ifaceNamespace)
		{
			System.out.println( "Could not find iface: " + scopeNamespaceManager );
		}
		else
		{
			System.out.println( "Found iface: " + scopeNamespaceManager );
			System.out.println( "Calling Get()" );

			Uri uri = new Uri("http://www.tvguide.com");
			String strResource = new String();

			// Use the Namespace-specific interface
			int nNameSpcReturn = ifaceNamespace.getResource(uri, null);
				
			if (NamespaceInterface.NAMESPC_SUCCESS == nNameSpcReturn)
			{
				System.out.println( "GET success! Resource=" + strResource);
			}
			else
			{
				System.out.println( "GET failure!" );
			}
		}

		// Test UnRegister()
		SubsystemRegistry.unregister( scopeBogus );

		// List the registered subsystems
		for ( Enumeration e = SubsystemRegistry.subsystems(); e.hasMoreElements(); )
		{
			SubsystemScope scope = (SubsystemScope) e.nextElement();
			System.out.println( scope.getType() );
		}
		*/
	}
};