/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.common;

import java.io.File;

/**
 * Wraps a uri to insulate us from changing standards for
 * URLs, URIs and URNs.  As specified in RFC1738, the path
 * separator for a Uri is the '/' character.  However, when
 * determining the mapping between a Uri for a resource and
 * the location of that resource on the file server, the
 * getFullPath() function will prepend the doc root to the
 * Uri and convert the path separator as necessary.  The doc
 * root is obtained by retrieving the "server.root" system
 * property, and appending "public_html" to it.
 *
 * <P>For example, if the server.root property is
 * "D:\Projects\JavaWebServer" and the Uri is "/testing/index.html",
 * and the file server requires path separators to be '\', calling
 * getFullPath() will return 
 * "D:\Projects\JavaWebServer\public_html\testing\index.html".
 *  
 * @author		Marc Eaddy
 * @version		2.0, 3 Nov 1997
 */
public class Uri
{
	private String strUri;
	static private String strServerRoot;
	static private String strPublicHtmlDir;

	static
	{
		String str = System.getProperty("server.root");
		if (str != null)
		{
			strServerRoot = new String(str);
		}
		else
		{
			strServerRoot = new String("");
		}

		strPublicHtmlDir = new String(strServerRoot + File.separatorChar + 
			"public_html");
	}

	public Uri(Uri uri)
	{
		if (uri != null && uri.toString() != null)
		{
			this.strUri = new String(uri.toString());
            strUri.replace('\\', '/');
        }
	}

	public Uri(String strUri)
	{
		if (strUri != null)
		{
			this.strUri = new String(strUri);
            strUri.replace('\\', '/');
		}
	}

	public Uri(File file)
	{
		if (file == null)
		{
			return;
		}

		String strPath = file.getAbsolutePath();

		if (strPath == null)
		{
			return;
		}

		int nStartPos = strPath.indexOf(strPublicHtmlDir);
		if (nStartPos < 0)
		{
			return;
		}

		nStartPos = nStartPos + strPublicHtmlDir.length();

		if (nStartPos < strPath.length())
		{
			this.strUri = strPath.substring(nStartPos);
		}

        strUri.replace('\\', '/');
    }

	public String toString()
	{
		return strUri;
	}

	public String getFullPath()
	{
		if (strUri == null)
		{
			return null;
		}

		if (File.separatorChar != '/')
		{
			return new String(strPublicHtmlDir + 
                                strUri.replace('/', File.separatorChar));
		}
		else
		{
			return new String(strPublicHtmlDir + 
                                strUri.replace('\\', File.separatorChar));
		}
	}

	// Tests this class
	public static void main(String[] args)
	{
		Uri uri = new Uri("http://www.tvguide.com/WhatTheHell+is_this?joe=jack&mama=ou");
		System.out.println(uri);

		// Test null initialization
		Uri uri2 = new Uri(uri);
		Uri uriNull = null;
		Uri uri3 = new Uri(uriNull);
		String strNull = null;
		Uri uri4 = new Uri(strNull);

		Uri uri5 = new Uri("\\marc\\eaddy");
		System.out.println(uri5);
		Uri uri6 = new Uri("/marc/eaddy");
		System.out.println(uri6);

		System.out.println(File.separatorChar);
		System.out.println(File.pathSeparatorChar);

		File file = new File("\\public_html\\index.html");
		Uri uri7 = new Uri(file);
		System.out.println(uri7);
	}
};	