/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.common;

import javax.servlet.ServletInputStream;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Enumeration;

/**
 * Wraps the HttpServletRequest class to abstract the
 * specific protocol used.  To support other protocols
 * we would only need to modify this class and the
 * WebDavRetCode classes.
 *
 * @see			WebDavRetCode
 * @author		Marc Eaddy
 * @version		1.0, 16 Nov 1997
 */

public class WebDavRequest
{
	private Principal principal;
	private String strMethodName;
	private Uri uriRequest;
	private Hashtable tableAttributes = new Hashtable();
	private ServletInputStream streamIn;
	private IOException exIO;

	public WebDavRequest(HttpServletRequest req)
		throws IOException
	{
		// Extract the headers
		for (Enumeration enum = req.getHeaderNames(); enum.hasMoreElements(); )
		{
			String strName = (String) enum.nextElement();
			String strValue = req.getHeader(strName);
			tableAttributes.put(strName, strValue);
		}

		// Extract the method name
		strMethodName = new String(req.getMethod());

		// Extract the user info
		principal = new Principal(req.getRemoteUser());

		// Extract the request uri
		String strRequestUri = new String(req.getPathInfo());

		uriRequest = new Uri(strRequestUri);

		// Extract the input stream
		try
		{
			streamIn = req.getInputStream();
		}
		catch (IOException ex)
		{
			exIO = new IOException(ex.getMessage());
			throw ex;
		}
	}

	public Principal getPrincipal()
	{
		return principal;
	}

	public String getMethodName()
	{
		return strMethodName;
	}

	public Uri getRequestUri()
	{
		return uriRequest;
	}

	public String getValue(String strName)
	{
		if (tableAttributes.containsKey(strName))
		{
			return (String) tableAttributes.get(strName);
		}
		else
		{
			return "Not Found!";
		}
	}

	public Enumeration getAttributeNames()
	{
		return tableAttributes.keys();
	}

	public ServletInputStream getInputStream() 
		throws IOException
	{
		if (null == exIO)
		{
			return streamIn;
		}
		else
		{
			throw exIO;
		}
	}

	/**
	 * WebDAV HTTP Headers
	 */
	public static final String HEADER_COLL_MEMBER		= new String("Collection-Member");

	public static final String HEADER_DAV				= new String("DAV");
	public static final String HEADER_DAV_1				= new String("1");
	public static final String HEADER_DAV_2				= new String("2");

	public static final String HEADER_DEPTH				= new String("Depth");
	public static final String HEADER_DEPTH_0			= new String("0");
	public static final String HEADER_DEPTH_1			= new String("1");
	public static final String HEADER_DEPTH_INFINTY		= new String("infinity");

	public static final String HEADER_DESTINATION		= new String("Destination");

	public static final String HEADER_DESTROY			= new String("Destroy");
	public static final String HEADER_DESTROY_VER		= new String("VersionDestroy");
	public static final String HEADER_DESTROY_NO_UNDO	= new String("NoUndelete");
	public static final String HEADER_DESTROY_UNDO		= new String("Undelete");

	public static final String HEADER_ENFORCE_LIVE		= new String("Enforce-Live-Properties_ _");
	public static final String HEADER_ENFORCE_LIVE_ALL	= new String("_*_");
	public static final String HEADER_ENFORCE_LIVE_OMIT = new String("Omit");

	public static final String HEADER_IF_NONE_STATE_MATCH = new String("If-None-State-Match");
	
	public static final String HEADER_IF_STATE_MATCH	= new String("If-State-Match");
	public static final String HEADER_IF_STATE_MATCH_AND = new String("AND");
	public static final String HEADER_IF_STATE_MATCH_OR = new String("OR");

	public static final String HEADER_LOCK_INFO			= new String("Lock-Info");
	public static final String HEADER_LOCK_TYPE			= new String("LockType");
	public static final String HEADER_LOCK_TYPE_WRITE	= new String("Write");
	public static final String HEADER_LOCK_SCOPE		= new String("LockScope");
	public static final String HEADER_LOCK_SCOPE_EXCL	= new String("Exclusive");
	public static final String HEADER_LOCK_SCOPE_SHR	= new String("Shared");
	public static final String HEADER_ADDITIONAL_LCOKS	= new String("AddLocks");
	public static final String HEADER_LOCK_TREE			= new String("Lock-Tree");
	public static final String HEADER_LOCK_TREE_TRUE	= new String("T");
	public static final String HEADER_LOCK_TREE_FALSE	= new String("F");

	public static final String HEADER_LOCK_TOKEN		= new String("Lock-Token");

	public static final String HEADER_OVERWRITE			= new String("Overwrite");
	public static final String HEADER_OVERWRITE_TRUE	= new String("T");
	public static final String HEADER_OVERWRITE_FALSE	= new String("F");

	public static final String HEADER_PROPFIND			= new String("Propfind");
	public static final String HEADER_PROPFIND_ALL		= new String("allprop");
	public static final String HEADER_PROPFIND_NAME		= new String("propname");

	public static final String HEADER_STATUS_URI		= new String("Status-URI");
	
	public static final String HEADER_TIMEOUT			= new String("Timeout");
	public static final String HEADER_TIMEOUT_INFINITE	= new String("Infinite");
};