/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.executive;

import webdav.common.*;

import java.util.Vector;
import java.util.Enumeration;
import java.net.MalformedURLException;

/**
 * A CommandQueue queues up a sequence of commands and executes
 * them in the order they were queued when execute() is called.
 * If a Command in the queue fails, the direction of traversal
 * of the queue is reversed and each Command is undone until
 * we reach the beginning of the queue.
 *
 * @see			webdav.executive.Command
 * @author		Marc Eaddy
 * @version		2.0, 14 Dec 1997
 */
public class CommandQueue 
{
	// DATA
	private final static int DIR_FORWARD = +1;
	private final static int DIR_BACKWARD = -1;

	private Vector vecCommands;
	private Vector vecUndoOnFailure;

	/**
	 * Create a command queue which will hold nCount Commands.
	 *
	 * @param	nCount is the total number of Commands this
	 *			queue will hold
	 */
	public CommandQueue()
	{ 
		vecCommands = new Vector();
		vecUndoOnFailure = new Vector();
	}

	/**
	 * Add a command to the queue.
	 *
	 * @param cmd				[IN] The Command to add.
	 * @param bUndoOnFailure	[IN] Whether or not the failure
	 *							of this command will cause the
	 *							CommandQueue to rollback.  If
	 *							true, the CommandQueue will
	 *							rollback on failure.  If false
	 *							the CommandQueue will not
	 *							rollback.
	 */
	public void add(Command cmd, boolean bUndoOnFailure)
	{
		vecCommands.addElement(cmd);
		vecUndoOnFailure.addElement(new Boolean(bUndoOnFailure));
	}

	/**
	 * Execute each command in the queue in the order they were added.
	 * If a command fails, backup and undo all the previous Commands
	 * in the queue in reverse order (skipping the failed Command).
	 *
	 * @return	Returns WebDavRetCode indicating success if no errors
	 *			occurred.  Otherwise, the WebDavRetCode will contain
	 *			the actual return code of the failed Command.
	 */
	public int execute()
	{
		int nCurrentIndex = 0;
		int nTotal = vecCommands.size();
		int nDirection = DIR_FORWARD;

		int retCode = WebDavRetCode.WEBDAV_SUCCESS;

		while (nCurrentIndex >= 0 && nCurrentIndex < nTotal)
		{
			Command cmdCurrent = (Command) vecCommands.elementAt(nCurrentIndex);

			// Execute each command - normal operation
			if (DIR_FORWARD == nDirection)
			{
				retCode = cmdCurrent.Do();

				System.err.println("  " + cmdCurrent.getClass().getName() + " returned " + retCode);

				if (retCode < WebDavRetCode.WEBDAV_SUCCESS)
				{
					Boolean bUndoOnFailure = (Boolean) vecUndoOnFailure.elementAt(nCurrentIndex);
					
					if (bUndoOnFailure.booleanValue() == true)
					{
						System.err.println("  ==> ROLLBACK!");
						nDirection = DIR_BACKWARD;	// Backup and undo the commands!
					}
				}
			}

			// Undo each command - rollback operation
			else if (DIR_BACKWARD == nDirection)
			{
				int retCommand = cmdCurrent.UnDo();

				if (retCommand < 0)
				{
					// What do we do here?
				}
			}
			else
			{ }	// ASSERT(false);  --OR-- throw exception

			nCurrentIndex += nDirection;
		}

		return retCode;
	}

	/**
	 * @return An enumeration of the commands within the CommandQueue
	 */
	public Enumeration elements()
	{
		return vecCommands.elements();
	}

	// Tests this class
	public static void main(String[] args)
	{
		/*
		SubsystemRegistry.main(args);

		CommandQueue queueCommands = new CommandQueue(3);

		try
		{
			queueCommands.add( new ValidateUriCmd( new Uri("http://www.tvguide.com"), false ) );
			queueCommands.add( new ValidateUriCmd( new Uri("http://www.invalid.com"), false ) );
			queueCommands.add( new ValidateUriCmd( new Uri("http://www.microsoft.com"), false ) );
		}
		catch (MalformedURLException ex)
		{
		}

		int retCode = queueCommands.execute();

		if (retCode == WebDavRetCode.WEBDAV_SUCCESS)
		{
			System.out.println("Command queue succeeded");
		}
		else
		{
			System.out.println("Command queue failed");
		}
		*/
	}
};