/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.executive;

import webdav.common.*;
import webdav.nsman.NamespaceInterface;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletOutputStream;
import java.io.IOException;
import java.util.Hashtable;

/**
 * The GetMethod class encapsulates the Commands necessary
 * to validate and retrieve a resource file pointed to by
 * a Uri.
 *
 * @see			Method
 * @see         CommandQueue
 * @author		Marc Eaddy
 * @version		1.0, 8 Nov 1997
 */
public class GetMethod 
	extends Method
{
	/**
	 * @see Method
	 */
	public GetMethod(	Uri uri, Principal principal, 
				HttpServletRequest req, HttpServletResponse res)
	{
		super(uri, principal, req, res, DO_NOT_ENFORCE_LOCKS);

		try
		{
			queueCommands.add( new ValidateUriCmd( uri ), true );
			queueCommands.add( 
				new GetResourceCmd( uri, res.getOutputStream() ), false );
		}
		catch (IOException ex)
		{ }
	}

	/**
	 * This Hashtable contains the mapping of WebDAV subsystem
	 * return codes into the appropriate HTTP return code for
	 * this Method.  This is a static variable.
	 */
	private static Hashtable mapStatusCodes = new Hashtable();

	/**
	 * Adds a new WebDAV -> HTTP status code mapping.  This is a static
	 * method because the mapping is a static variable.
	 * 
	 * @param	nKey [IN] WebDAV status code
	 * @param	nVal [IN] HTTP status code
	 * @see		WebDavRetCode
	 */
	private static void addStatusCodeMap(int nKey, int nVal)
	{
		mapStatusCodes.put( new Integer(nKey), new Integer(nVal) );
	}

	/**
	 * Returns the HTTP status code for the WebDav status code specified
	 * by looking it up in the static mapping.  This is a static function.
	 * Since each Method subclass must implement getStatusCode(), the
	 * implementation should simply call the static version of the
	 * function, getStatusCodeImpl().
	 * 
	 * @param nKey	[IN] WebDAV status code
	 */
	private static int getStatusCodeImpl(int nKey)
	{
		Integer intKey = new Integer(nKey);

		if (!mapStatusCodes.containsKey(intKey))
		{
			return -1;
		}
		else
		{
			Integer intVal = (Integer) mapStatusCodes.get(intKey);
			return intVal.intValue();
		}
	}

	// Create the mapping of WebDav subsystem return codes to HTTP
	// response codes for the GetMethod.
	static
	{
		addStatusCodeMap(NamespaceInterface.NAMESPC_SUCCESS,
								WebDavResponse.SC_OK);

		addStatusCodeMap(NamespaceInterface.NAMESPC_UNKNOWN_ERROR,
								WebDavResponse.SC_INTERNAL_SERVER_ERROR);

		addStatusCodeMap(NamespaceInterface.NAMESPC_INTERNAL_ERROR,
								WebDavResponse.SC_INTERNAL_SERVER_ERROR);

		addStatusCodeMap(NamespaceInterface.NAMESPC_URI_NOT_FOUND,
								WebDavResponse.SC_NOT_FOUND);

		addStatusCodeMap(NamespaceInterface.NAMESPC_URI_INVALID,
								WebDavResponse.SC_BAD_REQUEST);

		addStatusCodeMap(NamespaceInterface.NAMESPC_URI_READ_ERROR,
								WebDavResponse.SC_METHOD_FAILURE);
	}

	/**
	 * Delegates the call to the static getStatusCodeImpl().
	 * Also sets the content type header for the response.
	 *
	 * @param	nWebDavReturn	[IN] The return code from the 
	 *							CommandQueue
	 * @return	The HTTP status code translation
	 */
	protected int getStatusCode(int nWebDavReturn)
	{
		int nHttpReturnCode = getStatusCodeImpl(nWebDavReturn);
		
		// On success, set the content-type
		if (WebDavResponse.isSuccess(nHttpReturnCode))
		{
			_res.setContentType(_req.getContentType());
		}

		return nHttpReturnCode;
	}
};

