/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.executive;

import webdav.common.*;
import webdav.lockman.LockManCore;
import webdav.lockman.LockManRequest;
import webdav.lockman.LockManIntf;
import webdav.authentication.AuthenticationInterface;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletOutputStream;
import java.util.Hashtable;
import java.util.NoSuchElementException;

/**
 * The LockMethod class encapsulates the Commands necessary
 * to lock a resource file pointed to by a Uri on behalf
 * of the requesting Principal.
 *
 * @see			Method
 * @see         CommandQueue
 * @author		Marc Eaddy
 * @version		2.0, 14 Dec 1997
 */
class LockMethod 
	extends Method
{
	/**
	 * Create and initialize the LockMethod.
	 *
	 * @see Method
	 */
	LockMethod(	Uri uri, Principal principal, 
				HttpServletRequest req, HttpServletResponse res)
	{
		super(uri, principal, req, res, ENFORCE_LOCKS);

        queueCommands.add( _cmdAuthenticate, true);
		queueCommands.add( _cmdLockResource, false);
	}

	/**
	 * This Hashtable contains the mapping of WebDAV subsystem
	 * return codes into the appropriate HTTP return code for
	 * this Method.  This is a static variable.
	 */
	private static Hashtable mapStatusCodes = new Hashtable();

	/**
	 * Adds a new WebDAV -> HTTP status code mapping.  This is a static
	 * method because the mapping is a static variable.
	 * 
	 * @param	nKey [IN] WebDAV status code
	 * @param	nVal [IN] HTTP status code
	 * @see		WebDavRetCode
	 */
	static void addStatusCodeMap(int nKey, int nVal)
	{
		mapStatusCodes.put( new Integer(nKey), new Integer(nVal) );
	}

	/**
	 * Returns the HTTP status code for the WebDav status code specified
	 * by looking it up in the static mapping.  This is a static function.
	 * Since each Method subclass must implement getStatusCode(), the
	 * implementation should simply call the static version of the
	 * function, getStatusCodeImpl().
	 * 
	 * @param nKey	[IN] WebDAV status code
	 */
	static int getStatusCodeImpl(int nKey)
		throws NoSuchElementException
	{
		Integer intKey = new Integer(nKey);

		if (!mapStatusCodes.containsKey(intKey))
		{
			throw new NoSuchElementException("Invalid LockManager Interface Status Code returned: " +
											intKey);
		}
		else
		{
			Integer intVal = (Integer) mapStatusCodes.get(intKey);
			return intVal.intValue();
		}
	}

	// Create the mapping of WebDav subsystem return codes to HTTP
	// response codes for the LockMethod.
	static
	{
		addStatusCodeMap(LockManIntf.LOCKMAN_SUCCESS,
								WebDavResponse.SC_OK);	
		addStatusCodeMap(LockManIntf.LOCKMAN_LOCK_CREATED,
								WebDavResponse.SC_OK);	
		addStatusCodeMap(LockManIntf.LOCKMAN_INTERNAL_ERROR,
								WebDavResponse.SC_INTERNAL_SERVER_ERROR);	
		addStatusCodeMap(LockManIntf.LOCKMAN_FAILURE,
								WebDavResponse.SC_METHOD_FAILURE);	
		addStatusCodeMap(LockManIntf.LOCKMAN_ALREADY_LOCKED,
								WebDavResponse.SC_CONFLICT);
		addStatusCodeMap(LockManIntf.LOCKMAN_BAD_REQUEST,
								WebDavResponse.SC_BAD_REQUEST);

		// These would only be returned by unlock() after a rollback
		addStatusCodeMap(LockManIntf.LOCKMAN_LOCK_NOT_FOUND,
								WebDavResponse.SC_METHOD_FAILURE);	
		addStatusCodeMap(LockManIntf.LOCKMAN_NO_REMOVE_RELOCK,
								WebDavResponse.SC_METHOD_FAILURE);	

        // Handle possible authentication errors
        addStatusCodeMap(AuthenticationInterface.AUTH_FAILURE,
                                WebDavResponse.SC_UNAUTHORIZED);
        addStatusCodeMap(AuthenticationInterface.AUTH_INTERNAL_ERROR,
                                WebDavResponse.SC_INTERNAL_SERVER_ERROR);
		addStatusCodeMap(AuthenticationInterface.AUTH_BAD_REQUEST,
								WebDavResponse.SC_BAD_REQUEST);
    }

	/**
	 * Delegates the call to the static getStatusCodeImpl().
	 * Also sets the lock token header for the HTTP response.
	 *
	 * @param	nWebDavReturn	[IN] The return code from the 
	 *							CommandQueue
	 * @return	The HTTP status code translation
	 */
	protected int getStatusCode(int nWebDavReturn)
	{
		LockManCore lockToken = _cmdLockResource.getLockToken();

		_res.setHeader(	WebDavRequest.HEADER_LOCK_TOKEN, 
						lockToken.getLockTokenString());

        int nHttpStatusCode = getStatusCodeImpl(nWebDavReturn);

        if (nWebDavReturn == AuthenticationInterface.AUTH_FAILURE)
        {
            _cmdAuthenticate.sendWWWAuthenticateHeaders(_res);
        }

		return nHttpStatusCode;
    }
};