/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.executive;

import webdav.common.*;
import webdav.lockman.*;	// Only deals with the iface
import java.util.NoSuchElementException;
import java.io.OutputStream;

/**
 * The LockResourceCmd command locks the resource.  The undo
 * behaviour is to unlock the resource.
 *
 * @see			Command
 * @see         NamespaceInterface
 * @author		Marc Eaddy
 * @version		1.0, 8 Nov 1997
 */
public class LockResourceCmd
	extends Command
{
	private LockManIntf ifaceLockManager;	// Should be static

	private Uri uri;
	private LockManRequest lockRequest = null;
	private LockManCore lockResponse = null;

	/**
	 * Create and initialize the LockResourceCmd command.
	 *
	 * @param uri			[IN] Location of the resource file
	 * @param lockRequest	[IN] Owner's lock request
	 * @param lockResponse	[OUT] After the call to DoImpl() this will
	 *						contain the lock token
	 */
	public LockResourceCmd(	Uri uri, 
							LockManRequest lockRequest, 
							LockManCore lockResponse)
		throws NoSuchElementException
	{
		super();

		// Copy the input parameters
		this.uri = uri;
		this.lockRequest = lockRequest;
		this.lockResponse = lockResponse;

		//*** Connect to the Namespace Manager ***

		// Note: This could cause a problem if the destination
		// Uri is in a different namespace manager scope.

		ifaceLockManager = (LockManIntf) 
			SubsystemRegistry.lookup( "Lock", uri );
	}

	/**
	 * See Command documentation.
	 */
	protected int DoImpl()
	{
		return ifaceLockManager.lock(uri, lockRequest, lockResponse);
	}

	/**
	 * See Command documentation.
	 */
	protected int UnDoImpl()
	{ 
		return ifaceLockManager.unlock(uri, lockResponse);
	}

	/**
	 * @return The lock token retrieved from the call to lock().
	 */
	public LockManCore getLockToken()
	{
		return lockResponse;
	}

	public static final void main(String args[] )
	{
		Uri uri = new Uri("/index2.html");

		System.out.println("Uri=" + uri.getFullPath());

		new SubsystemRegistry();

		LockManRequest req = new LockManRequest("meaddy@newscorp.com",
												null,
												"Write",
												"Exclusive");
		LockManCore lockToken = new LockManCore();

		LockResourceCmd cmdLockResource = new LockResourceCmd(uri, req, lockToken);

		System.out.println("Creating a lock on '" + uri + "' with request:\n" + req);

		// Create a lock - should succeed

		int nLockRet = cmdLockResource.DoImpl();

		if (nLockRet < WebDavRetCode.WEBDAV_SUCCESS)
		{
			System.out.println("ERROR: Lock was not created!!! Return: " + nLockRet);
			return;
		}

		System.out.println("Lock Token returned:\n" + cmdLockResource.getLockToken());

		// Rollback the lock
		System.out.println("Removing the lock on '" + uri + "' with request:\n" + req);

		nLockRet = cmdLockResource.UnDoImpl();

		if (nLockRet < WebDavRetCode.WEBDAV_SUCCESS)
		{
			System.out.println("ERROR: Lock was not removed!!! Return: " + nLockRet);
			return;
		}

		System.out.println("Lock Token returned:\n" + cmdLockResource.getLockToken());
	}
};