/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.executive;

import webdav.common.*;
import webdav.lockman.LockManRequest;
import webdav.lockman.LockManCore;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * Each HTTP method implemented by the WebDav Server will
 * have a Method subclass associated with it.  The Method
 * subclass will create the CommandQueue needed to carry-
 * out the HTTP method.
 *
 * @see			MethodQueue
 * @see         CommandQueue
 * @author		Marc Eaddy
 * @version		2.0, 14 Dec 1997
 */
abstract public class Method
{
	protected static final boolean ENFORCE_LOCKS = true;
	protected static final boolean DO_NOT_ENFORCE_LOCKS = false;

	protected CommandQueue			queueCommands = new CommandQueue();

	// Stores the current request
	protected Uri					_uri; 
	protected HttpServletRequest	_req;
	protected HttpServletResponse	_res;

	// Whether or not to execute the CommandQueue
	protected boolean				_bExecute = true;

	// Helper variables for handling internal locking and unlocking
	protected LockManRequest		_lockRequest = null;
	protected LockManCore			_lockResponse = null;
	protected LockResourceCmd		_cmdLockResource = null;
	protected UnlockResourceCmd		_cmdUnlockResource = null;
    protected AuthenticateCmd       _cmdAuthenticate = null;

	private int _nHttpStatusCode;

	/**
	 * Create a Method object.  This function will only be
	 * invoked by a Method subclasses by calling super().
	 *
	 * @param uri		[IN] Uri of the request
	 * @param principal	[IN] The Principal identifying the 
	 *					requesting client
	 * @param req		[IN] The HttpServletRequest passed to
	 *					the WebDavServlet.
	 * @param res		[OUT] The HttpServletResponse passed to
	 *					the WebDavServlet.  After this method's
	 *					execution, will contain the HTTP response
	 *					headers and response body.
	 */
	public Method(	Uri uri, 
					Principal principal, 
					HttpServletRequest req, 
					HttpServletResponse res,
					boolean bEnforceLocks)
	{ 
		this._req = req;
		this._res = res;
		this._uri = uri;
		this._nHttpStatusCode = WebDavResponse.SC_OK;

		if (bEnforceLocks)
		{
			String strOwnerInfoHeader = req.getHeader("Owner-Info");
			String strLockTokenHeader = req.getHeader(WebDavRequest.HEADER_LOCK_TOKEN);
			String strLockTypeHeader  = req.getHeader(WebDavRequest.HEADER_LOCK_TYPE);
			String strLockScopeHeader = req.getHeader(WebDavRequest.HEADER_LOCK_SCOPE);
			String strAuthorizationHeader = req.getHeader("Authorization");

            _cmdAuthenticate = new AuthenticateCmd( strAuthorizationHeader,
                                                    req.getRemoteAddr(),
                                                    "my realm",
                                                    req.getMethod(),
                                                    uri);

			_lockRequest = new LockManRequest(	strOwnerInfoHeader, 
												strLockTokenHeader,
												strLockTypeHeader,
												strLockScopeHeader);

			_lockResponse = new LockManCore(	strOwnerInfoHeader,
												strLockTokenHeader);

			_cmdLockResource = new LockResourceCmd( uri, 
													_lockRequest, 
													_lockResponse);

			_cmdUnlockResource = new UnlockResourceCmd( uri, 
														_lockResponse);
		}
	}

	/**
	 * Execute this method and set the response HTTP status code
	 * and status text.  If an error occurs, output an error page.
	 *
	 * @return		Returns <0 for error, 0 for success, >0 for
	 *				success with info
	 *
	 */
	public int Do()
	{ 
		System.err.println("\nExecuting " + getClass().getName() + " CommandQueue:");

		int nQueueReturn;

		// Execute the queue unless the method specifically
		// set the execute flag to false
		if (_bExecute)
		{
			nQueueReturn = queueCommands.execute();
			
			// Translate the WebDAV return code into an HTTP response
			// code
			_nHttpStatusCode = getStatusCode(nQueueReturn);
		}
		else
		{
			// If the method does not want the queue to be executed,
			// these default errors get generated:

			nQueueReturn = WebDavRetCode.WEBDAV_FAILURE;

			_nHttpStatusCode = WebDavResponse.SC_BAD_REQUEST;
		}

		// If an error occurred during the processing of the queue,
		// output an error page
		if (nQueueReturn < WebDavRetCode.WEBDAV_SUCCESS &&
			!WebDavResponse.isSuccess(_nHttpStatusCode))
		{
			WebDavResponse.outputErrorPage(	_res,
											_nHttpStatusCode,
											_req);
		}

		// If no mapping for the internal subsystem status code exists,
		// then the HTTP status code will be -1.  This indicates a
        // programming error by who ever wrote the Method.  We must catch
        // this here.
		if (_nHttpStatusCode == -1)
		{
			_nHttpStatusCode = WebDavResponse.SC_INTERNAL_SERVER_ERROR;

			System.out.println("Status code unknown (Native Error Code: " +
				+ nQueueReturn + ")");
		}

		// Get the status text for the HTTP status code
		String strStatusText = WebDavResponse.getStatusText(_nHttpStatusCode);

		System.out.println("Status Code: " + _nHttpStatusCode +
							" " + strStatusText);

		_res.setStatus(_nHttpStatusCode, strStatusText);

		return nQueueReturn;
	}

	/**
	 * All subclasses will implement this method to return
	 * the HTTP status code that is mapped for all possible
	 * WebDAV subsystem return codes for this method.
	 *
	 * @param	nWebDavCode is the subsystem return code
	 *			returned by a Command subclass to the
	 *			Method subclass
	 * @return  The HTTP status code mapped for this WebDAV
	 *			subsystem error code, or -1 if no mapping
	 *			exists.
	 * @see		GetMethod
	 */
	abstract protected int getStatusCode(int nWebDavCode);
};