/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.executive;

import webdav.common.*;
import webdav.nsman.NamespaceInterface;
import webdav.lockman.LockManCore;
import webdav.lockman.LockManIntf;
import webdav.propman.PropertyManager;
import webdav.authentication.AuthenticationInterface;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletOutputStream;
import java.io.IOException;
import java.util.Hashtable;

/**
 * The MoveMethod class encapsulates the Commands necessary
 * to validate and move a resource.
 *
 * @see			Method
 * @see         CommandQueue
 * @see         MoveResourceCmd
 * @author		Marc Eaddy
 * @version		2.0, 14 Dec 1997
 */
public class MoveMethod 
	extends Method
{
	private LockManCore _lockDestToken = new LockManCore();

	/**
	 * @see Method
	 */
	public MoveMethod(	Uri uri, Principal principal, 
				HttpServletRequest req, HttpServletResponse res)
	{
		super(uri, principal, req, res, ENFORCE_LOCKS);

		boolean bOverwrite;

		// Get the Destination uri

		String strDestinationUri = req.getHeader(WebDavRequest.HEADER_DESTINATION);

		if (strDestinationUri == null)
		{
			// Header has invalid value
			_bExecute = false;
			return;
		}

		// Get the Depth header

		Depth depth = new Depth(req.getHeader(WebDavRequest.HEADER_DEPTH));

		if (!depth.isValid())
		{
			_bExecute = false;
			return;
		}

		// Get the Overwrite header

		String strOverwrite = req.getHeader(WebDavRequest.HEADER_OVERWRITE);

		if (strOverwrite == null ||
			strOverwrite.equals(WebDavRequest.HEADER_OVERWRITE_TRUE))
		{
			bOverwrite = true;
		}
		else if (strOverwrite.equals(WebDavRequest.HEADER_OVERWRITE_FALSE))
		{
			bOverwrite = false;
		}
		else
		{
			// Header has invalid value
			_bExecute = false;
			return;
		}

		Uri uriSrc = new Uri(uri);
		Uri uriDest = new Uri(strDestinationUri);

		System.out.println ("SRC URI is " + uriSrc.toString()); // RSA
		System.out.println ("dest URI is " + uriDest.toString()); // RSA

		LockResourceCmd cmdLockDestResource = new LockResourceCmd(	uriDest,
																	_lockRequest,
																	_lockDestToken);

		UnlockResourceCmd cmdUnlockDestResource = new UnlockResourceCmd( uriDest,
																	_lockDestToken);

		MoveResourceCmd cmdMoveResource = new MoveResourceCmd( uriSrc, 
												uriDest,
												bOverwrite,
												depth); 

		MovePropCmd cmdMoveProperties = new MovePropCmd (	uriSrc,
															uriDest,
															bOverwrite,
															depth);
		queueCommands.add( new ValidateUriCmd( uri ), true );
        queueCommands.add( _cmdAuthenticate, true);
		queueCommands.add( cmdLockDestResource, true );
		queueCommands.add( _cmdLockResource, true);
		queueCommands.add( cmdMoveResource, true );
		queueCommands.add( cmdMoveProperties, true );
		queueCommands.add( _cmdUnlockResource, true);
		queueCommands.add( cmdUnlockDestResource, false);
	}

	/**
	 * This Hashtable contains the mapping of WebDAV subsystem
	 * return codes into the appropriate HTTP return code for
	 * this Method.  This is a static variable.
	 */
	private static Hashtable mapStatusCodes = new Hashtable();

	/**
	 * Adds a new WebDAV -> HTTP status code mapping.  This is a static
	 * method because the mapping is a static variable.
	 * 
	 * @param	nKey [IN] WebDAV status code
	 * @param	nVal [IN] HTTP status code
	 * @see		WebDavRetCode
	 */
	static void addStatusCodeMap(int nKey, int nVal)
	{
		mapStatusCodes.put( new Integer(nKey), new Integer(nVal) );
	}

	/**
	 * Returns the HTTP status code for the WebDav status code specified
	 * by looking it up in the static mapping.  This is a static function.
	 * Since each Method subclass must implement getStatusCode(), the
	 * implementation should simply call the static version of the
	 * function, getStatusCodeImpl().
	 * 
	 * @param nKey	[IN] WebDAV status code
	 * @return		The HTTP status code for the WebDav status code
	 */
	static int getStatusCodeImpl(int nKey)
	{
		Integer intKey = new Integer(nKey);

		if (!mapStatusCodes.containsKey(intKey))
		{
			return -1;
		}
		else
		{
			Integer intVal = (Integer) mapStatusCodes.get(intKey);
			return intVal.intValue();
		}
	}

	// Create the mapping of WebDav subsystem return codes to HTTP
	// response codes for the GetMethod.
	static
	{
		addStatusCodeMap(NamespaceInterface.NAMESPC_SUCCESS,
								WebDavResponse.SC_OK);

		addStatusCodeMap(NamespaceInterface.NAMESPC_RESOURCE_CREATED,
								WebDavResponse.SC_CREATED);

		addStatusCodeMap(NamespaceInterface.NAMESPC_NO_OVERWRITE,
								WebDavResponse.SC_PRECONDITION_FAILED);

		addStatusCodeMap(NamespaceInterface.NAMESPC_UNKNOWN_ERROR,
								WebDavResponse.SC_INTERNAL_SERVER_ERROR);

		addStatusCodeMap(NamespaceInterface.NAMESPC_INTERNAL_ERROR,
								WebDavResponse.SC_INTERNAL_SERVER_ERROR);

		addStatusCodeMap(NamespaceInterface.NAMESPC_URI_NOT_FOUND,
								WebDavResponse.SC_NOT_FOUND);

		addStatusCodeMap(NamespaceInterface.NAMESPC_URI_INVALID,
								WebDavResponse.SC_BAD_REQUEST);

		addStatusCodeMap(NamespaceInterface.NAMESPC_URI_READ_ERROR,
								WebDavResponse.SC_METHOD_FAILURE);

		addStatusCodeMap(NamespaceInterface.NAMESPC_URI_NOT_WRITEABLE,
								WebDavResponse.SC_METHOD_FAILURE);

		addStatusCodeMap(NamespaceInterface.NAMESPC_NOT_ENOUGH_SPACE,
								WebDavResponse.SC_INSUFFICIENT_SPACE_ON_RESOURCE);

		addStatusCodeMap(NamespaceInterface.NAMESPC_PARENT_NOT_FOUND,
								WebDavResponse.SC_METHOD_FAILURE);

		addStatusCodeMap(NamespaceInterface.NAMESPC_COLL_NOT_SUPPORTED,
								WebDavResponse.SC_METHOD_FAILURE);

		addStatusCodeMap(NamespaceInterface.NAMESPC_COPY_TO_SELF,
								WebDavResponse.SC_METHOD_FAILURE);

		addStatusCodeMap(NamespaceInterface.NAMESPC_URI_DELETE_ERROR,
		 						WebDavResponse.SC_METHOD_FAILURE);

		// Handle possible property manager errors
		addStatusCodeMap(PropertyManager.pm_err_general,
								WebDavResponse.SC_METHOD_FAILURE);	

		// Handle possible lock manager errors
		addStatusCodeMap(LockManIntf.LOCKMAN_INTERNAL_ERROR,
								WebDavResponse.SC_INTERNAL_SERVER_ERROR);	
		addStatusCodeMap(LockManIntf.LOCKMAN_FAILURE,
								WebDavResponse.SC_METHOD_FAILURE);	
		addStatusCodeMap(LockManIntf.LOCKMAN_ALREADY_LOCKED,
								WebDavResponse.SC_CONFLICT);
		addStatusCodeMap(LockManIntf.LOCKMAN_BAD_REQUEST,
								WebDavResponse.SC_BAD_REQUEST);
		addStatusCodeMap(LockManIntf.LOCKMAN_NO_REMOVE_RELOCK,
								WebDavResponse.SC_OK);

        // Handle possible authentication errors
        addStatusCodeMap(AuthenticationInterface.AUTH_FAILURE,
                                WebDavResponse.SC_UNAUTHORIZED);
        addStatusCodeMap(AuthenticationInterface.AUTH_INTERNAL_ERROR,
                                WebDavResponse.SC_INTERNAL_SERVER_ERROR);
		addStatusCodeMap(AuthenticationInterface.AUTH_BAD_REQUEST,
								WebDavResponse.SC_BAD_REQUEST);
    }

	/**
	 * Delegates the call to the static getStatusCodeImpl().
	 *
	 * @param	nWebDavReturn	[IN] The return code from the 
	 *							CommandQueue
	 * @return	The HTTP status code translation
	 */
	protected int getStatusCode(int nWebDavReturn)
	{
        int nHttpStatusCode = getStatusCodeImpl(nWebDavReturn);

        if (nWebDavReturn == AuthenticationInterface.AUTH_FAILURE)
        {
            _cmdAuthenticate.sendWWWAuthenticateHeaders(_res);
        }

		return nHttpStatusCode;
	}
};

