/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.executive;

import java.io.*;
import java.util.Vector;
import java.util.Enumeration;
import java.util.StringTokenizer;
import javax.servlet.http.*;
import com.ms.xml.util.Name;
import webdav.common.*;

/**
 * The PropFindMethod class encapsulates the Commands necessary
 * to get the names and values of properties defined on a resource.
 *
 * @see			Method
 * @see         CommandQueue
 * @author		Jonathan Shapiro, Marc Eaddy
 * @version		2.0, 14 Dec 1997
 */
public class PropFindMethod 
	extends Method
{
	/**
	 * Create and initialize the PropFindMethod.
	 *
	 * @param	[IN] uri - Location of the resource file
	 * @param	[IN] streamIn - Input stream which will
	 *			overwrite the contents of the resource file
	 */
	public PropFindMethod( Uri uri, Principal principal, 
							HttpServletRequest req, HttpServletResponse res)
	{
		super(uri, principal, req, res, DO_NOT_ENFORCE_LOCKS);

		_depth = new Depth(req.getHeader(WebDavRequest.HEADER_DEPTH));
		_propfindHeader = req.getHeader(WebDavRequest.HEADER_PROPFIND);

		if (!_depth.isValid() || _propfindHeader == null)
		{
			_returnCode = WebDavResponse.SC_BAD_REQUEST;
			_bExecute = false;
		}
		else
		{
			queueCommands = prepareQueue(_uri, _propfindHeader, _depth);
		}

		if (queueCommands == null)
		{
			_bExecute = false;
			_returnCode = WebDavResponse.SC_BAD_REQUEST;
		}
	}

	protected int getStatusCode(int nWebDavCode)
	{
		_returnCode = getResponse(queueCommands);
		return _returnCode;
	}

	/**
	 * The PROPFIND method has three variants depending on the value of
	 * the propfind header. Each requires a different command queue structure
	 * and each can be executed to a depth of 0, 1, or infinity:<br>
	 * <pre>
	 *		Header Value		QueueStructure						<br>
	 *		-------------------------------------------				<br>
	 *		list of properties	1 or more GetProperty commands		<br>
	 *		allprop				1 GetAllProps command				<br>
	 *		propname			1 GetAllNames command				<br>
	 * </pre>
	 * @see Method
	 */
	private CommandQueue prepareQueue(	Uri uri, 
										String strPropfindHeader, 
										Depth depth)
	{
		Vector vecPropertyNames = new Vector();
		Vector tree = new Vector();

		getTree(_uri, depth, tree);

		if (!strPropfindHeader.equals(WebDavRequest.HEADER_PROPFIND_ALL) && 
			!strPropfindHeader.equals(WebDavRequest.HEADER_PROPFIND_NAME))
		{
			// make a list of property names
			getProps(strPropfindHeader, vecPropertyNames);
			if (vecPropertyNames == null || vecPropertyNames.size() == 0)
			{
				return null; 
			}
		}

		// All PROPFIIND queues start with a precondition that the
		// request URI is valid
		queueCommands.add( new ValidateUriCmd( _uri), true );		

		Enumeration enumTree = tree.elements();
		Uri href = null;

		while (enumTree.hasMoreElements())
		{
			href = (Uri) enumTree.nextElement();

			if (strPropfindHeader.equalsIgnoreCase( 
				WebDavRequest.HEADER_PROPFIND_ALL ))
			{
				queueCommands.add(new GetAllPropsCmd(href), false);
			} 
			else if (strPropfindHeader.equalsIgnoreCase( 
				WebDavRequest.HEADER_PROPFIND_NAME ))
			{
				queueCommands.add(new GetAllNamesCmd(href), false);

			} 
			else
			{
				Enumeration enumProps = vecPropertyNames.elements();
				while (enumProps.hasMoreElements())
				{
					queueCommands.add(	new GetPropCmd(	href, 
														(Name) enumProps.nextElement()),
														false);
				}
			}
		}

		return queueCommands;
	}

	private boolean getProps(String propHeader, Vector vecPropertyNames)
	{
		Name propName = null;
		StringTokenizer tokenizer = new StringTokenizer(propHeader, "< >");

		while (tokenizer.hasMoreTokens())
		{
			propName = Name.create((String)tokenizer.nextToken());
			vecPropertyNames.addElement(propName);
		}
		return true;
	}

	private boolean getTree(Uri baseUri, Depth depth, Vector allUris)
	{
		/*
		   When we get around to supporting collections, this method will
		   contain code for taking a Uri, deciding if it represents a collection
		   and returning a vector of all of its members to the specified depth.

		   In pseudo-code:

			if (!isCollection(baseUri) || depth = 0) { //for non-colections or collections a 0 depth
				allUris.addElement(baseUri);
				return true;
			} else if (depth == 1)
				allUris.addElement(baseUri);
				for all children
					getTree(baseUri, 0, allUris);
				return true;
			} else if (depth == Integer.MAX_VALUE) {
				allUris.addElement(baseUri)
				for all children
					getTree(baseUri, Integer.MAX_VALUE, allUris);
				return true;
			} else {
				return false
			}

		   For now, however, just put the baseUri on the vector and return.
		*/

		allUris.addElement(baseUri);
		return true;
	}

	/**
	 * Construct a Mulit-status response to reflect the final state of the
	 * command queue
	 */
	int getResponse(CommandQueue queueCommands)
	{
		MultiStatusElement elemMultiStatus = 
			new MultiStatusElement();
		
		Enumeration enumCommands = queueCommands.elements();

		while (enumCommands.hasMoreElements())
		{
			Command cmd = (Command) enumCommands.nextElement();

			if (cmd instanceof ValidateUriCmd)
				continue; //Precondition. Generates no response info

			if (cmd instanceof GetAllNamesCmd ||
				cmd instanceof GetAllPropsCmd ||
				cmd instanceof GetPropCmd) {

				PropCmd propCmd = (PropCmd)cmd;

				int nHttpStatusCode = propCmd.getStatusCodeImpl(propCmd.getInternalStatusCode());

				elemMultiStatus.addResponse(	propCmd.getHref(),
												propCmd.getOutput(),
												nHttpStatusCode);
			}
		}

		_res.setContentType("text/xml");

		try 
		{
			elemMultiStatus.save(_res.getOutputStream());
		} 
		catch (IOException e)
		{
			return WebDavResponse.SC_INTERNAL_SERVER_ERROR;
		}

		return WebDavResponse.SC_MULTI_STATUS;
	}

	private Depth _depth = null;
	private String _propfindHeader = null;
	private int _returnCode = WebDavResponse.SC_OK;
};
