/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.executive;

import webdav.common.*;
import webdav.lockman.LockManIntf;
import webdav.authentication.AuthenticationInterface;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import com.ms.xml.om.*;
import com.ms.xml.util.XMLInputStream;
import com.ms.xml.util.XMLOutputStream;
import com.ms.xml.util.Name;
import com.ms.xml.parser.ParseException;
import java.io.IOException;
import java.util.*;

/**
 * The PropFindMethod class encapsulates the Commands necessary
 * to get the names and values of properties defined on a resource.
 *
 * @see			Method
 * @see         CommandQueue
 * @author		Jonathan Shapiro, Marc Eaddy
 * @version		2.0, 14 Dec 1997
 */
public class PropPatchMethod extends Method
{
	/**
	* @see Method
	*/
	public PropPatchMethod(	Uri uri, Principal principal, 
							HttpServletRequest req, HttpServletResponse res)
	{ 
		super(uri, principal, req, res, ENFORCE_LOCKS);

		//Parse request body

		try 
        {
			XMLInputStream xml_in = new XMLInputStream(_req.getInputStream());
			_reqDocument.load(xml_in);

//			_returnCode = validateBody();

			//build queue
			if (_returnCode == WebDavResponse.SC_MULTI_STATUS) 
            {
				queueCommands = prepareQueue(_uri, _reqDocument);
				if (queueCommands != null) 
                {
					_returnCode = WebDavResponse.SC_OK;
					return;
				}
			}

			if (queueCommands == null) 
            {
				_bExecute = false;
				_returnCode = WebDavResponse.SC_INTERNAL_SERVER_ERROR;
			}
		} 
        catch (IOException e) 
        {
			e.printStackTrace();
			_returnCode = WebDavResponse.SC_INTERNAL_SERVER_ERROR;
		}  
        catch (ParseException e) 
        {
			e.printStackTrace();
			_returnCode = WebDavResponse.SC_INTERNAL_SERVER_ERROR;
		}
	}

	/**
	 * This Hashtable contains the mapping of WebDAV subsystem
	 * return codes into the appropriate HTTP return code for
	 * this Method.  This is a static variable.
	 */
	private static Hashtable mapStatusCodes = new Hashtable();

	/**
	 * Adds a new WebDAV -> HTTP status code mapping.  This is a static
	 * method because the mapping is a static variable.
	 * 
	 * @param	nKey [IN] WebDAV status code
	 * @param	nVal [IN] HTTP status code
	 * @see		WebDavRetCode
	 */
	static void addStatusCodeMap(int nKey, int nVal)
	{
		mapStatusCodes.put( new Integer(nKey), new Integer(nVal) );
	}

	/**
	 * Returns the HTTP status code for the WebDav status code specified
	 * by looking it up in the static mapping.  This is a static function.
	 * Since each Method subclass must implement getStatusCode(), the
	 * implementation should simply call the static version of the
	 * function, getStatusCodeImpl().
	 * 
	 * @param nKey	[IN] WebDAV status code
	 */
	static int getStatusCodeImpl(int nKey)
	{
		Integer intKey = new Integer(nKey);

		if (!mapStatusCodes.containsKey(intKey))
		{
			return -1;
		}
		else
		{
			Integer intVal = (Integer) mapStatusCodes.get(intKey);
			return intVal.intValue();
		}
	}

	// Create the mapping of WebDav subsystem return codes to HTTP
	// response codes for the PropPatchMethod
	static
	{
		// Handle possible lock manager errors
		addStatusCodeMap(LockManIntf.LOCKMAN_INTERNAL_ERROR,
								WebDavResponse.SC_INTERNAL_SERVER_ERROR);	
		addStatusCodeMap(LockManIntf.LOCKMAN_FAILURE,
								WebDavResponse.SC_METHOD_FAILURE);	
		addStatusCodeMap(LockManIntf.LOCKMAN_ALREADY_LOCKED,
								WebDavResponse.SC_CONFLICT);
		addStatusCodeMap(LockManIntf.LOCKMAN_BAD_REQUEST,
								WebDavResponse.SC_BAD_REQUEST);
		addStatusCodeMap(LockManIntf.LOCKMAN_NO_REMOVE_RELOCK,
								WebDavResponse.SC_OK);

        // Handle possible authentication errors
        addStatusCodeMap(AuthenticationInterface.AUTH_FAILURE,
                                WebDavResponse.SC_UNAUTHORIZED);
        addStatusCodeMap(AuthenticationInterface.AUTH_INTERNAL_ERROR,
                                WebDavResponse.SC_INTERNAL_SERVER_ERROR);
		addStatusCodeMap(AuthenticationInterface.AUTH_BAD_REQUEST,
								WebDavResponse.SC_BAD_REQUEST);
	}

	/**
	* @see Method
	*/
	protected int getStatusCode(int nWebDavCode) 
    {
		_returnCode = getResponse(queueCommands);
		return _returnCode;
	}


	/*Private Methods*/
	private CommandQueue prepareQueue(Uri uri, Document doc) 
    {
		try 
        {
			XMLOutputStream xml_out = new XMLOutputStream(System.err);
			doc.save(xml_out);
		} 
        catch (IOException e) 
        {
			e.printStackTrace();
		}

		CommandQueue queue = new CommandQueue();
		if (queue == null) 
			return null;

		/* Start with preconditions:
			1.Is this a valid uri? 
            2.Is the user authenticated?
			3.Is the resource unlocked?
		*/
		queue.add( new ValidateUriCmd( uri), true );
        queue.add( _cmdAuthenticate, true);
		queue.add( _cmdLockResource, true);

		Enumeration enumDoc = doc.getElements();
		while (enumDoc.hasMoreElements()) 
        {
			Element e = (Element) enumDoc.nextElement();

			if (!e.getTagName().equals(WebdavXML.ELEM_PROPERTYUPDATE)) 
            {
                continue;
            }

			Enumeration enumUpdates = e.getElements();
			while (enumUpdates.hasMoreElements()) 
            {
				Element elemCmd = (Element) enumUpdates.nextElement();

			    Element elemProp = null;

                // Find the PROP element
                Enumeration enumProp = elemCmd.getElements();
                while (enumProp.hasMoreElements()) 
                {
					Element elemTmp = (Element) enumProp.nextElement();
					if (elemTmp.getTagName() != null && 
                        elemTmp.getTagName().equals(WebdavXML.ELEM_PROP)) 
                    {
					    elemProp = elemTmp;
					}
				}

                // Validate the request
				if (elemCmd.getTagName() == null || elemProp == null ||
                    (!elemCmd.getTagName().equals(WebdavXML.ELEM_SET) &&
					!elemCmd.getTagName().equals(WebdavXML.ELEM_REMOVE)) ) 
                {
					_returnCode = WebDavResponse.SC_BAD_REQUEST;
                    continue;
                }

                Command cmd = null;

				if (elemCmd.getTagName().equals(WebdavXML.ELEM_SET)) 
                {
				    cmd = new SetPropCmd(uri, elemProp);
                } 
                else if (elemProp != null && 
                        elemCmd.getTagName().equals(WebdavXML.ELEM_REMOVE)) 
                {
				    cmd = new DeletePropCmd(uri, elemProp);
				}
                else
                {
                    Assert.assert(false);
                }

				queue.add(cmd, true);
			}
		}

	    queue.add( _cmdUnlockResource, false);

		return queue;
	}

	/**
	 * Construct a Mulit-status response to reflect the final state of the
	 * command queue
	 */
	private int getResponse(CommandQueue queueCommands) 
    {
		MultiStatusElement elemMultiStatus = 
			new MultiStatusElement();
		
		Enumeration enumCommands = queueCommands.elements();

		while (enumCommands.hasMoreElements()) 
        {
			Command cmd = (Command) enumCommands.nextElement();

            if (cmd instanceof ValidateUriCmd ||
                cmd instanceof UnlockResourceCmd ||
                cmd instanceof AuthenticateCmd ||
                cmd instanceof LockResourceCmd)
            {
                int nCmdReturn = cmd.getReturnCode();

                if (nCmdReturn == AuthenticationInterface.AUTH_FAILURE)
                {
                    _cmdAuthenticate.sendWWWAuthenticateHeaders(_res);
                }

                if (nCmdReturn < WebDavRetCode.WEBDAV_SUCCESS)
                {
                    return getStatusCodeImpl(nCmdReturn);
                }
            }

			else if (cmd instanceof SetPropCmd ||
				     cmd instanceof DeletePropCmd) 
            {
				PropCmd propCmd = (PropCmd) cmd;

				int nHttpStatusCode = 
                    propCmd.getStatusCodeImpl(propCmd.getInternalStatusCode());

				elemMultiStatus.addResponse(	propCmd.getHref(),
												propCmd.getOutput(),
												nHttpStatusCode);
			}
            else
            {
                Assert.assert(false);
            }
		}

		_res.setContentType("text/xml");

		try 
        {
			elemMultiStatus.save(_res.getOutputStream());
		} 
        catch (IOException e) 
        {
			return WebDavResponse.SC_INTERNAL_SERVER_ERROR;
		}

		return WebDavResponse.SC_MULTI_STATUS;
	}

	private Document _reqDocument = new Document(); 
	private int _returnCode = WebDavResponse.SC_MULTI_STATUS;
}