/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.executive;

import webdav.common.*;
import webdav.nsman.NamespaceInterface;	// Only deals with the iface
import java.util.NoSuchElementException;
import java.io.*;

/**
 * The PutResourceCmd command retrieves the file contents
 * of a resource Uri.  The undo behaviour is to restore
 * the original state of the resource.
 *
 * @see			Command
 * @see         NamespaceInterface
 * @author		Marc Eaddy
 * @version		2.0, 14 Dec 1997
 */
public class PutResourceCmd
	extends Command
{
	private Uri uri;
	private String strResource = new String();
	private NamespaceInterface ifaceNamespace;	// Should be static
	private InputStream streamIn;
	private	boolean bOverwrite = true;

	private ByteArrayOutputStream streamOriginalFile;
	private int nGetReturn;

	/**
	 * Create and initialize the PutResourceCmd command.
	 *
	 * @param	uri			[IN] Location of the resource file
	 * @param	streamIn	[IN] Input stream which will overwrite
	 *						the contents of the resource file
	 */
	public PutResourceCmd(Uri uri, InputStream streamIn)
		throws NoSuchElementException
	{
		super();

		// Copy the input parameters
		this.uri = new Uri(uri);
		this.streamIn = streamIn;

		//*** Connect to the Namespace Manager ***

		ifaceNamespace = (NamespaceInterface) 
			SubsystemRegistry.lookup( "Namespace", uri );
	}

	/**
	 * See Command documentation.
	 */
	protected int DoImpl()
	{
		streamOriginalFile = new ByteArrayOutputStream();

		// Retrieve the original file in case we need to undo
		nGetReturn = ifaceNamespace.getResource(	uri, 
													streamOriginalFile);
		if (nGetReturn < 0 && 
			nGetReturn != ifaceNamespace.NAMESPC_URI_NOT_FOUND)
		{
			return ifaceNamespace.NAMESPC_INTERNAL_ERROR;
		}

		return ifaceNamespace.putResource(uri, streamIn, bOverwrite);
	}

	/**
	 * See Command documentation.
	 */
	protected int UnDoImpl()
	{ 
		if (nGetReturn != ifaceNamespace.NAMESPC_URI_NOT_FOUND)
		{
			ByteArrayInputStream streamOriginal = new
				ByteArrayInputStream(streamOriginalFile.toByteArray());

			return ifaceNamespace.putResource(uri, streamOriginal, true);
		}
		else
		{
			return ifaceNamespace.deleteResource(uri);
		}
	}

	public static void main(String[] args)
	{
//		SubsystemRegistry jack = new SubsystemRegistry();

		Uri uri = new Uri("/test.out");

		System.out.println(uri.getFullPath());

		StringBufferInputStream streamIn = 
			new StringBufferInputStream("This is a new file 8!");

		ByteArrayOutputStream streamOut = new ByteArrayOutputStream();

		PutResourceCmd cmdPut = new PutResourceCmd(uri, streamIn);
		GetResourceCmd cmdGet = new GetResourceCmd(uri, streamOut);

		cmdGet.Do();

		System.out.println("Before: " + streamOut);

		cmdPut.Do();

		streamOut.reset();		// Remove previous contents

		cmdGet.Do();

		System.out.println("After: " + streamOut);

		cmdPut.UnDo();

		streamOut.reset();		// Remove previous contents

		cmdGet.Do();

		System.out.println("Undo: " + streamOut);
	}
};
