/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.executive;

import webdav.common.*;
import webdav.lockman.*;	// Only deals with the iface
import java.util.NoSuchElementException;
import java.io.OutputStream;

/**
 * The UnlockResourceCmd command unlocks the resource.  The
 * undo behaviour is to relock the resource with the previous
 * lock (or none).
 *
 * @see			Command
 * @see         NamespaceInterface
 * @author		Marc Eaddy
 * @version		1.0, 8 Nov 1997
 */
public class UnlockResourceCmd
	extends Command
{
	private LockManIntf ifaceLockManager;	// Should be static

	private Uri uri;
	private LockManCore lockToRemove;
	private LockManCore lockResponse = null;

	/**
	 * Create and initialize the UnlockResourceCmd command.
	 *
	 * @param	uri			[IN] Location of the resource file
	 * @param	lockRequest [IN] Owner's lock request
	 */
	public UnlockResourceCmd(Uri uri, LockManCore lockToRemove)
		throws NoSuchElementException
	{
		super();

		// Copy the input parameters
		this.uri = uri;
		this.lockToRemove = lockToRemove;

		//*** Connect to the Namespace Manager ***

		// Note: This could cause a problem if the destination
		// Uri is in a different namespace manager scope.

		ifaceLockManager = (LockManIntf) 
			SubsystemRegistry.lookup( "Lock", uri );
	}

	/**
	 * See Command documentation.
	 */
	protected int DoImpl()
	{
		return ifaceLockManager.unlock(uri, lockToRemove);
	}

	/**
	 * See Command documentation.
	 */
	protected int UnDoImpl()
	{ 
		LockManRequest lockRequest = new LockManRequest(lockToRemove);

		lockResponse = new LockManCore();

		return ifaceLockManager.lock(uri, lockRequest, lockResponse);
	}

	/**
	 * If this command is undone, it will call the lock() function which
	 * will return a lock token.  If the client wants the lock token they
	 * can call this function to retrieve it.
	 *
	 * @return		null if this command did not rollback, or the lock()
	 *				function was not called, or if the lock() function
	 *				failed
	 * @return		a lock token if this command did rollback successfully
	 */
	public LockManCore getLockTokenIfUndo()
	{
		if (nNativeUndoReturnCode >= WebDavRetCode.WEBDAV_SUCCESS)
			return lockResponse;
		else
			return null;
	}

	public static final void main(String args[] )
	{ }
};