/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.lockman;

import java.util.*;
import webdav.common.Principal;
import webdav.common.Uri;

/** 
 * Represents a lock token that has been generated via a
 * call to the LockManIntf.lock() function.
 *
 * @see		LockManIntf
 * @see     LockManRequest
 * @author	Shao Rong, Marc Eaddy
 * @version 2.0
 *
 */
public class LockManCore
{
	public static final String LOCK_TOKEN_SEPARATOR = "\247";

	private Principal	m_principal;
	private Uri			m_uri;
	private String		m_strOwnerInfo;
	private String		m_strLockType;
	private String		m_strLockScope;
	private String		m_strTimeType;
	private String		m_strTimeOut;
	private String		m_strLockToken;
	private boolean		m_bRelocked = false;

	// CREATORS

	/**
	 * Convert a lock token request into a real lock token.
	 *
	 * @param	[IN] req - Lock token request
	 * @param	[IN] uri - Uri of the resource
	 */
	public LockManCore(LockManRequest req, Uri uri)
	{
		this.m_principal	= new Principal(req.getPrincipal());
		this.m_uri			= new Uri(uri);
		this.m_strOwnerInfo = new String(req.getOwnerInfo());
		this.m_strLockToken = new String("");
		this.m_strLockType	= new String(req.getLockType());
		this.m_strLockScope = new String(req.getLockScope());
		this.m_strTimeType	= new String(req.getTimeType());
		this.m_strTimeOut	= new String(req.getTimeout());
	}

	/**
	 * Convert a Lock-Token header into its internal
	 * representation.  If the Lock-Token is not found or
	 * is empty the lock token will be invalid.
	 *
	 * @param strLockToken	[IN] Lock-Token header that was 
	 *						previously returned by
	 *						LockManCore.getLockTokenString()
	 */
	public LockManCore(String strOwnerInfoHeader, String strLockToken)
	{
		if (strOwnerInfoHeader == null || strOwnerInfoHeader.equals("") ||
			strLockToken == null || strLockToken.equals(""))
		{
			return;
		}

		m_strLockToken = new String(strLockToken);

		StringTokenizer tokens = new StringTokenizer(	strLockToken, 
														LOCK_TOKEN_SEPARATOR);

		try 
		{
			m_uri			= new Uri( tokens.nextToken() );
			m_principal		= new Principal( tokens.nextToken() );
			m_strOwnerInfo	= new String( tokens.nextToken() );
			m_strLockType	= new String( tokens.nextToken() );
			m_strLockScope	= new String( tokens.nextToken() );
		}
		catch (NoSuchElementException e)
		{
			System.out.println ("Lock-Token has bad format: " + strLockToken);
		}

		// Make sure the owner info header says that the requester
		// of the lock token is the same as the owner of the lock
		// token.  If its not then invalidate this lock token
		if (!m_strOwnerInfo.equals(strOwnerInfoHeader))
		{
			m_strOwnerInfo	= strOwnerInfoHeader;
			m_principal		= new Principal(strOwnerInfoHeader);
		}
	}

	/**
	 * Creates an invalid lock token.  You must call the
	 * set functions to set the attributes to make this
	 * lock token valid.
	 */
	public LockManCore()
	{
		this.m_principal	= null;
		this.m_uri			= null;
		this.m_strOwnerInfo = "";
		this.m_strLockToken = "";
		this.m_strLockType	= "";
		this.m_strLockScope = "";
		this.m_strTimeType	= "";
		this.m_strTimeOut	= "";
	}


	// MANIPULATORS

	public void setPrincipal(Principal p)	{ m_principal    = new Principal(p);	}
	public void setResourceUri(Uri uri)		{ m_uri          = new Uri(uri);		}
	public void setRelocked(boolean b)		{ m_bRelocked    = b;					}
	public void setOwnerInfo(String str)	{ m_strOwnerInfo = new String(str);		}
	public void setLockType(String str)		{ m_strLockType  = new String(str);		}
	public void setLockScope(String str)	{ m_strLockScope = new String(str);		}
	public void setTimeType(String str)		{ m_strTimeType  = new String(str);		}
	public void setTimeout(String str)		{ m_strTimeOut   = new String(str);		}

	/**
	 * Creates the lock token header by munging together the
	 * attributes of this class.
	 */
	private void generateLockToken()
	{
		m_strLockToken =	m_uri			+ LOCK_TOKEN_SEPARATOR +
							m_principal		+ LOCK_TOKEN_SEPARATOR +
							m_strOwnerInfo	+ LOCK_TOKEN_SEPARATOR +
							m_strLockType	+ LOCK_TOKEN_SEPARATOR +
							m_strLockScope	+ LOCK_TOKEN_SEPARATOR;
	}

	/**
	 * Convert a lock token request into a real lock token.
	 *
	 * @param	[IN] req - Lock token request
	 * @param	[IN] uri - Uri of the resource
	 */
	public void set(LockManCore that)
	{
		this.m_principal	= that.m_principal;
		this.m_uri			= that.m_uri;
		this.m_strOwnerInfo = that.m_strOwnerInfo;
		this.m_strLockToken = that.m_strLockToken;
		this.m_strLockType	= that.m_strLockType;
		this.m_strLockScope = that.m_strLockScope;
		this.m_strTimeType	= that.m_strTimeType;
		this.m_strTimeOut	= that.m_strTimeOut;
	}

	// ACCESSORS

	public Principal	getPrincipal()		{ return m_principal;	}
	public Uri			getResourceUri()	{ return m_uri;			}
	public String		getOwnerInfo()		{ return m_strOwnerInfo;}
	public String		getLockType()		{ return m_strLockType;	}
	public String		getLockScope()		{ return m_strLockScope;}
	public String		getTimeType()		{ return m_strTimeType;	}
	public String		getTimeout()		{ return m_strTimeOut;	}

	/** 
	 * Returns true if this lock token is valid, otherwise returns
	 * false.
	 */
	public boolean isValid() 
	{ 
		generateLockToken();
		return (m_principal != null &&
				!m_principal.toString().equals("") &&
				!m_strOwnerInfo.equals("") &&
				!m_strLockToken.equals(""));
	}

	/**
	 * Returns the string to be used for the Lock-Token header
	 */
	public String getLockTokenString()
	{ 
		generateLockToken();
		return m_strLockToken;	
	}

	/**
	 * Returns true if this lock token was relocked, false if it
	 * was created.
	 */
	public boolean isRelock()
	{
		return m_bRelocked;
	}

	/**
	 * Returns a diagnostic string containing information about
	 * this lock token.
	 */
	public String toString()
	{
		generateLockToken();
		return	"Principal  = " + m_principal.toString() + "\n" +
				"Owner-Info = " + m_strOwnerInfo + "\n" +
				"Lock-Token = " + m_strLockToken + "\n" +
				"LockType   = " + m_strLockType + "\n" +
				"LockScope  = " + m_strLockScope + "\n" +
				"Relocked?  = " + m_bRelocked + "\n";
	}

	/**
	 * @param that	[IN] Can be either a LockManCore object or
	 *				a LockManRequest object.
	 */
	public boolean equals(Object that)
	{
        if (this == that) 
        {
            return true;
        }

		if (that == null)
		{
			return false;
		}

		if (that instanceof LockManCore)
		{
			LockManCore core = (LockManCore) that;
			
			generateLockToken();

			return	getLockTokenString().equals( core.getLockTokenString()) &&
					getPrincipal().equals( core.getPrincipal()) &&
					getOwnerInfo().equals( core.getOwnerInfo()) &&
					getLockType().equals(  core.getLockType() ) &&
					getLockScope().equals( core.getLockScope());
		}
		else if (that instanceof LockManRequest)
		{
			LockManRequest req = (LockManRequest) that;

			// First check the requester's currently held lock tokens to see
			// it contains the right token
			Enumeration enumTokens = req.getLockTokens().elements();
			while (enumTokens.hasMoreElements())
			{
				LockManCore lockToken = (LockManCore) enumTokens.nextElement();

				// Check to see if one of the requester's currently
				// held lock tokens is equal to this lock token.  We
				// also have to make sure that the requester is the
				// same as the owner of this lock token
				if (this.equals(lockToken) &&
					this.getOwnerInfo().equals(req.getOwnerInfo()))
				{
					return true;
				}
			}

			return	getPrincipal().equals( req.getPrincipal()) &&
					getOwnerInfo().equals( req.getOwnerInfo()) &&
					getLockType().equals(  req.getLockType() ) &&
					getLockScope().equals( req.getLockScope());
        }
		else
		{
			return false;
		}
	}

	public static Vector createLockTokensFromHeader(String strOwnerInfoHeader,
													String strLockTokenHeader)
	{
		Vector vecLockTokens = new Vector();
		LockManCore lockToken = new LockManCore(strOwnerInfoHeader,
												strLockTokenHeader);

		if (lockToken.isValid())
		{
			vecLockTokens.addElement(lockToken);
		}

		return vecLockTokens;
	}

	public static void main(String[] args)
	{
		String str = new String("/index.html.meaddywriteexclusive");

		//LockManCore lock = new LockManCore("Marc", str);
	}
}
