/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.lockman;

import webdav.common.*;
import java.util.Vector;

/** 
 * This interface defines the lock related methods used by
 * other modules of the Webdav.
 *
 * @author		Shao Rong, Marc Eaddy, Jonathan Shapiro
 * @version		1.0
 */

public interface LockManIntf 
{
	/**
	 * Perform lock discovery for a resource.
	 *
	 * @param uri			[IN] Uri of the resource
	 * @param vecLockTokens	[OUT] Will contain all the lock
	 *						tokens defined for the resource.
	 *
	 * @return		One of the following status codes:<P>
	 *				<UL>
	 *				<LI>LOCKMAN_SUCCESS
	 *				<LI>LOCKMAN_LOCK_NOT_FOUND	- Not an error!
	 *				<LI>LOCKMAN_INTERNAL_ERROR
	 *				<LI>LOCKMAN_FAILURE
	 *				</UL>
	 */
	public int getLocks (Uri uri, Vector vecLockTokens);

	/** 
	 * Lock the resource.
	 *
	 * @param uri			[IN] Uri of the resource
	 * @param lockRequest	[IN] lock request for the resource
	 * @param lockResponse	[OUT] on success, filled by the lock
	 *						method with the lock that was granted
	 *
	 * @return		One of the following status codes:<P>
	 *				<UL>
	 *				<LI>LOCKMAN_SUCCESS
	 *				<LI>LOCKMAN_LOCK_CREATED		- Success!
	 *				<LI>LOCKMAN_INTERNAL_ERROR
	 *				<LI>LOCKMAN_FAILURE
	 *				<LI>LOCKMAN_ALREADY_LOCKED
	 *				<LI>LOCKMAN_BAD_REQUEST			- Lock Token invalid
	 *				</UL>
	 */
	public int lock (Uri uri, LockManRequest lockRequest, LockManCore lockToken); 

	/**
	 * Unlock the resource.
	 *
	 * @param uri			[IN] Uri of the resource
	 * @param lockRequest	[IN] A lock token that was previously
	 *						created using lock()
	 *
	 * @return		One of the following status codes:<P>
	 *				<UL>
	 *				<LI>LOCKMAN_SUCCESS
	 *				<LI>LOCKMAN_LOCK_NOT_FOUND		- This IS an error!
	 *				<LI>LOCKMAN_NO_REMOVE_RELOCK	- Not an error!
	 *				<LI>LOCKMAN_INTERNAL_ERROR
	 *				<LI>LOCKMAN_ALREADY_LOCKED		- Requester does not
	 *												have the right lock 
	 *												token
	 *				<LI>LOCKMAN_FAILURE
	 *				<LI>LOCKMAN_BAD_REQUEST			- Lock Token invalid
	 *				</UL>
	 */
	public int unlock (Uri uri, LockManCore lockRequest);

	/** Success */
	public final static int LOCKMAN_SUCCESS				= 0;

	/** Unknown or unhandled failure */
	public final static int LOCKMAN_FAILURE				= -2000;

	/** Internal error */
	public final static int LOCKMAN_INTERNAL_ERROR		= -2001;

	/** 
	 * The resource is already locked and the requester does
	 * possess the appropriate lock token.
	 */
	public final static int LOCKMAN_ALREADY_LOCKED		= -2002;

	/** The lock token or lock request was invalid */
	public final static int LOCKMAN_BAD_REQUEST			= -2003;

	/** The lock was created for this resource */
	public final static int LOCKMAN_LOCK_CREATED		= +2000;

	/** No locks exist for this resource */
	public final static int LOCKMAN_LOCK_NOT_FOUND		= +2001;

	/** Did not remove relocked lock */
	public final static int LOCKMAN_NO_REMOVE_RELOCK	= +2002;
}
