/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.lockman;

import java.util.*;
import webdav.common.Principal;
import webdav.common.WebDavRequest;

/** 
 * This class encapsulates a request for a lock token from
 * the client which must be created even if the requester
 * already has the lock token for the resource.
 * <P>
 * When a client calls a method that can modify a resource
 * (e.g., PUT), the resource must be locked during modification
 * to prevent changes to the resource from being overwritten.
 * All locks must be associated with a particular user to allow
 * the user to perform lock discovery in order to remove locks.
 * If the user does not already own a lock for the resource,
 * the executive will take-out a temporary lock on behalf of
 * the user (i.e., the user owns it).
 * <P>
 * A LockManRequest encapsulates the many types of scenarios
 * that can occur.  When a method needs to lock a resource
 * for modification, it creates a LockManRequest and initializes
 * it with the owner information of the user and whatever
 * lock tokens the user currently has.  If the user does not
 * already own a lock on the resource, the Lock Manager will
 * use the owner information to create a lock.  If the user
 * already owns a lock, the Lock Manager will detect it by
 * examining the user's currently held locks.  If the resource
 * is already locked by another owner, the Lock Manager will
 * return an error.
 *
 * @author		Marc Eaddy
 * @version		1.0
 *
 */
public class LockManRequest
	// This class is not very well designed.  Too much functionality is duplicated
	// in the LockManCore class.  There must be a better way to encapsulate lock
	// requests.
{
	private Principal				m_principal;
	private String					m_strOwnerInfo;
	private String					m_strLockType;
	private String					m_strLockScope;
	private String					m_strTimeType;
	private String					m_strTimeOut;
	private Vector					m_vecCurrentlyHeldLockTokens = new Vector();

	// CREATORS

	/**
	 * Create a lock request using HTTP headers.  The lock
	 * will not timeout.
	 *
	 * @param strOwnerInfoHeader	[IN] If null, defaults
	 *								to Owner-Info = ""
	 * @param strLockTokenHeader	[IN] If null, defaults
	 *								to no lock tokens currently held
	 * @param strLockTypeHeader		[IN] If null, defaults
	 *								to "Write"
	 * @param strLockScopeHeader	[IN] If null, defaults
	 *								to "Exclusive"
	 */
	public LockManRequest(	String strOwnerInfoHeader,
							String strLockTokenHeader,
							String strLockTypeHeader,
							String strLockScopeHeader)
	{
		this.m_strTimeType	= "";
		this.m_strTimeOut	= "";

        System.out.println("Owner-Info: " + strOwnerInfoHeader);

		if (strOwnerInfoHeader != null)
		{
			this.m_principal	= new Principal(strOwnerInfoHeader);
			this.m_strOwnerInfo	= new String(strOwnerInfoHeader);
		}
		else
		{
			this.m_principal = null;
			this.m_strOwnerInfo = "";
		}

		if (strLockTypeHeader != null)
		{
			this.m_strLockType	= new String(strLockTypeHeader);
		}
		else
		{
			this.m_strLockType	= WebDavRequest.HEADER_LOCK_TYPE_WRITE;
		}

		if (strLockScopeHeader != null)
		{
			this.m_strLockScope = new String(strLockScopeHeader);
		}
		else
		{
			this.m_strLockScope = WebDavRequest.HEADER_LOCK_SCOPE_EXCL;
		}

		this.m_vecCurrentlyHeldLockTokens = 
			LockManCore.createLockTokensFromHeader(	strOwnerInfoHeader,
													strLockTokenHeader);
	}

	/**
	 * Convert a lock token request into a real lock token.
	 *
	 * @param that [IN] A valid lock token
	 */
	public LockManRequest(LockManCore that)
	{
		this.m_principal	= new Principal(that.getPrincipal());
		this.m_strOwnerInfo = new String(that.getOwnerInfo());
		this.m_strLockType	= new String(that.getLockType());
		this.m_strLockScope = new String(that.getLockScope());
		this.m_strTimeType	= new String(that.getTimeType());
		this.m_strTimeOut	= new String(that.getTimeout());
	}


	// MANIPULATORS

	// ACCESSORS

	public Principal getPrincipal()	{ return m_principal;		}
	public String getOwnerInfo()	{ return m_strOwnerInfo;	}
	public String getLockType()		{ return m_strLockType;		}
	public String getLockScope()	{ return m_strLockScope;	}
	public String getTimeType()		{ return m_strTimeType;		}
	public String getTimeout()		{ return m_strTimeOut;		}
	public Vector getLockTokens()	{ return m_vecCurrentlyHeldLockTokens;	}

	/**
	 * Returns true if the lock request is valid
	 *
	 * @return true if the lock request is valid, otherwise false
	 */
	public boolean isValid()
	{
		System.out.println(this.toString());

		return	(m_principal != null && 
				!m_principal.toString().equals("") &&
				!m_strOwnerInfo.equals("") &&
				!m_strLockType.equals("") &&
				!m_strLockScope.equals("") );
	}

	/**
	 * Returns a string which contains diagnostic information
	 * about the lock request.
	 *
	 * @return A diagnostic string containing the lock request.
	 */
	public String toString()
	{
		String strPrincipal = (m_principal != null ? m_principal.toString() : "");

		String str =	"LockManRequest:\n" +
						"  Principal  = " + strPrincipal + "\n" +
						"  Owner-Info = " + m_strOwnerInfo + "\n" +
						"  LockType   = " + m_strLockType + "\n" +
						"  LockScope  = " + m_strLockScope + "\n\n" +
						"  Currently held lock tokens:\n";
		Enumeration e = m_vecCurrentlyHeldLockTokens.elements();
		while (e.hasMoreElements())
		{
			LockManCore lockToken = (LockManCore) e.nextElement();
			str = str + "  " + lockToken;
		}

		return str;
	}
}
