/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.lockman;

import webdav.common.SubsystemInterface;
import webdav.common.Uri;
import java.util.Vector;
import java.util.Enumeration;
import webdav.common.SubsystemRegistry; // For testing with main()
import webdav.common.Principal;			// For testing with main()

/** 
 * This class defines an interface and a partial implementation for a
 * Lock Manager.  Although the underlying implementation of locking
 * is left to the class that extends this abstract class, there are
 * generic algorithms for each method.  This class is meant to capture
 * the semantics of the generic algorithms.
 *
 * @author		Marc Eaddy
 * @version		1.0, 14 Dec 1997
 */
public abstract class SimpleLockManager
	implements LockManIntf, SubsystemInterface
{
	public SimpleLockManager ()
	{ 
		super();
	}

	/** 
	 * Perform lock discovery to find the locks for this resource.
	 *
	 * @see LockManIntf
	 */
	public int getLocks (Uri uri, Vector vecLockManCore)
	{
		return getLocksImpl(uri, vecLockManCore);
	}

	/** 
	 * Defines the general algorithm for the lock function.  The
	 * algorithm is as follows:
	 *
	 * <UL>
	 *	<LI> Step 1: Perform lock discovery to retrieve all the locks
	 *	<LI> Step 2: If there are no locks, create a new lock and
	 *         return LOCKMAN_CREATED.
	 *	<LI> Step 3: If the requester already has a lock, return LOCKMAN_SUCCESS
	 *	<LI> Step 4: Someone else has the lock, return LOCKMAN_ALREADY_LOCKED
	 * </UL>
	 *
	 * @see LockManIntf
	 */
	public int lock (Uri uri, LockManRequest lockRequest, LockManCore lockToken)
	{
		if (uri == null || lockRequest == null || lockToken == null)
		{
			return LOCKMAN_INTERNAL_ERROR;
		}

		// Make sure the request is valid.  The request itself does not have
		// to be valid if the requester already own the lock token.
		if (!lockRequest.isValid() && lockRequest.getLockTokens().size() == 0)
		{
			return LOCKMAN_BAD_REQUEST;
		}

		Vector vecLockTokens = new Vector();

		// Step 1: Perform lock discovery
		int nGetLocks = getLocks(uri, vecLockTokens);
		if (nGetLocks < LOCKMAN_SUCCESS)
		{
			System.out.println("getLocks() returned: " + nGetLocks);
			return nGetLocks;
		}

		// Step 2: No locks were found so create one
		if (nGetLocks == LOCKMAN_LOCK_NOT_FOUND)
		{
			// The request must be valid in order to create a lock
			if (!lockRequest.isValid())
			{
				return LOCKMAN_BAD_REQUEST;
			}

			System.out.println("Creating lock...");

			return createLock(uri, lockRequest, lockToken);
		}
		
		// Step 3: See if the requester already owns the lock
		else if (isLockedByRequester(vecLockTokens, lockRequest, lockToken) 
			== LOCKMAN_SUCCESS)
		{
			lockToken.setRelocked(true);
			return LOCKMAN_SUCCESS;
		}

		// Step 4: Someone else has the lock
		else
		{
			return LOCKMAN_ALREADY_LOCKED;
		}
	}

	/** 
	 * Defines the general algorithm for the unlock function.  The
	 * algorithm is as follows:
	 *
	 * <UL>
	 *	<LI> Step 1: Perform lock discovery to retrieve all the locks
	 *	<LI> Step 2: If there are no locks to remove, return LOCKMAN_LOCK_NOT_FOUND
	 *	<LI> Step 3: Make sure we own the lock to remove. If not, return
	 *				 LOCKMAN_FAILURE
	 *	<LI> Step 4: Make sure we don't remove a lock that was relocked
	 *				 by an internal operation (internal system lock).  If
	 *				 this lock was relocked, return LOCKMAN_NO_REMOVE_RELOCK.
	 *	<LI> Step 5: Otherwise, remove the lock and return LOCKMAN_SUCCESS.
	 * </UL>
	 *
	 * @see LockManIntf
	 */
	public int unlock (Uri uri, LockManCore lockRequest)
	{
		if (uri == null || lockRequest == null)
		{
			return LOCKMAN_INTERNAL_ERROR;
		}

		if (!lockRequest.isValid())
		{
			return LOCKMAN_BAD_REQUEST;
		}

		Vector vecLockTokens = new Vector();

		// Step 1: Perform lock discovery
		int nGetLocks = getLocks(uri, vecLockTokens);
		if (nGetLocks < LOCKMAN_SUCCESS || 
			nGetLocks == LOCKMAN_LOCK_NOT_FOUND)	// Step 2: No locks to remove
		{
			return nGetLocks;
		}

		// Step 3: Make sure the requester is also the lock owner
		if (isLockedByRequester(vecLockTokens, lockRequest) != LOCKMAN_SUCCESS)
		{
			System.out.println("Could not find lock token: " + lockRequest);
			
			// Could not find a matching token.  Possibly trying to
			// illegally remove someone else's lock
			return LOCKMAN_ALREADY_LOCKED;
		}

		// Step 4: Don't remove the lock if this is a relock instead of a created lock
		if (lockRequest.isRelock())
		{
			return LOCKMAN_NO_REMOVE_RELOCK;
		}

		// Step 5: Remove the lock

		if (!vecLockTokens.removeElement(lockRequest))
		{
			return LOCKMAN_INTERNAL_ERROR;
		}

		return setLocksImpl(uri, vecLockTokens);
	}

	/**
	 * A helper function which searches a vector of lock tokens to see
	 * if it contains a lock token that matches the request token.
	 *
	 * @param vecLockTokens	[IN] Lock tokens returned from getLocks()
	 * @param lockRequest	[IN] Owner's lock request token
	 * @param lockResponse	[OUT] The matching lock token if found
	 *
	 * @return	One of the following status codes:<P>
	 *			<UL>
	 *				<LI>LOCKMAN_SUCCESS if the request token already
	 *				matches one	of the locks
	 *				<LI>LOCKMAN_FAILURE if the request token does
	 *				not match one of the locks
	 *				<LI>LOCKMAN_INTERNAL_ERROR if vecLockTokens or
	 *				lockRequest are	invalid (null)
	 *			</UL>
	 */
	static int isLockedByRequester(	Vector vecLockTokens, 
									LockManRequest lockRequest,
									LockManCore lockResponse)
	{
		if (vecLockTokens == null || lockRequest == null || lockResponse == null)
		{
			return LOCKMAN_INTERNAL_ERROR;
		}

		Enumeration enumLockTokens = vecLockTokens.elements();
		while (enumLockTokens.hasMoreElements())
		{
			LockManCore lockToken = (LockManCore) enumLockTokens.nextElement();
		
			if (lockToken.equals(lockRequest))
			{
				lockResponse.set(lockToken);
				return LOCKMAN_SUCCESS;
			}
		}

		return LOCKMAN_FAILURE;
	}

	/**
	 * A helper function that searches a vector of lock tokens to see
	 * if it contains a lock token that matches the request token.
	 *
	 * @param vecLockTokens [IN] Lock tokens returned from getLocks()
	 * @param lockRequest	[IN] Owner's lock token
	 *
	 * @return	One of the following status codes:<P>
	 *			<UL>
	 *				<LI>LOCKMAN_SUCCESS if the request token already
	 *				matches one	of the locks
	 *				<LI>LOCKMAN_FAILURE if the request token does
	 *				not match one of the locks
	 *				<LI>LOCKMAN_INTERNAL_ERROR if vecLockTokens or
	 *				lockRequest are	invalid (null)
	 *			</UL>
	 */
	static int isLockedByRequester(	Vector vecLockTokens, 
									LockManCore lockRequest)
	{
		if (vecLockTokens == null || lockRequest == null)
		{
			return LOCKMAN_INTERNAL_ERROR;
		}

		Enumeration enumLockTokens = vecLockTokens.elements();
		while (enumLockTokens.hasMoreElements())
		{
			LockManCore lockToken = (LockManCore) enumLockTokens.nextElement();

			if (lockToken.equals(lockRequest))
			{
				return LOCKMAN_SUCCESS;
			}
		}

		return LOCKMAN_FAILURE;
	}

	/**
	 * Creates a lock.
	 *
	 * @param uri			[IN] Uri of the resource.
	 * @param lockRequest	[IN] Requester's lock request
	 * @param lockResponse	[OUT] On success will contain the lock
	 *						token created
	 *
	 * @return	One of the following status codes:<P>
	 *			<UL>
	 *				<LI>LOCKMAN_LOCK_CREATED if the lock was created
	 *				<LI>LOCKMAN_INTERNAL_ERROR if the arguments are
	 *				invalid (null)
	 *				<LI>LOCKMAN_FAILURE if the lock was not created due
	 *				to an error with the Lock Manager
	 *			</UL>
	 */
	private int createLock(	Uri uri, 
							LockManRequest lockRequest, 
							LockManCore lockResponse)
	{
		if (uri == null || lockRequest == null || lockResponse == null)
		{
			return LOCKMAN_INTERNAL_ERROR;
		}


		LockManCore lockNewToken = new LockManCore(lockRequest, uri);

		Vector vecOfOneToken = new Vector();
		vecOfOneToken.addElement(lockNewToken);

		int nSetLockReturn = setLocksImpl(uri, vecOfOneToken);

		if (nSetLockReturn < LOCKMAN_SUCCESS)
		{
			return nSetLockReturn;
		}

		lockResponse.set(lockNewToken);		// Set the response to the newly created token
		lockResponse.setRelocked(false);	// The lock was just created - not a relock

		return LOCKMAN_LOCK_CREATED;
	}

	/** Subclasses must implement this */
	protected abstract int getLocksImpl(Uri uri, Vector vecLockManCore);

	/** Subclasses must implement this */
	protected abstract int setLocksImpl(Uri uri, Vector vecLockTokens);

	public static void main(String args[] )
	{
		Uri uri = new Uri("/index.html");
		Uri uri2 = new Uri("/index2.html");

		System.out.println("Uri=" + uri.getFullPath());

		new SubsystemRegistry();

		LockManIntf lockman = new SimpleLockManagerImpl();

		LockManRequest req = new LockManRequest("meaddy@newscorp.com",
												null,
												"Write",
												"Exclusive");

		LockManCore lockToken = new LockManCore();

		System.out.println("Creating a lock on '" + uri2 + "' with request:\n" + req);

		// Create a lock - should succeed

		int nLockRet = lockman.lock(uri2, req, lockToken);

		if (nLockRet != LOCKMAN_LOCK_CREATED)
		{
			System.out.println("ERROR: Lock was not created!!!");
			return;
		}

		System.out.println("Lock Token returned:\n" + lockToken);

		// Create same lock - should succeed

		nLockRet = lockman.lock(uri2, req, lockToken);

		if (nLockRet != LOCKMAN_SUCCESS)
		{
			System.out.println("ERROR: Lock was verified to be equal!!!");
			return;
		}

		System.out.println("Lock Token returned:\n" + lockToken);

		// Attempt to create different lock - should fail
		LockManRequest req2 = new LockManRequest("jshapiro@newscorp.com",
												null,
												"Write",
												"Exclusive");
		nLockRet = lockman.lock(uri2, req2, lockToken);

		if (nLockRet != LOCKMAN_ALREADY_LOCKED)
		{
			System.out.println("ERROR: Lock was incorrectly set!!! Return=" + nLockRet);
			return;
		}
		System.out.println("Lock Token returned:\n" + lockToken);

		// Attempt to unlock - should succeed with info
		lockToken.setRelocked(true);
		nLockRet = lockman.unlock(uri2, lockToken);

		if (nLockRet != LOCKMAN_NO_REMOVE_RELOCK)
		{
			System.out.println("ERROR: Lock was not unlocked!!! Return=" + nLockRet);
			return;
		}

		// Attempt to unlock - should succeed
		lockToken.setRelocked(false);
		nLockRet = lockman.unlock(uri2, lockToken);

		if (nLockRet != LOCKMAN_SUCCESS)
		{
			System.out.println("ERROR: Lock was not unlocked!!! Return=" + nLockRet);
			return;
		}

		// Attempt to unlock - should fail
		lockToken.setRelocked(false);
		nLockRet = lockman.unlock(uri2, lockToken);

		if (nLockRet != LOCKMAN_LOCK_NOT_FOUND)
		{
			System.out.println("ERROR: Lock was not unlocked!!! Return=" + nLockRet);
			return;
		}
	}
}

