/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.lockman;

// I listed each of these required classes explicitly so that
// we could see all the dependencies of this class.

import webdav.common.Principal;
import webdav.common.WebDavRequest;
import webdav.common.WebDavRetCode;
import webdav.common.Uri;
import webdav.common.WebdavXML;
import webdav.executive.GetPropCmd;
import webdav.executive.SetPropCmd;
import webdav.executive.DeletePropCmd;
import com.ms.xml.util.Name;
import com.ms.xml.om.Element;
import com.ms.xml.om.ElementImpl;
import java.util.Vector;
import java.util.Enumeration;
import webdav.common.SubsystemRegistry; // For testing with main()
import com.ms.xml.om.Document;			// For testing with main()
import com.ms.xml.util.XMLOutputStream;	// For testing with main()
import java.io.IOException;				// For testing with main()
import webdav.propman.PropertyManager;	// Error codes should be in PropertySubsystem

/** 
 * Provides a concrete implementation of the SimpleLockManager
 * which uses the PropertyManager to store lock properties.
 *
 * @author		Marc Eaddy
 * @version		1.0
 */
public class SimpleLockManagerImpl 
	extends SimpleLockManager
{
	public SimpleLockManagerImpl ()
	{ 
		super();
	}

	/**
	 * Implements the getLocks() function by retrieving the lock
	 * properties that have been defined on the resource and then
	 * converting those lock properties into lock tokens.
	 *
	 * @param uri			[IN] The uri of the resource
	 * @param vecLockTokens [OUT] All the lock tokens for the resource
	 *
	 * @return	One of the following status codes:<P>
	 *			<UL>
	 *				<LI>LOCKMAN_INTERNAL_ERROR if the arguments are
	 *				invalid (null)
	 *				<LI>LOCKMAN_FAILURE if the property subsystem
	 *				returns an error when trying to fetch the
	 *				properties.
	 *				<LI>LOCKMAN_LOCK_NOT_FOUND if the properties 
	 *				returned by the property subsystem do not
	 *				contian the proper lock property tree.
	 *				<LI>LOCKMAN_SUCCESS on success
	 *			</UL>
	 */
	protected int getLocksImpl (Uri uri, Vector vecLockTokens)
	{
		if (uri == null || vecLockTokens == null)
		{
			System.out.println("uri=" + (uri==null?"null":uri.toString()) + " vec=" +
				(vecLockTokens==null?"null": (new Integer(vecLockTokens.size())).toString()));
			return LOCKMAN_INTERNAL_ERROR;
		}

		vecLockTokens.removeAllElements();		// Reset the vector

		GetPropCmd cmdGetLockProp = new GetPropCmd(uri, ELEM_LOCKS);

		int nGetLockReturn = cmdGetLockProp.Do();

		// If the lock property is not found, don't treat it as an error
		if (nGetLockReturn < WebDavRetCode.WEBDAV_SUCCESS &&
			nGetLockReturn != PropertyManager.pm_err_propnotfound)
		{
			// Should transform this to LOCKMAN_ error code
			System.out.println("cmdGetLockProp.Do() returned " + nGetLockReturn);
			return LOCKMAN_FAILURE;
		}

		// Retrieve the Element created by the get property command
		Element elemLockProps = cmdGetLockProp.getOutput();

		// Convert the 
		return element2tokens(elemLockProps, uri, vecLockTokens);
	}

	/**
	 * Implements the setLocks() function by converting lock tokens
	 * to lock property elements and then setting that lock property
	 * on the resource.  If the no lock tokens are specified (the
	 * vector of lock tokens is empty), the lock property is removed
	 * from the resource.
	 *
	 * @param uri			[IN] The Uri of the resource
	 * @param vecLockTokens [IN] The lock tokens to set on the
	 *						resource
	 *
	 * @return	One of the following status codes:<P>
	 *			<UL>
	 *				<LI>LOCKMAN_INTERNAL_ERROR if the arguments
	 *				are invalid (null)
	 *				<LI>LOCKMAN_FAILURE if the property subsystem
	 *				returns an error when trying to set the properties.
	 *				<LI>LOCKMAN_SUCCESS on success
	 *			</UL>
	 */
	protected int setLocksImpl(Uri uri, Vector vecLockTokens)
	{
		if (uri == null || vecLockTokens == null)
		{
			return LOCKMAN_INTERNAL_ERROR;
		}

		// Create a base property element
		ElementImpl elemLock = new ElementImpl(	WebdavXML.ELEM_PROP, 
												Element.ELEMENT);

		// Convert the lock tokens into a lock property element
		int nConvertReturn = tokens2element(vecLockTokens, elemLock);

		if (nConvertReturn < LOCKMAN_SUCCESS)
		{
			return nConvertReturn;
		}

		// If there were one or more tokens then set the lock property
		if (vecLockTokens.size() > 0)
		{
			SetPropCmd cmdSetLockProp = new SetPropCmd(uri, elemLock);

			int nSetLockReturn = cmdSetLockProp.Do();

			if (nSetLockReturn < WebDavRetCode.WEBDAV_SUCCESS)
			{
				System.out.println("Error returned by SetPropCmd=" + nSetLockReturn);
				return LOCKMAN_FAILURE;
			}
		}

		// Otherwise, remove the lock property entirely
		else
		{
			DeletePropCmd cmdDeleteLockProp = new DeletePropCmd(uri, elemLock);

			int nDeleteLockReturn = cmdDeleteLockProp.Do();

			if (nDeleteLockReturn < WebDavRetCode.WEBDAV_SUCCESS)
			{
				System.out.println("Error returned by DeletePropCmd=" + nDeleteLockReturn);
				return LOCKMAN_FAILURE;
			}
		}

		return LOCKMAN_SUCCESS;
	}

	/**
	 * Converts a lock property element into zero or more lock tokens.
	 *
	 * @param	elemLockProps	[IN] contains an element tree with
	 *			one or more locks.  The element tree should look like:
	 *
	 *			<PRE>
	 *          &lt;_internal_locks&gt;
	 *              &lt;lock&gt;
	 *                  &lt;Owner-Info&gt;meaddy@newscorp.com&lt;/Owner-Info&gt;
	 *                  &lt;Principal&gt;meaddy&lt;/Principal&gt;
	 *                  &lt;LockType&gt;Write&lt;/LockType&gt;
	 *                  &lt;LockScope&gt;Shared&lt;/LockScope&gt;
	 *              &lt;/lock&gt;
	 *              &lt;lock&gt;
	 *                  &lt;Owner-Info&gt;jshapiro@newscorp.com&lt;/Owner-Info&gt;
	 *                  &lt;Principal&gt;jshapiro&lt;/Principal&gt;
	 *                  &lt;LockType&gt;Write&lt;/LockType&gt;
	 *                  &lt;LockScope&gt;Shared&lt;/LockScope&gt;
	 *              &lt;/lock&gt;
	 *          &lt;/_internal_locks&gt;
	 *			</PRE>
	 *
	 * @param	uri				[IN] Uri of the resource
	 * @param	vecLockTokens	[OUT] contains the lock tokens converted
	 *			from the lock property element.  For now, this will only
	 *			be one token no matter how many locks are defined in the
	 *			lock property element.
	 *
	 * @return	One of the following status codes:<P>
	 *			<UL>
	 *				<LI>LOCKMAN_INTERNAL_ERROR if the args are
	 *				invalid (null) or if a lock token created
	 *				is not valid.
	 *				<LI>LOCKMAN_LOCK_NOT_FOUND if the lock
	 *				property element does not contain the
	 *				_internal_locks or the lock elements.
	 *				Maybe this should be an error.
	 *				<LI>LOCKMAN_SUCCESS on success
	 *			</UL>
	 */
	private static int element2tokens(	Element elemLockProps,
										Uri uri,
										Vector vecLockTokens)
	{
		if (vecLockTokens == null || uri == null || elemLockProps == null)
		{
			System.out.println("uri=" + (uri==null?"null":uri.toString()) + " vec=" +
				(vecLockTokens==null?"null": (new Integer(vecLockTokens.size())).toString()) +
				" elem=" + (elemLockProps==null?"null":"non-null"));
			return LOCKMAN_INTERNAL_ERROR;
		}

		vecLockTokens.removeAllElements();

		Element elemInternalLocks = findElementOnce(elemLockProps, ELEM_LOCKS);

		if (elemInternalLocks == null)
		{
			return LOCKMAN_LOCK_NOT_FOUND;
		}

		Element elemLock = findElementOnce(elemInternalLocks, ELEM_LOCK);
		
		if (elemLock == null)
		{
			return LOCKMAN_LOCK_NOT_FOUND;
		}

		// Create an initialize a new lock token
		LockManCore lockNewToken = new LockManCore();
		lockNewToken.setResourceUri(uri);

		// Iterate over all the lock info for this lock
		Enumeration enumLockInfo = elemLock.getElements();
		while (enumLockInfo.hasMoreElements())
		{
			Element elemLockInfo = (Element) enumLockInfo.nextElement();
			Name nameLockInfo = elemLockInfo.getTagName();

			if (nameLockInfo == null) continue;	// Skip null elements

			String strValue = elemLockInfo.getText();
			System.out.println("    " + nameLockInfo + " = " + strValue);

			// Set the corresponding lock token attributes using the
			// lock info properties

			if (nameLockInfo.equals(ELEM_OWNER_INFO))
			{
				lockNewToken.setOwnerInfo(strValue);
			}
			else if (nameLockInfo.equals(ELEM_PRINCIPAL))
			{
				lockNewToken.setPrincipal(new Principal(strValue));
			}
			else if (nameLockInfo.equals(ELEM_LOCK_TYPE))
			{
				lockNewToken.setLockType(strValue);
			}
			else if (nameLockInfo.equals(ELEM_LOCK_SCOPE))
			{
				lockNewToken.setLockScope(strValue);
			}
			else
			{
				// Unknown lock information tag...
				System.out.println("  Unrecognized lock property: " + nameLockInfo);
			}
		}

		// If the token is valid, add it to the list of tokens
		if (lockNewToken.isValid())
		{
			vecLockTokens.addElement(lockNewToken);
		}
		else
		{
			System.out.println("Lock Token is invalid: " + lockNewToken);

			return LOCKMAN_INTERNAL_ERROR;
		}

		return LOCKMAN_SUCCESS;
	}

	/**
	 * Converts zero or more lock tokens into a lock property element
	 *
	 * @param	vecLockTokens [IN] Contains zero or more lock tokens
	 * @param	elemLockProps [OUT] On Success, will contain an element
	 *			tree with one or more locks.  The element tree should look
	 *			like:
	 *
	 *			<PRE>
	 *          &lt;_internal_locks&gt;
	 *              &lt;lock&gt;
	 *                  &lt;Owner-Info&gt;meaddy@newscorp.com&lt;/Owner-Info&gt;
	 *                  &lt;Principal&gt;meaddy&lt;/Principal&gt;
	 *                  &lt;LockType&gt;Write&lt;/LockType&gt;
	 *                  &lt;LockScope&gt;Shared&lt;/LockScope&gt;
	 *              &lt;/lock&gt;
	 *              &lt;lock&gt;
	 *                  &lt;Owner-Info&gt;jshapiro@newscorp.com&lt;/Owner-Info&gt;
	 *                  &lt;Principal&gt;jshapiro&lt;/Principal&gt;
	 *                  &lt;LockType&gt;Write&lt;/LockType&gt;
	 *                  &lt;LockScope&gt;Shared&lt;/LockScope&gt;
	 *              &lt;/lock&gt;
	 *          &lt;/_internal_locks&gt;
	 *			</PRE>
	 *
	 * @return	One of the following status codes:<P>
	 *			<UL>
	 *				<LI>LOCKMAN_INTERNAL_ERROR if the args are
	 *				invalid (null) or if a lock token created
	 *				is not valid.
	 *				<LI>LOCKMAN_LOCK_NOT_FOUND if the lock
	 *				property element does not contain the 
	 *				_internal_locks or the lock elements.
	 *				Maybe this should be an error.
	 *				<LI>LOCKMAN_SUCCESS on success
	 *			</UL>
	 */
	private static int tokens2element(	Vector vecLockTokens,
										Element elemLockProps)
	{
		if (vecLockTokens == null || elemLockProps == null)
		{
			return LOCKMAN_INTERNAL_ERROR;
		}

		ElementImpl elemInternalLocks = new ElementImpl(ELEM_LOCKS, Element.ELEMENT);

		Enumeration enumLockTokens = vecLockTokens.elements();
		while (enumLockTokens.hasMoreElements())
		{
			LockManCore lockToken = (LockManCore) enumLockTokens.nextElement();

			ElementImpl elemPrincipal	= new ElementImpl(ELEM_PRINCIPAL, Element.ELEMENT);
			ElementImpl valPrincipal	= new ElementImpl(null, Element.PCDATA);
			valPrincipal.setText(lockToken.getPrincipal().toString());
			elemPrincipal.addChild(valPrincipal, null);

			ElementImpl elemOwnerInfo	= new ElementImpl(ELEM_OWNER_INFO, Element.ELEMENT);
			ElementImpl valOwnerInfo	= new ElementImpl(null, Element.PCDATA);
			valOwnerInfo.setText(lockToken.getOwnerInfo());
			elemOwnerInfo.addChild(valOwnerInfo, null);

			ElementImpl elemLockType	= new ElementImpl(ELEM_LOCK_TYPE, Element.ELEMENT);
			ElementImpl valLockType		= new ElementImpl(null, Element.PCDATA);
			valLockType.setText(lockToken.getLockType());
			elemLockType.addChild(valLockType, null);

			ElementImpl elemLockScope	= new ElementImpl(ELEM_LOCK_SCOPE, Element.ELEMENT);
			ElementImpl valLockScope	= new ElementImpl(null, Element.PCDATA);
			valLockScope.setText(lockToken.getLockScope());
			elemLockScope.addChild(valLockScope, null);

			ElementImpl elemLock		= new ElementImpl(ELEM_LOCK, Element.ELEMENT);

			elemLock.addChild(elemPrincipal, null);
			elemLock.addChild(elemOwnerInfo, null);
			elemLock.addChild(elemLockType,  null);
			elemLock.addChild(elemLockScope, null);

			elemInternalLocks.addChild(elemLock, null);
		}

		elemLockProps.addChild(elemInternalLocks, null);

		return LOCKMAN_SUCCESS;
	}

	/**
	 * Helper function to find an element in the first level of an
	 * element tree.  Helps to avoid using multiply-nested while
	 * loops.
	 *
	 * @param		elem [IN] Root of the Element tree
	 * @param		name [IN] Name to look for in the first level
	 *				of the Element tree
	 * @return		The element found with the name specified or null
	 */
	private static Element findElementOnce(Element elem, Name name)
	{
		Enumeration enumElements = elem.getElements();
		while (enumElements.hasMoreElements())
		{
			Element elemCurrent = (Element) enumElements.nextElement();
			Name nameElement = elemCurrent.getTagName();

			if (nameElement == null)
			{
				continue;
			}

			System.out.println(nameElement);

			if (nameElement.equals(name))
			{
				return elemCurrent;
			}
		}

		return null;
	}

	public static void main(String args[] )
	{
		Uri uri = new Uri("/index.html");

		System.out.println("Uri=" + uri.getFullPath());

		new SubsystemRegistry();

		Vector vec = new Vector();
		SimpleLockManager lockman = new SimpleLockManagerImpl();

		lockman.getLocks(uri, vec);

		Enumeration enumTokens = vec.elements();
		while (enumTokens.hasMoreElements())
		{
			LockManCore lockToken = (LockManCore) enumTokens.nextElement();
			System.out.println(lockToken);
		}

		ElementImpl elemLocks = new ElementImpl(WebdavXML.ELEM_PROP, Element.ELEMENT);

		SimpleLockManagerImpl.tokens2element(vec, elemLocks);

		XMLOutputStream xml_out = new XMLOutputStream(System.out);

		try
		{
			Document doc = new Document();
			doc.addChild(elemLocks, null);
			doc.save(xml_out);
		}
		catch(IOException ex)
		{ }
	}

	private static final Name ELEM_LOCKS = Name.create("_internal_locks");
	private static final Name ELEM_LOCK = Name.create("lock");
	private static final Name ELEM_OWNER_INFO = Name.create("Owner-Info");
	private static final Name ELEM_PRINCIPAL = Name.create("Principal");
	private static final Name ELEM_LOCK_TYPE = Name.create(WebDavRequest.HEADER_LOCK_TYPE);
	private static final Name ELEM_LOCK_SCOPE = Name.create(WebDavRequest.HEADER_LOCK_SCOPE);

	// For runtime debugging using main()
/*
	private static final Name ELEM_LOCKS = Name.create(("_internal_locks").toUpperCase());
	private static final Name ELEM_LOCK = Name.create(("lock").toUpperCase());
	private static final Name ELEM_OWNER_INFO = Name.create(("Owner-Info").toUpperCase());
	private static final Name ELEM_PRINCIPAL = Name.create(("Principal").toUpperCase());
	private static final Name ELEM_LOCK_TYPE = Name.create((WebDavRequest.HEADER_LOCK_TYPE).toUpperCase());
	private static final Name ELEM_LOCK_SCOPE = Name.create((WebDavRequest.HEADER_LOCK_SCOPE).toUpperCase());
*/	
}

