/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.lockman;

import java.util.*;
import java.io.*;
import webdav.common.*;

/** 
  * This class implement the lock related methods. 
 * @author Shao Rong
 * @version 1.0
 */
public class LockMan implements LockManIntf, LockManErrorCode, SubsystemInterface  {
  final static String LockMan_dbName = "lockman_db";
  final static int MAX_LOCKS = 1;
  String d_lockMan_dbName;
  LockManDBM d_lockManDBM;

  public LockMan () {
    super ();
    d_lockMan_dbName = new String (LockMan_dbName);
    d_lockManDBM = new LockManDBM();
    d_lockManDBM.db_open (d_lockMan_dbName);
  }

  public LockMan (String dbName) {
    super ();
    d_lockMan_dbName = new String (dbName);
    d_lockManDBM = new LockManDBM();
    d_lockManDBM.db_open (d_lockMan_dbName);
  }

  public void finalize () {
    d_lockManDBM.db_close (d_lockMan_dbName);
  }  

  /** @see LockManIntf */
  public int isLocked (Uri uri, String lockType, String lockScope, LockManCore lockRec) {
    int rc ;
    StringBuffer lockInfo = new StringBuffer ();

    // only implement one lock situiation
    rc = d_lockManDBM.db_read (uri.toString()+lockType+lockScope, lockInfo); 
    if (rc != LockMan_SUCCESS) {
      if (rc == DB_REC_NOT_FOUND) return NotLocked;
      if (debug) System.out.println ("isLocked failed at db read");
      return rc;  
    }
    rc = lockRec.toLockManCore (lockInfo.toString ());
    if (rc != LockMan_SUCCESS) {
      if (debug) System.out.println ("isLocked failed at decompose a lock rec");
      return rc;
    }

    return IsLocked;
  }

  /** @see LockManIntf */
  public int isLocked (Uri uri, String lockType, String lockScope) {
    int rc ;
    StringBuffer lockInfo = new StringBuffer ();

    // only implement one lock situiation
    rc = d_lockManDBM.db_read (uri.toString()+lockType+lockScope, lockInfo); 
    if (rc != LockMan_SUCCESS) {
      if (rc == DB_REC_NOT_FOUND) return NotLocked;
      if (debug) System.out.println ("isLocked failed at db read");
      return rc;
    }
    return IsLocked;  
  }

  /** @see LockManIntf */
  public int lock (LockManCore requestToken, LockManCore responseToken) {

    Uri uri = new Uri (requestToken.getLockURI ());
    String lockType = new String (requestToken.getLockType ());
    String lockScope = new String (requestToken.getLockScope ());
    LockUID lockID = new LockUID ();

    int rc;
    
    responseToken.setLockURI (requestToken.getLockURI());
    responseToken.setRequester (requestToken.getRequester());
    responseToken.setLockType (requestToken.getLockType());
    responseToken.setLockScope (requestToken.getLockScope());
    responseToken.setOwnerInfo (requestToken.getOwnerInfo());
    responseToken.setTimeType (requestToken.getTimeType());
    responseToken.setTimeOutVal (requestToken.getTimeOutVal());
    responseToken.setLockId (lockID.getID (uri,lockType,lockScope));

    rc = d_lockManDBM.db_write (uri+lockType+lockScope, responseToken.to_string(), LockManDBM.LOCKMAN_DB_UPDATE);

    if (rc != LockMan_SUCCESS) {
      if (debug) System.out.println ("lock failed at db write");
      return rc;
    }

    return LOCK_SUCCESS;
  }

  /** @see LockManIntf */
  public int unlock (LockManCore requestToken) {

    Uri uri = new Uri (requestToken.getLockURI ());
    String lockType = new String (requestToken.getLockType ());
    String lockScope = new String (requestToken.getLockScope ());
    String lockId = new String (requestToken.getLockId ());
    LockManCore lockCore = new LockManCore ();
    int rc;

    if ((rc=isLocked(uri, lockType, lockScope, lockCore)) == NotLocked) return UNLOCK_SUCCESS;
    else 
    if (rc != IsLocked) {
      return rc;
    }

    // check if state token match
    if (!lockId.equals (lockCore.getLockId ())) return ACCESS_VIOLATION;
    
    String key = new String (uri+lockType+lockScope);
    rc = d_lockManDBM.db_write (key, requestToken.to_string(), LockManDBM.LOCKMAN_DB_DELETE);

    if (rc != LockMan_SUCCESS) {
      if (debug) System.out.println ("lock failed at db write");
      return rc;
    }

    return UNLOCK_SUCCESS;

  }

  public final static void main (String args[] ) {
    LockMan lockMan = new LockMan ("test");
    Uri uri = new Uri ("www.cs.columbia.edu");
    String lockType = new String ("write");
    String lockScope = new String ("exclusive");
    LockManCore  lockCore = new LockManCore ();
    LockManCore  rspLockCore = new LockManCore ();

    lockCore.setLockURI (uri);
    lockCore.setRequester ("eric");
    lockCore.setOwnerInfo ("new york");
    lockCore.setTimeType ("infinite");
    lockCore.setTimeOutVal ("0");

    if (lockMan.isLocked (uri, lockType, lockScope) == IsLocked) {
      System.out.println ("uri " + uri.toString());
      System.out.println ("LockType " + lockType);
      System.out.println ("LockScope " + lockScope);
      System.out.println ("Is locked ");
    }
    else {
      System.out.println ("uri " + uri.toString());
      System.out.println ("LockType " + lockType);
      System.out.println ("LockScope " + lockScope);
      System.out.println ("Is not locked ");
    }
    if (lockMan.isLocked (uri, lockType, lockScope, rspLockCore) == IsLocked) {
      System.out.println ("uri " + uri.toString());
      System.out.println ("LockType " + lockType);
      System.out.println ("LockScope " + lockScope);
      System.out.println ("Is locked ");
      System.out.println ("and the lock rec is ");
      System.out.println (rspLockCore.to_string());
    }
    else {
      System.out.println ("uri " + uri.toString());
      System.out.println ("LockType " + lockType);
      System.out.println ("LockScope " + lockScope);
      System.out.println ("Is not locked ");
    }

    if (lockMan.lock (lockCore, rspLockCore) == LOCK_SUCCESS) {
      System.out.println ("Lock response : ");
      System.out.println (rspLockCore.to_string());
    }
    else {
      System.out.println ("Lock failed ");
    }
    if (lockMan.isLocked (uri, lockType, lockScope) == IsLocked) {
      System.out.println ("uri " + uri.toString());
      System.out.println ("LockType " + lockType);
      System.out.println ("LockScope " + lockScope);
      System.out.println ("Is locked ");
    }
    else {
      System.out.println ("uri " + uri.toString());
      System.out.println ("LockType " + lockType);
      System.out.println ("LockScope " + lockScope);
      System.out.println ("Is not locked ");
    }
    if (lockMan.isLocked (uri, lockType, lockScope, rspLockCore) == IsLocked) {
      System.out.println ("uri " + uri.toString());
      System.out.println ("LockType " + lockType);
      System.out.println ("LockScope " + lockScope);
      System.out.println ("Is locked ");
      System.out.println ("and the lock rec is ");
      System.out.println (rspLockCore.to_string());
    }
    else {
      System.out.println ("uri " + uri.toString());
      System.out.println ("LockType " + lockType);
      System.out.println ("LockScope " + lockScope);
      System.out.println ("Is not locked ");
    }

    if (lockMan.unlock (rspLockCore) != UNLOCK_SUCCESS) {
      System.out.println ("Unlock failed ");
    }

    if (lockMan.isLocked (uri, lockType, lockScope) == IsLocked) {
      System.out.println ("uri " + uri.toString());
      System.out.println ("LockType " + lockType);
      System.out.println ("LockScope " + lockScope);
      System.out.println ("Is locked ");
    }
    else {
      System.out.println ("uri " + uri.toString());
      System.out.println ("LockType " + lockType);
      System.out.println ("LockScope " + lockScope);
      System.out.println ("Is not locked ");
    }
    if (lockMan.isLocked (uri, lockType, lockScope, rspLockCore) == IsLocked) {
      System.out.println ("uri " + uri.toString());
      System.out.println ("LockType " + lockType);
      System.out.println ("LockScope " + lockScope);
      System.out.println ("Is locked ");
      System.out.println ("and the lock rec is ");
      System.out.println (rspLockCore.to_string());
    }
    else {
      System.out.println ("uri " + uri.toString());
      System.out.println ("LockType " + lockType);
      System.out.println ("LockScope " + lockScope);
      System.out.println ("Is not locked ");
    }
  }

}

