/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.nsman;

import webdav.common.Uri;
import webdav.common.UriHelper;
import webdav.common.WebdavXML;
import webdav.common.WebDavResponse;
import java.io.File;
import java.util.Date;
import com.ms.xml.om.Element;
import com.ms.xml.om.ElementImpl;

/**
 * A WebDAV collection contains CollectionMembers which are very
 * similar to Uri except that they also contain information about
 * the CollectionMember including size, last modified date,
 * whether or not the CollectionMember is a collection (i.e., 
 * folder) or a member (i.e., file), and whether or not the
 * CollectionMember refers to an external or internal resource.
 * <P>
 * Some helper functions have been included to help with
 * displaying the CollectionMember as HTML or as an XML Element.
 *
 * @author		Marc Eaddy
 * @version		1.0, 18 Dec 1997
 */
public class CollectionMember
{
	// DATA

	/** Uri of the collection member */
	private Uri			uri				= null;

	/** Whether or not this member is also a collection */
	private boolean		bIsCollection	= false;

	/** 
	 * Whether or not this memeber refers to an external
	 * or internal resource.
	 */
	private boolean		bIsExternal		= false;

	/** Size of this collection member */
	private long		nItemSize		= 0;

	/** Last modified date for this collection member */
	private Date		date			= new Date();

	/** Used internally */
	private static String strSpaces = "                                 ";

	// CREATORS

	/**
	 * Creates a collection member based on a uri.
	 *
	 * @param uri	[IN] Uri of the collection member
	 */
	public CollectionMember(Uri uri)
	{
		this.uri = new Uri(uri);

		initAttributes(new File(uri.getFullPath()));
	}

	/**
	 * Creates a collection member based on a string uri.
	 *
	 * @param strUri	[IN] String uri of the collection
	 *					member.  Should be obtained from
	 *					Uri.toString().
	 */
	public CollectionMember(String strUri)
	{
		this.uri = new Uri(strUri);
		initAttributes(new File(strUri));
	}

	/**
	 * Creates a collection member based on a file.
	 *
	 * @param file		[IN] File associated with a
	 *					resource.
	 */
	public CollectionMember(File file)
	{
		this.uri = new Uri(file);
		initAttributes(file);
	}

	// MANIPULATORS

	/**
	 * Initializes the CollectionMember.
	 *
	 * @param file	[IN] File associated with the Collection
	 *				member.
	 */
	private void initAttributes(File file)
	{
		if (file == null)
		{
			return;
		}

		bIsCollection = file.isDirectory();
		bIsExternal = false;	// Default
		nItemSize = file.length();

		if (!bIsCollection)
		{
			date = new Date(file.lastModified());
		}
	}

	// ACCESSORS

	/**
	 * @return The size of the CollectionMember in bytes (long).
	 */
	public long getItemSize()
	{
		return nItemSize;
	}

	/**
	 * @return	TRUE if the CollectionMember refers to another
	 *			collection.  Otherwise, FALSE.
	 */
	public boolean isCollection()
	{
		return bIsCollection;
	}

	/**
	 * @return	TRUE if the CollectionMember refers to an
	 *			external resource.  FALSE if it refers to
	 *			an internal resource.
	 */
	public boolean isExternal()
	{
		return bIsExternal;
	}

	/**
	 * @return	The Uri representation of this CollectionMember.
	 */
	public Uri uri()
	{
		return uri;
	}

	/**
	 * @return	Returns an HTML image tag which is different if
	 *			the CollectionMember is a collection or a member
	 */
	private String getHTMLPrefixTag()
	{
		if (bIsCollection)
		{
			return "<IMG SRC=\"/icons/collection.gif\" ALT=\"[COLLECTION]\">";
		}
		else
		{
			return "<IMG SRC=\"/icons/member.gif\" ALT=\"[MEMBER]\">";
		}
	}

	/**
	 * @return	Returns the local date of this CollectionMember.
	 */
	private String getHTMLDate()
	{
		return date.toLocaleString();
	}

	/**
	 * @return	Returns the size of this CollectionMember in
	 *			bytes.
	 */
	private String getHTMLSize()
	{
		return Long.toString(nItemSize);
	}

	/**
	 * Creates an HTML line describing this CollectionMember.
	 * The HTML is meant to be used to display to the user when
	 * the user performs a GET on a directory (collection).  The
	 * directory listing will contain multiple CollectionMember
	 * entries.
	 * <P>
	 * For example:<BR>
	 * <PRE>
	 * &lt;IMG SRC="/icons/member.gif" ALT="[MEMBER]"&gt; &lt;A HREF="\index.html"&gt;\index.html&lt;/A&gt;            31-Dec-69 6:59:59 PM        1796
	 * </PRE>
	 * Will display as:<BR>
	 * <IMG SRC="/icons/member.gif" ALT="[MEMBER]"> <A HREF="\index.html">\index.html</A>            31-Dec-69 6:59:59 PM        1796
	 */
	public String toString()
	{
		String strHref = uri.toString();

		int NAME_LENGTH = 23;
		int DATE_LENGTH = 28;
		int SIZE_LENGTH = 6;

		if (strHref.length() > NAME_LENGTH)
		{
			strHref = createFixedWidth( strHref, NAME_LENGTH);
			strHref = UriHelper.createHrefHTML(strHref, strHref);
		}
		else
		{
			strHref = UriHelper.createHrefHTML(strHref, strHref) + 
				strSpaces.substring(strHref.length(), NAME_LENGTH);
		}

		String strDate = createFixedWidth(	getHTMLDate(), DATE_LENGTH);
		String strSize = createFixedWidth(	getHTMLSize(), SIZE_LENGTH);

		String strHTML = 
			getHTMLPrefixTag() + " " +
			strHref + 
			strDate +
			strSize + "\n";

		return strHTML;
	}

	// HELPERS

	/**
	 * Creates a &lt;resourcetype&gt; XML element describing this
	 * CollectionMember.
	 *
	 * @return	Returns a &lt;resourcetype&gt; XML element
	 */
	public Element createResourceTypeElement()
	{
		ElementImpl elemResourceType =
			new ElementImpl(WebdavXML.ELEM_RESOURCE_TYPE, Element.ELEMENT);

		if (bIsCollection)
		{
			ElementImpl elemCollection =
				new ElementImpl(WebdavXML.ELEM_COLLECTION_RESOURCE_TYPE, Element.ELEMENT);

			elemResourceType.addChild(elemCollection, null);
		}

		return elemResourceType;
	}

	/**
	 * Takes <var>strItem</var>, truncates it, and adds "..." if
	 * the string is longer than the <var>nWidth</var> specified.
	 * If <var>strItem</var> is shorter than the width specified it
	 * pads the strings with blanks.  The returned string will
	 * have the exact <var>nWidth</var> specified.
	 *
	 * @param strItem	[IN] String to be truncated or padded to
	 *					fit in the number of characters specified
	 *					by <var>nWidth</var>.
	 * @param nWidth	[IN] Desired width for the returned string.
	 * 
	 * @return			Returns a string with the exact width of
	 *					<var>nWidth</var>, or <PRE>"INTERNAL ERROR:
	 *					INVALID PARAMETER"</PRE> if <var>strItem</var>
	 *					parameter is null.
	 */
	private static String createFixedWidth(String strItem, int nWidth)
	{
		if (strItem == null)
		{
			return "INTERNAL ERROR: INVALID PARAMETER";
		}

		int nLen = strItem.length();

		if (nLen < nWidth)
		{
			return strItem + strSpaces.substring(nLen, nWidth);
		}
		else
		{
			return strItem.substring(0, nWidth-3) + "...";
		}
	}

	public static final void main(String[] args)
	{
		System.out.println(new CollectionMember("/lsdjflajsdlfasldfjasafsadfasdf"));
		System.out.println(new CollectionMember("/index.html"));
	}
};