/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.nsman;

import webdav.common.Uri;
import webdav.common.Depth;
import java.io.OutputStream;
import java.io.InputStream;
import java.util.Vector;

/**
 * This interface is part of our WebDAV server interface core.  Namespace
 * Subsystem Providers must implement this interface.
 *
 * @see			SubsystemInterface
 * @author		Marc Eaddy
 * @version		2.0, 16 Nov 1997
 */
public interface NamespaceInterface 
{
	/**
	 * Determine if the URI is valid for the underlying system.
	 * This function should be called before any operations are
	 * attempted for this URI.
	 *
	 * @param	uri [IN] Uri of the resource.
	 *
	 * @return	One of the following status codes:<P>
	 *			<UL>
	 *				<LI>NAMESPC_SUCCESS
	 *				<LI>NAMESPC_UNKNOWN_ERROR
	 *				<LI>NAMEPSC_INTERNAL_ERROR
	 *				<LI>NAMESPC_URI_INVALID
	 *				<LI>NAMESPC_URI_NOT_FOUND
	 *			</UL>
	 */
	public int isValid(Uri uri);

	/**
	 * Retrieves the resource.
	 *
	 * @param	uri			[IN] Uri of the resource
	 * @param	streamOut	[OUT] Will contain the 
	 *						contents of the resource
	 *
	 * @return	One of the following status codes:<P>
	 *			<UL>
	 *				<LI>NAMESPC_SUCCESS
	 *				<LI>NAMESPC_UNKNOWN_ERROR
	 *				<LI>NAMEPSC_INTERNAL_ERROR
	 *				<LI>NAMESPC_URI_INVALID
	 *				<LI>NAMESPC_URI_NOT_FOUND
	 *				<LI>NAMESPC_URI_READ_ERROR
	 *			</UL>
	 */
	public int getResource(Uri uri, OutputStream streamOut);

	/**
	 * Overwrites the resource.
	 *
	 * @param	uri			[IN] Uri of the resource
	 * @param	streamIn	[IN] Will be used to overwrite the
	 *						contents of the resource
	 * @param	bOverwrite	[IN] Whether or not to overwrite the
	 *						resource if it already exists
	 *
	 * @return	One of the following status codes:<P>
	 *			<UL>
	 *				<LI>NAMESPC_SUCCESS				- Success - uri overwritten
	 *				<LI>NAMESPC_RESOURCE_CREATED	- Success - uri created
	 *				<LI>NAMESPC_NO_OVERWRITE		- Resource already exists
	 *				<LI>NAMESPC_UNKNOWN_ERROR		- Unknown error
	 *				<LI>NAMEPSC_INTERNAL_ERROR		- Internal error
	 *				<LI>NAMESPC_URI_INVALID			- Uri is null
	 *				<LI>NAMESPC_URI_NOT_WRITEABLE	- Failed to write uri
	 *				<LI>NAMESPC_URI_WRITE_ERROR		- Failed to write uri
	 *				<LI>NAMESPC_NOT_ENOUGH_SPACE	- Failed because disk is full
	 *				<LI>NAMESPC_COLL_NOT_SUPPORTED	- Can't put to a collection
	 *				<LI>NAMESPC_PARENT_NOT_FOUND	- Parent dir does not exist 
	 *			</UL>
	 */
	public int putResource(Uri uri, InputStream streamIn, boolean bOverwrite);

	/**
	 * Copies the resource.
	 *
	 * @param	uriSrc		[IN] Source resource to copy
	 * @param	uriDest		[IN] Destination resource to copy to
	 * @param	bOverwrite	[IN] Overwrite if the destination
	 *						resource already exists
	 * @param	depth		[IN] 0, 1, or infinity
	 *
	 * @return	One of the following status codes:<P>
	 *			<UL>
	 *				<LI>NAMESPC_SUCCESS				- Success - dest overwritten
	 *				<LI>NAMESPC_RESOURCE_CREATED	- Success - dest created
	 *				<LI>NAMESPC_NO_OVERWRITE		- Dest already exists
	 *				<LI>NAMESPC_UNKNOWN_ERROR		- Unknown error
	 *				<LI>NAMEPSC_INTERNAL_ERROR		- Internal error
	 *				<LI>NAMESPC_URI_INVALID			- Src or Dest uri is null
	 *				<LI>NAMESPC_URI_NOT_FOUND		- Src uri not found
	 *				<LI>NAMESPC_URI_READ_ERROR		- Failed to read src
	 *				<LI>NAMESPC_URI_NOT_WRITEABLE	- Failed to write dest
	 *				<LI>NAMESPC_URI_WRITE_ERROR		- Failed to write dest
	 *				<LI>NAMESPC_NOT_ENOUGH_SPACE	- Failed because disk is full
	 *				<LI>NAMESPC_PARENT_NOT_FOUND	- Parent dir does not exist 
	 *				<LI>NAMESPC_COLL_NOT_SUPPORTED	- Can't copy a collection
	 *				<LI>NAMESPC_COPY_TO_SELF		- Src and Dest uri are same
	 *			</UL>
	 */
	public int copyResource(	Uri uriSrc, 
								Uri uriDest, 
								boolean bOverwrite,
								Depth depth);

	/**
	 * Deletes the resource.
	 *
	 * @param	uri [IN] Uri of the resource to delete
	 *
	 * @return	One of the following status codes:<P>
	 *			<UL>
	 *				<LI>NAMESPC_SUCCESS
	 *				<LI>NAMESPC_URI_INVALID
	 *				<LI>NAMESPC_URI_DELETE_ERROR
	 *				<LI>NAMESPC_UNKNOWN_ERROR
	 *				<LI>NAMEPSC_INTERNAL_ERROR
	 *			</UL>
	 */
	public int deleteResource(Uri uri);

	/**
	 * Moves the resource.
	 *
	 * @param	uriSrc		[IN] Source resource to move
	 * @param	uriDest		[IN] Destination resource to move to
	 * @param	bOverwrite	[IN] Overwrite if the destination
	 *						resource already exists
	 * @param	depth		[IN] 0, 1, or infinity
	 *
	 * @return	One of the following status codes:<P>
	 *			<UL>
	 *				<LI>NAMESPC_SUCCESS				- Success - dest overwritten
	 *				<LI>NAMESPC_RESOURCE_CREATED	- Success - dest created
	 *				<LI>NAMESPC_NO_OVERWRITE		- Dest already exists
	 *				<LI>NAMESPC_UNKNOWN_ERROR		- Unknown error
	 *				<LI>NAMEPSC_INTERNAL_ERROR		- Internal error
	 *				<LI>NAMESPC_URI_INVALID			- Src or Dest uri is null
	 *				<LI>NAMESPC_URI_NOT_FOUND		- Src uri not found
	 *				<LI>NAMESPC_URI_READ_ERROR		- Failed to read src
	 *				<LI>NAMESPC_URI_NOT_WRITEABLE	- Failed to write dest
	 *				<LI>NAMESPC_URI_WRITE_ERROR		- Failed to write dest
	 *				<LI>NAMESPC_NOT_ENOUGH_SPACE	- Failed because disk is full
	 *				<LI>NAMESPC_PARENT_NOT_FOUND	- Parent dir does not exist 
	 *				<LI>NAMESPC_COLL_NOT_SUPPORTED	- Can't copy a collectino
	 *				<LI>NAMESPC_COPY_TO_SELF		- Src and Dest uri are same
	 *				<LI>NAMESPC_URI_DELETE_ERROR	- Could not delete src uri
	 *			</UL>
	 */
	public int moveResource(	Uri uriSrc, 
								Uri uriDest, 
								boolean bOverwrite,
								Depth depth);

	/**
	 * Creates a new collection.
	 *
	 * @param	uri			[IN] Uri of the collection
	 *
	 * @return	One of the following status codes:<P>
	 *			<UL>
	 *				<LI>NAMESPC_URI_INVALID
	 *				<LI>NAMESPC_COLL_ALREADY_EXISTS
	 *				<LI>NAMESPC_UNKNOWN_ERROR
	 *				<LI>NAMESPC_COLLECTION_CREATED
	 *			</UL>
	 */
	public int makeCollection(Uri uri);

	/**
	 * Lists the members of a collection.
	 *
	 * @param	uri			[IN] Uri of the collection
	 * @param	vecMembers	[OUT] Unordered list of the
	 *						members of the collection
	 * @see		CollectionMember
	 *
	 * @return	One of the following status codes:<P>
	 *			<UL>
	 *				<LI>NAMESPC_SUCCESS
	 *				<LI>NAMESPC_URI_INVALID
	 *				<LI>NAMESPC_URI_NOT_COLLECTION
	 *			</UL>
	 */
	public int listCollection(	Uri uri,
								Vector vecMembers );

	/** No error. */
	public static final int NAMESPC_SUCCESS				= 0;

	/** The uri specified does not exist or is not readable. */
	public static final int NAMESPC_URI_NOT_FOUND		= -1000;

	/** An I/O exception was thrown unexpectantly. */
	public static final int NAMESPC_UNKNOWN_ERROR		= -1001;

	/** 
	 * An error occurred when attempting to read the uri
	 * specified.
	 */
	public static final int NAMESPC_URI_READ_ERROR		= -1002;

	/** 
	 * An error occurred when attempting to write the uri
	 * specified.
	 */
	public static final int NAMESPC_URI_WRITE_ERROR		= -1003;

	/** 
	 * An error occurred when attempting to delete the uri
	 * specified
	 */
	public static final int NAMESPC_URI_DELETE_ERROR	= -1004;

	/** 
	 * An error occurred when attempting to copy the uri
	 * specified
	 */
	public static final int NAMESPC_COPY_ERROR			= -1005;

	/** 
	 * An error occurred when attempting to move the uri
	 * specified
	 */
	public static final int NAMESPC_MOVE_ERROR			= -1006;

	/** Read access it not allowed for the uri specified. */
	public static final int NAMESPC_URI_ACCESS_DENIED	= -1007;

	/** 
	 * The operation requested is not supported for collections.
	 */
	public static final int NAMESPC_COLL_NOT_SUPPORTED	= -1008;

	/** The URI was null or contained illegal characters */
	public static final int NAMESPC_URI_INVALID			= -1009;

	/** 
	 * An internal error occurred.  See the system logs
	 * for more information.
	 */
	public static final int NAMESPC_INTERNAL_ERROR		= -1010;

	/** 
	 * The destination already exists and the overwrite
	 * flag is false.
	 */
	public static final int NAMESPC_NO_OVERWRITE		= -1011;

	/** Could not open the uri for writing. */
	public static final int NAMESPC_URI_NOT_WRITEABLE	= -1012;

	/** 
	 * Could not write the resource due to insufficient space
	 * on the server.
	 */
	public static final int NAMESPC_NOT_ENOUGH_SPACE	= -1013;

	/** 
	 * Attempted to create a resource in a directory that
	 * does not exist.
	 */
	public static final int NAMESPC_PARENT_NOT_FOUND	= -1014;

	
	/** 
	 * Cannot copy a resource unto itself.
	 */
	public static final int NAMESPC_COPY_TO_SELF		= -1015;
	
	/**
	 * The uri exists but is not a collection.
	 */
	public static final int NAMESPC_URI_NOT_COLLECTION	= -1016;

	/**
	 * The collection specified already exists.
	 */
	public static final int NAMESPC_COLL_ALREADY_EXISTS	= -1017;

	/** 
	 * The destination resource was successfully created.
	 */
	public static final int NAMESPC_RESOURCE_CREATED	= +1000;

	/** 
	 * The collection was created.
	 */
	public static final int NAMESPC_COLLECTION_CREATED	= +1001;
};

