/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.nsman;

import webdav.common.SubsystemInterface;
import webdav.common.Uri;
import webdav.common.Depth;
import javax.servlet.ServletOutputStream;
import javax.servlet.ServletInputStream;
import java.io.*;
import java.util.Vector;
import java.util.Enumeration;	// For main()

/**
 * Provide functionality to manipulate the namespace (e.g., file system).
 *
 * @see			SubsystemInterface
 * @see			Uri
 * @author		Marc Eaddy
 * @version		1.0, 3 Nov 1997
 */
public class NamespaceManager 
	implements NamespaceInterface, SubsystemInterface
{
	public NamespaceManager()
	{
		super();
	}

	/**
	 * See NamespaceInterface for description.
	 */
	public int isValid(Uri uri)
	{
		if (uri == null)
		{
			return NAMESPC_URI_INVALID;
		}

		// Could use a Hashtable of files for efficiency
		File file = new File(uri.getFullPath());
		
		return (file.exists() ? NAMESPC_SUCCESS : NAMESPC_URI_NOT_FOUND);
	}

	/**
	 * Returns true if the uri refers to a directory
	 */
	private boolean isDirectory(Uri uri)
	{
		if (uri == null)
		{
			return false;
		}

		File file = new File(uri.getFullPath());

		return (file.exists() && file.isDirectory() ? true : false);
	}

	private boolean isEqual(Uri uriA, Uri uriB)
	{
		if (uriA == null || uriB == null)
		{
			return false;
		}

		File fileA = new File(uriA.getFullPath());
		File fileB = new File(uriB.getFullPath());

		return (fileA.equals(fileB));
	}

	/**
	 * See NamespaceInterface for description.
	 */
	public int getResource(Uri uri, OutputStream streamOut)
	{
		FileInputStream streamFileIn;
		File			file;
		long			nLength;
		long			nBytesCopied;

		String strPath = new String(uri.getFullPath());

		if (strPath == null)
		{
			return NAMESPC_URI_INVALID;
		}

		// Open the uri file
		file = new File(strPath);
			
		// Make sure its a file not a directory
		if (file.isDirectory())
		{
			return getDirectory(uri, streamOut);
		}

		nLength = file.length();

		try
		{
			streamFileIn = new FileInputStream(file);
		}
		catch (FileNotFoundException ex)
		{
			return NAMESPC_URI_NOT_FOUND;
		}

		// Copy the file

		try
		{
			nBytesCopied = copyBytes(streamFileIn, streamOut);
			streamFileIn.close();
		}
		catch(IOException ex)
		{
			return NAMESPC_URI_READ_ERROR;
		}

		if (nBytesCopied == -1 || nBytesCopied != nLength)
		{
			return NAMESPC_URI_READ_ERROR;
		}

		return NAMESPC_SUCCESS;
	}

	/**
	 * See NamespaceInterface for description.
	 */
	public int putResource(Uri uri, InputStream streamIn, boolean bOverwrite)
	{
		FileOutputStream streamFileOut;
		boolean bDestAlreadyExists = (isValid(uri) == NAMESPC_SUCCESS);
		long nBytesCopied;

		// Open the file for writing
		if (uri == null)
		{
			return NAMESPC_URI_INVALID;
		}

		if (bDestAlreadyExists)
		{
			if (!bOverwrite)
			{
				return NAMESPC_NO_OVERWRITE;
			}
			else if (isDirectory(uri))
			{
				return NAMESPC_COLL_NOT_SUPPORTED;
			}
		}

		try
		{
			streamFileOut = new FileOutputStream( uri.getFullPath() );
		}
		catch (IOException ex)
		{
			return NAMESPC_URI_NOT_WRITEABLE;
		}

		// Copy the file

		try
		{
			nBytesCopied = copyBytes(streamIn, streamFileOut);
			streamFileOut.close();
		}
		catch(IOException ex)
		{
			return NAMESPC_URI_WRITE_ERROR;
		}

		if (nBytesCopied == -1)
		{
			return NAMESPC_URI_WRITE_ERROR;
		}

		if (bDestAlreadyExists)
		{
			return NAMESPC_SUCCESS;
		}
		else
		{
			return NAMESPC_RESOURCE_CREATED;
		}
	}

	/**
	 * See NamespaceInterface for description.
	 */
	public int copyResource(	Uri uriSrc, 
								Uri uriDest, 
								boolean bOverwrite,
								Depth depth)	// Ignoring for now...
	{
		ByteArrayOutputStream streamSrc = new ByteArrayOutputStream();

		if (isEqual(uriSrc, uriDest))
		{
			return NAMESPC_COPY_TO_SELF;
		}

		if (isDirectory(uriSrc))
		{
			return NAMESPC_COLL_NOT_SUPPORTED;
		}

		int nGetReturn = getResource(uriSrc, streamSrc);
		if (nGetReturn < NAMESPC_SUCCESS)
		{
			return nGetReturn;
		}

		ByteArrayInputStream streamDest = 
			new ByteArrayInputStream(streamSrc.toByteArray());
		
		return putResource(uriDest, streamDest, bOverwrite);
	}

	/**
	 * See NamespaceInterface for description.
	 */
	public int deleteResource(Uri uri)
	{
		File file;

		if (uri == null)
		{
			return NAMESPC_URI_INVALID;
		}

		String strPath = uri.getFullPath();

		if (strPath == null)
		{
			return NAMESPC_URI_INVALID;
		}

		// Open the uri file
		System.err.println("Deleting: " + strPath);
		file = new File(strPath);
			
		if (!file.delete())
		{
			return NAMESPC_URI_DELETE_ERROR;
		}

		return NAMESPC_SUCCESS;
	}

	/**
	 * See NamespaceInterface for description.
	 */
	public int moveResource(	Uri uriSrc, 
								Uri uriDest, 
								boolean bOverwrite,
								Depth depth)
	{
		int nCopyReturn = copyResource(uriSrc, uriDest, bOverwrite, depth);
		if (nCopyReturn < NAMESPC_SUCCESS)
		{
			return nCopyReturn;
		}

		int nDeleteReturn = deleteResource(uriSrc);
		if (nDeleteReturn < NAMESPC_SUCCESS)
		{
			return nDeleteReturn;
		}

		// We aren't concerned with delete success status codes
		return nCopyReturn;
	}

	public int listCollection(	Uri uri,
								Vector vecMembers )
	{
		// Initialize the output vector
		vecMembers.removeAllElements();

		String strPath = uri.getFullPath();

		if (strPath == null)
		{
			return NAMESPC_URI_INVALID;
		}

		// Open the uri file
		File file = new File(strPath);
			
		// Make sure its a directory
		if (!file.isDirectory())
		{
			return NAMESPC_URI_NOT_COLLECTION;
		}

		// Add the directory name
		vecMembers.addElement( new CollectionMember(uri) );

		// Add all the file names
		String[] aryStrings = file.list();
		for(int i=0; i < aryStrings.length; ++i)
		{
			Uri uriNew = new Uri(uri.toString() + aryStrings[i]);

			vecMembers.addElement( new CollectionMember(uriNew) );
		}

		return NAMESPC_SUCCESS;
	}

	public int makeCollection(Uri uri)
	{
		File file;

		if (uri == null)
		{
			return NAMESPC_URI_INVALID;
		}

		String strPath = uri.getFullPath();

		if (strPath == null)
		{
			return NAMESPC_URI_INVALID;
		}

		file = new File(strPath);

		if (file.exists())
		{
			return NAMESPC_COLL_ALREADY_EXISTS;
		}

		if (!file.mkdir())
		{
			return NAMESPC_UNKNOWN_ERROR;
		}

		return NAMESPC_COLLECTION_CREATED;
	}

	private int getDirectory(Uri uri, OutputStream streamOut)
	{
		Vector vecMembers = new Vector();

		int nListReturn = listCollection(uri, vecMembers);
		
		if (nListReturn != NAMESPC_SUCCESS)
		{
			return nListReturn;
		}

		String strHeader = getDirHeaderHTML(uri);
		String strFooter = getDirFooterHTML();

		try
		{
			outputString(streamOut, strHeader);

			Enumeration enumMembers = vecMembers.elements();
			while (enumMembers.hasMoreElements())
			{
				CollectionMember member =
					(CollectionMember) enumMembers.nextElement();

				outputString(streamOut, member.toString());
			}

			outputString(streamOut, strFooter);
		}
		catch(IOException ex)
		{
			return NAMESPC_URI_READ_ERROR;
		}

		return NAMESPC_SUCCESS;
	}

	private String getDirHeaderHTML(Uri uri)
	{
		return	"<HTML>\n\n" +
				"<HEAD>\n" +
				"<TITLE>Index of " + uri.toString() + "</TITLE>\n" +
				"</HEAD>\n\n" +
				"<BODY>\n" +
				"<H1>Index of " + uri.toString() + "</H1>\n" +
				"<PRE>\n" +
				"<IMG SRC=\"/icons/blank.gif\" ALT=\"     \"> " +
				"Name                   " +
				"Last modified               " +
				"Size  " +
				"Description\n\n" +
				"<HR>\n";
	}

	private String getDirFooterHTML()
	{
		return	"</PRE>\n" +
				"</BODY>\n" +
				"</HTML>\n";
	}

	/**
	 * Copies data from the specified input stream to the output stream
	 * until end of file is met.
	 * @return    the total number of bytes written to the output stream
	 */
	public static long copyBytes(InputStream in, OutputStream out)
		throws IOException
	{
		byte[] buf = new byte[1024];
		long nTotal = 0;
		int nBytesRead;

		while ((nBytesRead = in.read(buf)) > -1) 
		{
			out.write(buf, 0, nBytesRead);
			nTotal += nBytesRead;
		}

		return nTotal;
	}

	private static void outputString(OutputStream out, String str)
		throws IOException
	{
		byte[] bytes = new byte[str.length()];

		str.getBytes(0, str.length(), bytes, 0);

		out.write(bytes);
	}

	public static void main(String[] args)
	{
		Uri uri = new Uri("/");

		System.out.println(uri.getFullPath());

		StringBufferInputStream streamIn = 
			new StringBufferInputStream("This is a new file!");

		ByteArrayOutputStream streamOut = new ByteArrayOutputStream();

		NamespaceManager nsman = new NamespaceManager();

		int nRet = nsman.putResource(uri, streamIn, true);
		if (nRet < 0)
		{
			System.out.println("putResource error: " + nRet);
		}

		nRet = nsman.getResource(uri, streamOut);
		if (nRet < 0)
		{
			System.out.println("getResource error: " + nRet);
		}

		System.out.println(streamOut);

		Uri uriDir = new Uri("/");

		System.out.println(uriDir.getFullPath());

		Vector vecDirItems = new Vector();

		nRet = nsman.listCollection(uriDir, vecDirItems);

		if (nRet < 0)
		{
			System.out.println("listDirectory error: " + nRet);
		}

		Enumeration enumDirItems = vecDirItems.elements();
		System.out.println("Directory Listing for:\n");
		while (enumDirItems.hasMoreElements())
		{
			CollectionMember item = 
				(CollectionMember) enumDirItems.nextElement();

			System.out.println(item);
		}

	}
};