/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.propman; 

import java.io.*;
import java.util.Vector;
import java.util.Enumeration;
import com.ms.xml.om.*;
import com.ms.xml.util.Name;
import webdav.common.*;
import webdav.lockman.*;

/**
 * The LockPropertyHandler implements the LivePropertyHandler and
 * PropertyManager interfaces to provide support for the <lockdiscovery>
 * property. Since the information returned in the value of this property
 * is really stored by the lock manager, we will install this live property
 * handler to intercept requests and implement them in terms of the Lock
 * Manager's interface. Note that this property will be implemented as
 * read-only. Setting it would be equivalent to locking the resourec, and
 * should only happen in the context of a LOCK method, or an internal system
 * lock being requested by the executive layer. 
 *
 * @author  Jonathan Shapiro, Shao Rong, Marc Eaddy
 * @version 1.1, 12/14/97
 */

public class LockPropertyHandler implements  LivePropertyHandler, PropertyManager {

  public LockPropertyHandler() {
    /*grab a reference to the lock manager*/
    _lockMan = (LockManIntf )SubsystemRegistry.lookup("Lock", new Uri(""));
  }

	/**
	 @see LivePropertyHandler 
	 */
  public void setDefaultPropertyManager(PropertyManager pm) {
    /*Null-method: Storage will not be delegated to the default manager */
    return;
  }

	/**
	 * @see LivePropertyHandler
	 */
    public Name[] querySupportedProperties() {
	Name props[] = new Name[1];

	/* This handler is only interested in the <lockdiscovery> property*/
	props[0] = Name.create("lockdiscovery");
	return props;
    }

    /*Public Interface [PropManager]*/

	
	/** @see PropertyManager */
	public int getProp(Uri uri, Element inProp, Element outProp) { 
	  Vector lockInfo = new Vector();

		int nGetLocks = _lockMan.getLocks(uri, lockInfo);

		if (nGetLocks < LockManIntf.LOCKMAN_SUCCESS)
		{
			System.out.println();
			return pm_err_propnotfound;	// This is not really appropriate
		}
	  else if (lockInfo.size() == 0) {
	    return pm_err_propnotfound;
	  }
	  else {
	    ElementImpl lockDiscElem = new ElementImpl(Name.create("lockdiscovery"), Element.ELEMENT);
	    ElementImpl activeLockElem = null;
		Enumeration lockEnum = lockInfo.elements();
	    while (lockEnum.hasMoreElements()) {
	      activeLockElem = lockManCoreToActiveLock((LockManCore)lockEnum.nextElement());
	      
	      if (activeLockElem != null) {
			lockDiscElem.addChild(activeLockElem, null);
	      }
	    }

		outProp.addChild(lockDiscElem, 0, 0);
		return pm_success;
	  }
	}  

	/** @see PropertyManager */
	public int setProp(Uri uri,	Element inProp, Element outProp) {
	  /*This would be equivalent to setting a lock. Such a request must go
            directly through the lock manager.*/
	  return pm_err_read_only;
	}

	/** @see PropertyManager */
	public int deleteProp(	Uri uri,	Element inProp) { 
	  /*
	    This would be equivalent to removing a lock. Such a request must go
	    directly through the lock manager.
	   */
	  return pm_err_read_only;
	}

	
	/** @see PropertyManager */
	public int getAll(Uri uri, Element outProp) {
	  ElementImpl propElem = new ElementImpl(Name.create("prop"), Element.ELEMENT);
	  propElem.addChild(new ElementImpl(Name.create("lockdiscovery"), Element.ELEMENT), null);
	  
	  int nRet = getProp(uri, propElem, outProp);

	  if (nRet == pm_err_propnotfound)	// Its okay if a lock does not
		  return pm_success;			//  exist on the resource
	  else
		  return nRet;
	
	}

	/** @see PropertyManager */
	public int copyAll(Uri srcUri, Uri destUri, Element outProp) {
	  /*Not an error condition, but the copied resource should not inherit any locks
	   on the original. Just return success.*/
	  return pm_success;
	}	

	/** @see PropertyManager */
	public int moveAll(Uri srcUri, Uri destUri) { 
	  /* Not an error condition, but fixing up the lock table in response to a moved 
	     resource is a matter to be resolved between the executive and LockManager.
	   */
	  return pm_success;
	}	

	/** @see PropertyManager */
	public int deleteAll( Uri uri ) {
	  /* Not an error condition, but fixing up the lock table in response to a moved 
	     resource is a matter to be resolved between the executive and LockManager.
	   */
	  return pm_success;
	}	

	/** @see PropertyManager */
	public int getAllNames(Uri uri, Element outProp) { 
	  /* The lockdiscovery property is a special property, it shouldn't show up in a 
	     list of returned by getAllNames*/
	  return pm_success;
	}

  private ElementImpl lockManCoreToActiveLock(LockManCore lmCore) {
    ElementImpl activeLockElem = new ElementImpl(Name.create("activelock"), Element.ELEMENT);
    ElementImpl tempElem = null;
	ElementImpl tempData = null;
    
    /* locktype */
    tempElem = new ElementImpl(Name.create("locktype"), Element.ELEMENT);
	tempData = new ElementImpl(null, Element.PCDATA);
    tempData.setText(lmCore.getLockType());
	tempElem.addChild(tempData, null);
    activeLockElem.addChild(tempElem, null);

    /* lockscope */
    tempElem = new ElementImpl(Name.create("lockscope"), Element.ELEMENT);    
	tempData = new ElementImpl(null, Element.PCDATA);
    tempData.setText(lmCore.getLockScope());
	tempElem.addChild(tempData, null);
    activeLockElem.addChild(tempElem, null);

    /* addlocks */
    //NOT CURRENTLY SUPPORTED

    /* owner */
    tempElem = new ElementImpl(Name.create("owner"), Element.ELEMENT);
	tempData = new ElementImpl(null, Element.PCDATA);
    tempData.setText(lmCore.getOwnerInfo());
	tempElem.addChild(tempData, null);
    activeLockElem.addChild(tempElem, null);

    /* timeout */
    tempElem = new ElementImpl(Name.create("timeout"), Element.ELEMENT);
	tempData = new ElementImpl(null, Element.PCDATA);
    tempData.setText(lmCore.getTimeout() + " " + lmCore.getTimeType());
	tempElem.addChild(tempData, null);
    activeLockElem.addChild(tempElem, null);

    /* locktoken */
    tempElem = new ElementImpl(Name.create("locktoken"), Element.ELEMENT);
	tempData = new ElementImpl(null, Element.PCDATA);
    tempData.setText(lmCore.getLockTokenString());
	tempElem.addChild(tempData, null);
    activeLockElem.addChild(tempElem, null);

    return activeLockElem;
  }

  private LockManIntf _lockMan = null;
}




