/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.propman;

import webdav.common.Uri;
import java.io.*;
import com.ms.xml.om.*;
import com.ms.xml.util.Name;

/**
 * An object that implements the PropertyManager interface  
 * is capable of fulfilling requests to get, set, enumerate and
 * manipulate property values on behalf of the Property subsystem
 *
 * @author  Jonathan Shapiro
 * @version 1.0, 11/15/97
 */


public interface PropertyManager {
	/**
	 * Gets the value of particular property on a given resource
	 * 
	 * @param URI		[IN] Contents of the file part of the requested URI (no protocol or host)
	 * @param inProp	[IN] An element of type ELEMENT with TagName = "prop" and a single child
	 *					element of type ELEMENT, TagName = requested-property-name.
	 * @param outProp	[OUT] On return, element of type ELEMENT with TagName = "prop" and a single child
	 *					element of type ELEMENT, TagName = requested-property-name, and values filled in
	 *					to reflect the current value of requested-property-name on resource URI.
	 * @return			pm_success on success
	 *					pm_err_*   on error
	 */
	int getProp(	Uri URI,        
					Element inProp,    
					Element outProp);  

	/**
	* Sets the value of a particular property on a given resource
	*
	*	@param URI		[IN] Contents of the file part of the requested URI (no protocol or host)
	*	@param inProp	[IN] Element of type ELEMENT with TagName = "prop" and a single child
	*					element of type ELEMENT, TagName = requested-property-name, and values filled in
	*					to reflect the current value of requested-property-name on resource URI.
	*	@param outProp [OUT] On return, element of type ELEMENT with TagName = "prop" and a single child
	*					element of type ELEMENT, TagName = requested-property-name.
	*
	*	@return			pm_success on success
	*					pm_err_*   on error
	*/
	int setProp(	Uri URI,			 
					Element inProp, 
					Element outProp);

	/**
	* Deletes a particular property on a given resource
	*
	* @param URI		[IN] Contents of the file part of the requested URI (no protocol or host)
	* @param inProp		[IN] Element of type ELEMENT with TagName = "prop" and a single child
	*	                element of type ELEMENT, TagName = requested-property-name, and values filled in
	*					to reflect the current value of requested-property-name on resource URI.
	*
	* @return 			pm_success on success
	*					pm_err_*   on error
	*
	*/
	int deleteProp(	Uri URI,			
					Element inProp);

	/**
	* Get values for all properties defined on a given resource
	*
	* @param URI		[IN] Contents of the file part of the requested URI (no protocol or host)
	* @param outProp	[OUT] On return, element of type ELEMENT with TagName = "prop" and possibly
	*	                multiple children, each of which is an element of type ELEMENT, and whose
	*					TagNames are the names of all defined properties that can be seen by the current
	*					user and whose contents are the values of those properties.
	*
	* @return			pm_success on success
	*					pm_err_*   on error
	*/
	int getAll(		Uri URI, 
					Element outProp);

	/**
	* Copies all property values for one resource to another resource
	*
	* @param srcURI		[IN] Contents of the file part of the source URI
	* @param destURI	[IN] Contents of the file part of the destination URI
	* @param outProp	[OUT] On return, element of type ELEMENT with TagName = "prop" and possibly
	*	                multiple children, each of which is an element of type ELEMENT, and whose
	*					TagNames are the names of all properties that COULD NOT BE COPIED.
	*
	* @return			pm_success on success
	*					pm_err_*   on error
	*/
	int copyAll(	Uri srcURI,		
					Uri destURI,		
					Element outProp);	

	/**
	 * Deletes all the properties for a resource.
	 *
	 * @param uri		[IN] Name of the resource.
	 */
	int deleteAll(	Uri uri );

	/**
	* Change all property values defined on a resource to be defined on another resource
	*	
	* @param srcURI		[IN] Contents of the file part of the source URI
	* @param destURI	[IN] Contents of the file part of the destination URI
	*
	* @return			pm_success on success
	*					pm_err_*   on error
	*/
	int moveAll(	Uri srcURI,		
					Uri destURI);	

	/**
	* Get the names of all properties for which values are defined on a given resource
	*
	* @param URI		[IN] Contents of the file part of the source URI
	* @param outProp	[OUT] On return, element of type ELEMENT with TagName = "prop" and possibly
	* 					multiple children, each of which is an element of type ELEMENT, and whose
	*					TagNames are the names of all defined properties. The values of the properties
	*					are omitted.
	* @return			pm_success on success
	*					pm_err_*   on error
	*/
	int getAllNames(Uri URI,			
					Element outProp);	

	//Return Codes//
	static final int pm_success					= 0;
	static final int pm_partial_success         = 1;
	static final int pm_err_general				= -1;
	static final int pm_err_propnotfound		= -2;
	static final int pm_err_invalid_input_elem	= -3;
	static final int pm_err_setprop				= -4;
	static final int pm_err_delete				= -5;
	static final int pm_err_read_only			= -6;
}


