/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.propman;

import webdav.common.SubsystemInterface;
import webdav.common.Uri;
import webdav.common.WebdavXML;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import com.ms.xml.om.*;
import com.ms.xml.util.Name;

//
//
// PropertySubsystem
//
//
public class PropertySubsystem implements PropertyManager, LivePropertyDelegator,
	SubsystemInterface
{
	public PropertySubsystem() {
		_defPropMan = new SimplePropManager();
		_handlers.addElement(_defPropMan);
	}

	/*Public Interface [PropManager]*/

	/*PropManager interface functions that take an inProp element as
	  an argument are interacting with a specific property. For this
	  type of function, we just locate the appropriate property manager
	  and pass the request along. 
	*/

	/** @see PropertyManager */
	public int getProp(Uri URI, Element inProp, Element outProp) { 
		PropertyManager pm = getHandler(inProp);
		return pm.getProp(URI, inProp, outProp);
	}  

	/** @see PropertyManager */
	public int setProp(Uri URI,	Element inProp, Element outProp) { 
		PropertyManager pm = getHandler(inProp);
		return pm.setProp(URI, inProp, outProp);
	}

	/** @see PropertyManager */
	public int deleteProp(	Uri URI,	Element inProp) { 
		PropertyManager pm = getHandler(inProp);
		return pm.deleteProp(URI, inProp);
	}

	/*PropManager interface functions that DO NOT take an inProp element as
	  an argument are interacting with all properties for a given URI. For this
	  type of function, we have to give all property handlers a chance to handle
	  the request by enumerating them, possibly accumulating properties in the 
	  outProp parameter in the process.
	*/
	/** @see PropertyManager */
	public int getAll(Uri URI, Element outProp) {
		int iFailures = 0;
		Enumeration enumHandlers = _handlers.elements();
		PropertyManager pm = null;

		while(enumHandlers.hasMoreElements()) {
			pm = (PropertyManager)enumHandlers.nextElement();
			if (pm.getAll(URI, outProp) < pm_success)
				iFailures++;
		}

		return (iFailures == 0) ? pm_success : pm_err_general;
	}

	/** @see PropertyManager */
	public int copyAll(Uri srcURI, Uri destURI, Element outProp) { 
		int iFailures = 0;
		Enumeration enumHandlers = _handlers.elements();
		PropertyManager pm = null;

		while(enumHandlers.hasMoreElements()) {
			pm = (PropertyManager)enumHandlers.nextElement();
			if (pm.copyAll(srcURI, destURI, outProp) < pm_success)
				iFailures++;
		}

		return (iFailures == 0) ? pm_success : pm_err_general;
	}	

	/** @see PropertyManager */
	public int moveAll(Uri srcURI, Uri destURI) { 
		int iFailures = 0;
		Enumeration enumHandlers = _handlers.elements();
		PropertyManager pm = null;

		while(enumHandlers.hasMoreElements()) {
			pm = (PropertyManager)enumHandlers.nextElement();
			if (pm.moveAll(srcURI, destURI) < pm_success)
				iFailures++;
		}

		return (iFailures == 0) ? pm_success : pm_err_general;
	}	

	/** @see PropertyManager */
	public int deleteAll( Uri uri )
	{
		int iFailures = 0;
		Enumeration enumHandlers = _handlers.elements();
		PropertyManager pm = null;

		while(enumHandlers.hasMoreElements()) {
			pm = (PropertyManager)enumHandlers.nextElement();
			if (pm.deleteAll(uri) < pm_success)
				iFailures++;
		}

		return (iFailures == 0) ? pm_success : pm_err_general;
	}

	/** @see PropertyManager */
	public int getAllNames(Uri URI, Element outProp) { 
		int iFailures = 0;
		Enumeration enumHandlers = _handlers.elements();
		PropertyManager pm = null;

		while(enumHandlers.hasMoreElements()) {
			pm = (PropertyManager)enumHandlers.nextElement();
			if (pm.getAllNames(URI, outProp) < pm_success)
				iFailures++;
		}

		return (iFailures == 0) ? pm_success : pm_err_general;
	}
	
	/*Public Interface [LivePropertyDelegator]*/

	/** @see LivePropertyDelegator */
	public boolean register(LivePropertyHandler lph) {
		//get a list of properties supported by this handler
		Name[] props = lph.querySupportedProperties();
		
		if (props.length == 0) return false;

		//Add it to the list of available handlers
		_handlers.addElement(lph); 

		//Record which properties it handles
		for(int i = 0; i < props.length; i++) {
			_propToHandler.put(props[i].toString(), lph);
		}

		//Tell it about the default property manager
		lph.setDefaultPropertyManager(_defPropMan);

		return true;
	}

	/*Private Methods*/

	/**
	* Given a prop tag, return the handler that supports it or the default property manager
	*
	* @param inProp		A property name encapsulated in a <prop> tag
	*/
	private PropertyManager getHandler(Element inProp) {
		Name requestTag = null;

		requestTag = WebdavXML.getNonNullTagName(inProp);		
//		Enumeration enum = inProp.getElements();
//		while (enum.hasMoreElements()) {
//			Element elemTmp = (Element)enum.nextElement();
//			requestTag = elemTmp.getTagName();
//			if (requestTag != null) break;
//		}

		PropertyManager pm = (requestTag != null) ? (PropertyManager)_propToHandler.get(requestTag.toString())
				                                  : null;

		return pm != null ? pm : _defPropMan; 
	}

	/*Private Data*/
	/** Maps property names to the live handler that handles them */
	private Hashtable _propToHandler = new Hashtable(); 
	/** A list of all available live property handlers */
	private Vector _handlers = new Vector();		  
	/** The default implementation of PropertyManager */
	private PropertyManager _defPropMan = null; 


}

