/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.propman;

import webdav.common.Uri;
import webdav.common.WebdavXML;
import java.io.*;
import java.util.Enumeration;
import com.ms.xml.om.*;
import com.ms.xml.parser.ParseException;
import com.ms.xml.util.Name;
import com.ms.xml.util.XMLOutputStream;


public class SimplePropManager implements PropertyManager {
	/*
		Properties are stored as XML files named after
		the resource itself in the following manner:

		resource name: /foo/bar.html
		property name: blat
		property stored in: /foo/bar.html.prop.blat
	*/


   /*Public Interface [PropManager]*/

	/** @see PropertyManager */
	public int getProp(Uri uri, Element inProp,Element outProp) {
		Name propTag = Name.create("prop");
		Name requestTag = null;

		//inProp must be an XML <prop> element with at least 1 child element
		if (inProp.numElements() < 1 ||
			!propTag.equals(inProp.getTagName()))
			return pm_err_invalid_input_elem;

		//the child gives the name of the property being requested
		requestTag = WebdavXML.getNonNullTagName(inProp);

		//read prop file into outProp
		return (readPropFile(uri.getFullPath(), requestTag, outProp));
	}

	/** @see PropertyManager */
	public int setProp(Uri uri, Element inProp, Element outProp) {
		Name propTag = Name.create("prop");
		Name requestTag = null;
		int iRet = pm_err_general;

		//inProp must be an XML <prop> element with at least 1 child element
		if (inProp.numElements() < 1 ||
			!propTag.equals(inProp.getTagName()))
			return pm_err_invalid_input_elem;

		//the child gives the name of the property being set
		requestTag = WebdavXML.getNonNullTagName(inProp);

		iRet = writePropFile(uri.getFullPath(), requestTag, inProp);
		
		//format output
		clearProperties(outProp);
		outProp.addChild(new ElementImpl(requestTag, Element.ELEMENT), null);

		return iRet;
	}

	/** @see PropertyManager */
	public int deleteProp(	Uri uri, Element inProp) {
		Name propTag = Name.create("prop");
		Name requestTag = null;

		//inProp must be an XML <prop> element with at least 1 child element
		if (inProp.numElements() < 1 ||
			!propTag.equals(inProp.getTagName()))
			return pm_err_invalid_input_elem;

		//the child gives the name of the property being requested
		requestTag = WebdavXML.getNonNullTagName(inProp);

		File propFile = new File(propFileName(uri.getFullPath(), requestTag.toString()));

		try {
			return propFile.delete() ? pm_success : pm_err_delete;
		} catch (SecurityException e) {
			return pm_err_delete;
		}
	}

	/** @see PropertyManager */
	public int getAll(Uri uri, Element outProp) {
		String propFiles[] = listPropFiles(uri.getFullPath());
		int iFailures = 0;

		//Set outProp element to be a single <prop> tag with no children
		clearProperties(outProp);

		for (int i=0; i < propFiles.length; i++) {
			Name tag = tagFromFilename(propFiles[i]);
			int tmpRet = readPropFile(uri.getFullPath(), tag, outProp);
			if (tmpRet != pm_success) {
				System.out.println("Could not read: " + 
					uri.getFullPath() + " Property=" + tag);
				iFailures++;
			}
		}

		return (iFailures == 0) ? pm_success
			 : (iFailures < propFiles.length) ? pm_partial_success 
			 : pm_err_general;
	}

	/** @see PropertyManager */
	public int copyAll(Uri srcUri, Uri destUri, Element outProp) {
		String[] propFiles = listPropFiles(srcUri.getFullPath());

		File src = new File(srcUri.getFullPath());
		int iFailures = 0;

		//Set outProp element to be a single <prop> tag with no children
		clearProperties(outProp);

		for (int i=0; i < propFiles.length; i++) {
			Name tag = tagFromFilename(propFiles[i]);
			
			try {
				int b;
				FileInputStream fin = new FileInputStream(src.getParent() + File.separator + propFiles[i]);
				FileOutputStream fout = new FileOutputStream(destUri.getFullPath() + 
					                                         PROP_FILENAME_SEPARATOR +
															 tag.toString());
				while ((b = fin.read()) != -1) {
					fout.write(b);
				}
				fout.close();
				fin.close();
			} catch (FileNotFoundException e) {
				//add uncopied tag to outProp
				outProp.addChild(new ElementImpl(tag, Element.ELEMENT), null);
				iFailures++;
			} catch (IOException e) {
				//add uncopied tag to outProp
				outProp.addChild(new ElementImpl(tag, Element.ELEMENT), null);
				iFailures++;
			}							
		}

		return (iFailures == 0) ? pm_success
			 : (iFailures < propFiles.length) ? pm_partial_success 
			 : pm_err_general;
	}

	/** @see PropertyManager */
	public int moveAll(Uri srcUri, Uri destUri) {

		ElementImpl elemOut = new ElementImpl(WebdavXML.ELEM_PROP,
												Element.ELEMENT);

		int nCopyReturn = copyAll(srcUri, destUri, elemOut);
		
		if (nCopyReturn != pm_success)
		{
			return nCopyReturn;
		}

		return deleteAll(srcUri);
	}

	/** @see PropertyManager */
	public int deleteAll( Uri uri )
	{
		String[] propFiles = listPropFiles(uri.getFullPath());
		File fileSrc = new File(uri.getFullPath());
		File dirSrc  = new File(fileSrc.getParent());
		int iFailures = 0;

		for (int i=0; i < propFiles.length; ++i)
		{
			File file = new File(dirSrc, propFiles[i]);
			System.out.println("Removing property file: " + file);
			if (!file.delete())
			{
				++iFailures;
			}
		}

		return (iFailures == 0) ? pm_success
			 : (iFailures < propFiles.length) ? pm_partial_success 
			 : pm_err_general;
	}

	/** @see PropertyManager */
	public int getAllNames(Uri uri, Element outProp) 
    {
		String propFiles[] = listPropFiles(uri.getFullPath());

		//Set outProp element to be a single <prop> tag with no children
		clearProperties(outProp);

		for (int i=0; i < propFiles.length; i++) 
        {
			Name tag = tagFromFilename(propFiles[i]);
			outProp.addChild(new ElementImpl(tag, Element.ELEMENT), null);
		}

		return pm_success;
	}


	/*Private methods*/

	/** 
	* Generate the name of the file where a property is stored. The file name
	* is a combination of the the resource uri and the property name
	*
	* @param uriPath		The resource name
	* @param propTag	Contains the property name
	*
	* @return			The name of the property file
	*/
	private String propFileName(String uriPath, String propTag) {
		StringBuffer fileName = new StringBuffer();

		fileName.append(uriPath);
		fileName.append(".prop.");
		fileName.append(propTag);

		return fileName.toString();
	}

	/** 
	* Read in a property file and add the properties it contains to
	* a specified prop tag element 
	*
	* @param uriPath		The resource name
	* @param propTag	Contains the property name
	* @param outProp	Container for read in properties
	*
	* @return			pm_success or pm_err_general
	*/
	private int readPropFile(String uriPath, Name propTag, Element outProp) {
		String fileName = propFileName(uriPath, propTag.toString());
		Document propFile = new Document();
		Element propTagElem = null;
		Element property = null;
		Enumeration enumProps = null;
		int iProps = 0;

		try {
			FileInputStream file_in = new FileInputStream(fileName);
			propFile.load(file_in);
			file_in.close();
		} catch (IOException e) {
			outProp.addChild(new ElementImpl(propTag, Element.ELEMENT), null);
			return PropertyManager.pm_err_propnotfound;
		} catch (ParseException e) {
			outProp.addChild(new ElementImpl(propTag, Element.ELEMENT), null);
			return PropertyManager.pm_err_propnotfound;
		}

		propTagElem = propFile.getRoot();
		enumProps = propTagElem.getElements();


		while (enumProps.hasMoreElements()) {
			property = (Element)enumProps.nextElement();
			if (property.getType() == Element.ELEMENT) {
				iProps++;
				System.out.println("  Is " + property.getTagName() +
									" == " + propTag + " ?");
				if (propTag.equals(property.getTagName())) {
					outProp.addChild(property, 0, 0);
					return PropertyManager.pm_success;
				}
			}
		}

		return PropertyManager.pm_err_general;
	}

	/** 
	* Write out a property file containing property values 
	*
	* @param uriPath		The resource name
	* @param propTag	Contains the property name
	* @param inProp		Contains the property value
	*
	* @return			pm_success or pm_err_setprop
	*/
	private int writePropFile(String uriPath, Name propTag, Element inProp) {
		String fileName = propFileName(uriPath, propTag.toString());
		Document propFile = new Document();

		propFile.addChild(inProp, null);

		try {
			FileOutputStream out = new FileOutputStream(fileName);
			XMLOutputStream xml_out = new XMLOutputStream(out);
			propFile.save(xml_out);
			xml_out.close();
			out.close();
		} catch (IOException e) {
			return pm_err_setprop;
		}
	
		return pm_success;
	}

	/** 
	* List all property file names containing property values for a given resource 
	*
	* @param uriPath		The resource name.
	*
	* @return			Array of file names.
	*/
	private String[] listPropFiles(String uriPath) 
    {
		File resource = new File(uriPath);
		String dir = resource.getParent(); 

        System.out.println("listPropFiles: " + uriPath + " DIR:" + dir);

        File propDir = new File(dir);
		String resourceName = resource.getName();
		StringBuffer filterSpec = new StringBuffer(resourceName);
		String propFiles[] = null;

		filterSpec.append(".prop.*");
		FileFilter filter = new FileFilter(filterSpec.toString());

		try 
        {
			propFiles = propDir.list(filter);	
		} 
        catch (SecurityException e) 
        {
		}

		return propFiles;
	}

	/** 
	* Extract the property name from the property file name 
	*
	* @param URI		The resource name.
	*
	* @return			Property Name.
	*/
	private Name tagFromFilename(String filename) {
		/*This probably deserves a comment. SimplePropManager stores a property on a resource
		  in a file named <resource-filename>.prop.<property name>. This function just parses
		  a file name looking for everything to the right of ".prop.", which is, of course, the 
		  property name, itself.*/
		int iIndex = filename.indexOf(PROP_FILENAME_SEPARATOR) + PROP_FILENAME_SEPARATOR.length();
		return (iIndex >= 0) ? Name.create(filename.substring(iIndex) + "") : null;
	}


	/** 
	* Empties a prop tag XML element of all children
	*
	* @param props		An XML Element object
	*
	*/
	private void clearProperties (Element props) {
		//remove all children of the root element
		Enumeration enumKids = props.getElements();

		while (enumKids.hasMoreElements()) {
			props.removeChild((Element)enumKids.nextElement());
		}

		return;
	}

	/*Private data*/
	private static final String PROP_XML_TAG_NAME = "Prop";
	private static final String PROP_FILENAME_SEPARATOR = ".prop.";
}


/** This class was ripped off the comp.lang.java newsgroup. It is a simple file filter
*	that allows a single wildcard '*'. Good enough for our purposes.
*/
class FileFilter implements FilenameFilter {
	private String _filter;
	protected FileFilter(String filter)
	{
	  this._filter = filter;
	}

	/** @see java.io.FilenameFilter */
	public boolean accept(File dir, String name)
	{
	  int index = _filter.indexOf('*');
	  String str1 = _filter.substring(0, index - 1);
	  String str2 = _filter.substring(index + 1);

	  return ((name.startsWith(str1)) &&
			  (name.endsWith(str2)) ? true :false );
	}
}

